@@ -2,7 +2,7 @@
  * Routines for QSIG packet dissection
  * 2007  Tomas Kukosa
  *
- * $Id: packet-qsig-template.c 30516 2009-10-11 16:24:29Z stig $
+ * $Id: packet-qsig-template.c 36108 2011-03-01 13:01:26Z kukosa $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -307,11 +307,17 @@ static gint ett_qsig = -1;
 static gint ett_qsig_ie = -1;
 static gint ett_qsig_unknown_extension = -1;
 #include "packet-qsig-ett.c"
+static gint ett_cnq_PSS1InformationElement = -1;
 
 /* Preferences */
 
+/* Subdissectors */
+static dissector_handle_t q931_ie_handle = NULL; 
+
 /* Global variables */
 static const char *extension_oid = NULL;
+static GHashTable *qsig_opcode2oid_hashtable = NULL;
+static GHashTable *qsig_oid2op_hashtable = NULL;
 
 /* Dissector tables */
 static dissector_table_t extension_dissector_table;
@@ -368,7 +374,7 @@ static int
 dissect_qsig_arg(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
   int offset;
   rose_ctx_t *rctx;
-  gint32 opcode, service;
+  gint32 opcode = 0, service;
   const qsig_op_t *op_ptr;
   const gchar *p;
   proto_item *ti, *ti_tmp;
@@ -379,10 +385,15 @@ dissect_qsig_arg(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
   DISSECTOR_ASSERT(rctx);
   if (rctx->d.pdu != 1)  /* invoke */
     return offset; 
-  if (rctx->d.code != 0)  /* local */
+  if (rctx->d.code == 0) {  /* local */
+    opcode = rctx->d.code_local;
+    op_ptr = get_op(opcode);
+  } else if (rctx->d.code == 1) {  /* global */
+    op_ptr = g_hash_table_lookup(qsig_oid2op_hashtable, rctx->d.code_global);
+    if (op_ptr) opcode = op_ptr->opcode;
+  } else {
     return offset; 
-  opcode = rctx->d.code_local;
-  op_ptr = get_op(opcode);
+  }
   if (!op_ptr)
     return offset; 
   service = get_service(opcode);
@@ -576,6 +587,32 @@ dissect_qsig_ie_cs5(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
   dissect_qsig_ie(tvb, pinfo, tree, 5);
 }
 
+/*--- qsig_init_tables ---------------------------------------------------------*/
+static void qsig_init_tables(void) {
+  guint i;
+  gint opcode, *key;
+  gchar *oid;
+
+  if (qsig_opcode2oid_hashtable)
+    g_hash_table_destroy(qsig_opcode2oid_hashtable);
+  qsig_opcode2oid_hashtable = g_hash_table_new_full(g_int_hash, g_int_equal, g_free, g_free);
+
+  if (qsig_oid2op_hashtable)
+    g_hash_table_destroy(qsig_oid2op_hashtable);
+  qsig_oid2op_hashtable = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, NULL);
+
+  /* fill-in global OIDs */
+  for (i=0; i<array_length(qsig_op_tab); i++) {
+    opcode = qsig_op_tab[i].opcode;
+    oid = g_strdup_printf("1.3.12.9.%d", opcode);
+    key = g_malloc(sizeof(gint));
+    *key = opcode;
+    g_hash_table_insert(qsig_opcode2oid_hashtable, key, oid);
+    g_hash_table_insert(qsig_oid2op_hashtable, g_strdup(oid), (gpointer)&qsig_op_tab[i]);
+  }
+
+}
+
 /*--- proto_register_qsig ---------------------------------------------------*/
 void proto_register_qsig(void) {
 
@@ -620,6 +657,7 @@ void proto_register_qsig(void) {
     &ett_qsig_ie,
     &ett_qsig_unknown_extension,
 #include "packet-qsig-ettarr.c"
+    &ett_cnq_PSS1InformationElement,
   };
 
   /* Register protocol and dissector */
@@ -631,12 +669,16 @@ void proto_register_qsig(void) {
 
   /* Register dissector tables */
   extension_dissector_table = register_dissector_table("qsig.ext", "QSIG Extension", FT_STRING, BASE_NONE);
+
+  qsig_init_tables();
 }
 
 
 /*--- proto_reg_handoff_qsig ------------------------------------------------*/
 void proto_reg_handoff_qsig(void) {
   int i;
+  gint key;
+  const gchar *oid;
   dissector_handle_t q931_handle; 
   dissector_handle_t qsig_arg_handle;
   dissector_handle_t qsig_res_handle;
@@ -644,25 +686,32 @@ void proto_reg_handoff_qsig(void) {
   dissector_handle_t qsig_ie_handle;
 
   q931_handle = find_dissector("q931");
+  q931_ie_handle = find_dissector("q931.ie");
 
   qsig_arg_handle = new_create_dissector_handle(dissect_qsig_arg, proto_qsig);
   qsig_res_handle = new_create_dissector_handle(dissect_qsig_res, proto_qsig);
   for (i=0; i<(int)array_length(qsig_op_tab); i++) {
-    dissector_add("q932.ros.local.arg", qsig_op_tab[i].opcode, qsig_arg_handle);
-    dissector_add("q932.ros.local.res", qsig_op_tab[i].opcode, qsig_res_handle);
+    dissector_add_uint("q932.ros.local.arg", qsig_op_tab[i].opcode, qsig_arg_handle);
+    dissector_add_uint("q932.ros.local.res", qsig_op_tab[i].opcode, qsig_res_handle);
+    key = qsig_op_tab[i].opcode;
+    oid = g_hash_table_lookup(qsig_opcode2oid_hashtable, &key);
+    if (oid) {
+      dissector_add_string("q932.ros.global.arg", oid, qsig_arg_handle);
+      dissector_add_string("q932.ros.global.res", oid, qsig_res_handle);
+    }
   }
   qsig_err_handle = new_create_dissector_handle(dissect_qsig_err, proto_qsig);
   for (i=0; i<(int)array_length(qsig_err_tab); i++) {
-    dissector_add("q932.ros.local.err", qsig_err_tab[i].errcode, qsig_err_handle);
+    dissector_add_uint("q932.ros.local.err", qsig_err_tab[i].errcode, qsig_err_handle);
   }
 
   qsig_ie_handle = create_dissector_handle(dissect_qsig_ie_cs4, proto_qsig);
   /* QSIG-TC - Transit counter */
-  dissector_add("q931.ie", CS4 | QSIG_IE_TRANSIT_COUNTER, qsig_ie_handle);
+  dissector_add_uint("q931.ie", CS4 | QSIG_IE_TRANSIT_COUNTER, qsig_ie_handle);
 
   qsig_ie_handle = create_dissector_handle(dissect_qsig_ie_cs5, proto_qsig);
   /* SSIG-BC - Party category */
-  dissector_add("q931.ie", CS5 | QSIG_IE_PARTY_CATEGORY, qsig_ie_handle);
+  dissector_add_uint("q931.ie", CS5 | QSIG_IE_PARTY_CATEGORY, qsig_ie_handle);
 
   /* RFC 3204, 3.2 QSIG Media Type */
   dissector_add_string("media_type", "application/qsig", q931_handle);
