@@ -1,7 +1,7 @@
 /* dftest.c
  * Shows display filter byte-code, for debugging dfilter routines.
  *
- * $Id: dftest.c 29824 2009-09-09 16:38:09Z krj $
+ * $Id: dftest.c 36446 2011-04-04 15:31:33Z wmeier $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -33,7 +33,7 @@
 #include <errno.h>
 
 #ifdef NEED_STRERROR_H
-#include "strerror.h"
+#include "wsutil/strerror.h"
 #endif
 
 #include <glib.h>
@@ -62,13 +62,12 @@ main(int argc, char **argv)
 	char		*gpf_path, *pf_path;
 	int		gpf_open_errno, gpf_read_errno;
 	int		pf_open_errno, pf_read_errno;
-	e_prefs		*prefs;
 	dfilter_t	*df;
 
 	/*
 	 * Get credential information for later use.
 	 */
-	get_credential_info();
+	init_process_policies();
 
 	/*
 	 * Attempt to get the pathname of the executable file.
@@ -80,6 +79,7 @@ main(int argc, char **argv)
 	}
 
 	timestamp_set_type(TS_RELATIVE);
+	timestamp_set_seconds_type(TS_SECONDS_DEFAULT);
 
 	/* Register all dissectors; we must do this before checking for the
 	   "-g" flag, as the "-g" flag dumps a list of fields registered
@@ -97,7 +97,7 @@ main(int argc, char **argv)
 	/* set the c-language locale to the native environment. */
 	setlocale(LC_ALL, "");
 
-	prefs = read_prefs(&gpf_open_errno, &gpf_read_errno, &gpf_path,
+	read_prefs(&gpf_open_errno, &gpf_read_errno, &gpf_path,
 		&pf_open_errno, &pf_read_errno, &pf_path);
 	if (gpf_path != NULL) {
 		if (gpf_open_errno != 0) {
