@@ -3,10 +3,10 @@
  *
  * Pigeon Point Systems <www.pigeonpoint.com>
  *
- * $Id: packet-i2c.c 26281 2008-09-26 22:07:45Z wmeier $
+ * $Id: packet-i2c.c 35224 2010-12-20 05:35:29Z guy $
  *
- * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@ethereal.com>
+ * Wireshark - Network traffic analyzer
+ * By Gerald Combs <gerald@wireshark.org>
  * Copyright 1998 Gerald Combs
  *
  * This program is free software; you can redistribute it and/or
@@ -206,7 +206,7 @@ dissect_i2c(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 	if (check_col(pinfo->cinfo, COL_PROTOCOL)) {
 		if (is_event)
-			col_add_fstr(pinfo->cinfo, COL_PROTOCOL, "I2C Event");
+			col_set_str(pinfo->cinfo, COL_PROTOCOL, "I2C Event");
 		else
 			col_add_fstr(pinfo->cinfo, COL_PROTOCOL, "I2C %s",
 					(flags & I2C_FLAG_RD) ? "Read" : "Write");
@@ -266,10 +266,10 @@ void
 proto_register_i2c(void)
 {
 	static hf_register_info hf[] = {
-		{ &hf_i2c_bus, { "Bus ID", "i2c.bus", FT_UINT8, BASE_DEC, NULL, 0, "", HFILL }},
-		{ &hf_i2c_addr, { "Target address", "i2c.addr", FT_UINT8, BASE_HEX, NULL, 0, "", HFILL }},
-		{ &hf_i2c_event, { "Event", "i2c.event", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
-		{ &hf_i2c_flags, { "Flags", "i2c.flags", FT_UINT32, BASE_HEX, NULL, 0, "", HFILL }},
+		{ &hf_i2c_bus, { "Bus ID", "i2c.bus", FT_UINT8, BASE_DEC, NULL, 0, NULL, HFILL }},
+		{ &hf_i2c_addr, { "Target address", "i2c.addr", FT_UINT8, BASE_HEX, NULL, 0, NULL, HFILL }},
+		{ &hf_i2c_event, { "Event", "i2c.event", FT_UINT32, BASE_HEX, NULL, 0, NULL, HFILL }},
+		{ &hf_i2c_flags, { "Flags", "i2c.flags", FT_UINT32, BASE_HEX, NULL, 0, NULL, HFILL }},
 	};
 	static gint *ett[] = {
 		&ett_i2c
@@ -298,5 +298,5 @@ proto_reg_handoff_i2c(void)
 	sub_handles[SUB_DATA] = find_dissector("data");
 	sub_handles[SUB_IPMB] = find_dissector("ipmi");
 	i2c_handle = create_dissector_handle(dissect_i2c, proto_i2c);
-	dissector_add("wtap_encap", WTAP_ENCAP_I2C, i2c_handle);
+	dissector_add_uint("wtap_encap", WTAP_ENCAP_I2C, i2c_handle);
 }
