@@ -3,7 +3,7 @@
  * Copyright (C) 2003, Digium
  * Mark Spencer <markster@digium.com>
  *
- * $Id: packet-iax2.h 27696 2009-03-11 06:43:46Z etxrab $
+ * $Id: packet-iax2.h 32942 2010-05-25 05:48:36Z etxrab $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -219,7 +219,7 @@ typedef enum {
   IAX2_MINI_VOICE_PACKET,
   IAX2_FULL_PACKET,
   IAX2_MINI_VIDEO_PACKET,
-  IAX2_META_PACKET
+  IAX2_TRUNK_PACKET
 } packet_type;
 
 /* Container for tapping relevant data */
