@@ -6,7 +6,7 @@
  * July 2005
  * Modified by: Brian Bogora <brian_bogora@mitel.com>
  *
- * $Id: packet-lldp.c 30662 2009-10-22 07:10:05Z jake $
+ * $Id: packet-lldp.c 37065 2011-05-11 20:35:57Z gerald $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -31,10 +31,6 @@
 # include "config.h"
 #endif
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
 #include <gmodule.h>
 #include <glib.h>
 
@@ -84,6 +80,7 @@ static int hf_mgn_obj_id = -1;
 static int hf_org_spc_oui = -1;
 static int hf_ieee_802_1_subtype = -1;
 static int hf_ieee_802_3_subtype = -1;
+static int hf_ieee_802_1qbg_subtype = -1;
 static int hf_media_tlv_subtype = -1;
 static int hf_profinet_tlv_subtype = -1;
 static int hf_profinet_class2_port_status = -1;
@@ -107,6 +104,7 @@ static int hf_profinet_orange_period_begin_valid = -1;
 static int hf_profinet_orange_period_begin_offset = -1;
 static int hf_profinet_green_period_begin_valid = -1;
 static int hf_profinet_green_period_begin_offset = -1;
+static int hf_cisco_subtype = -1;
 static int hf_unknown_subtype = -1;
 
 /* Initialize the subtree pointers */
@@ -128,8 +126,10 @@ static gint ett_802_3_flags = -1;
 static gint ett_802_3_autoneg_advertised = -1;
 static gint ett_802_3_power = -1;
 static gint ett_802_3_aggregation = -1;
+static gint ett_802_1qbg_capabilities_flags = -1;
 static gint ett_media_capabilities = -1;
 static gint ett_profinet_period = -1;
+static gint ett_cisco_fourwire_tlv = -1;
 
 static const value_string tlv_types[] = {
 	{ END_OF_LLDPDU_TLV_TYPE, 			"End of LLDPDU"},
@@ -181,6 +181,8 @@ static const value_string tlv_oui_subtype_vals[] = {
 	{ OUI_IEEE_802_3,     	"IEEE 802.3" },
 	{ OUI_MEDIA_ENDPOINT,	"TIA" },
 	{ OUI_PROFINET,         "PROFINET" },
+	{ OUI_CISCO_2,          "Cisco" },
+	{ OUI_IEEE_802_1QBG,	"IEEE 802.1Qbg" },
 	{ 0, NULL }
 };
 
@@ -218,6 +220,14 @@ static const value_string media_subtypes[] = {
 	{ 0, NULL }
 };
 
+/* IEEE 802.1Qbg Subtypes */
+static const value_string ieee_802_1qbg_subtypes[] = {
+	{ 0x00, "EVB" },
+	{ 0x01, "CDCP" },
+	{ 0, NULL }
+};
+
+
 /* Media Class Values */
 static const value_string media_class_values[] = {
 	{ 0,	"Type Not Defined" },
@@ -253,6 +263,12 @@ static const value_string profinet_subtypes[] = {
 	{ 0, NULL }
 };
 
+/* Cisco Subtypes */
+static const value_string cisco_subtypes[] = {
+	{ 1, "Four-wire Power-via-MDI" },
+	{ 0, NULL }
+};
+
 /* 802.3 Power Type */
 static const value_string power_type_802_3[] = {
 	{ 0,	"Type 2" },
@@ -456,10 +472,17 @@ static const value_string operational_mau_type_values[] = {
 #define INV_AUTONEG_1000BASE_T		0x4000 /* b1000baseT(14),   -- 1000BASE-T half duplex mode */
 #define INV_AUTONEG_1000BASE_TFD	0x8000 /* b1000baseTFD(15)  -- 1000BASE-T full duplex mode */
 
+#define EVB_CAPA_STD		0x8000
+#define EVB_CAPA_RR		0x4000
+
+#define EVB_CAPA_RTE		0x0004
+#define EVB_CAPA_ECP		0x0002
+#define EVB_CAPA_VDP		0x0001
+
 #define MAX_MAC_LEN	6
 
 
-const value_string profinet_port2_status_vals[] = {
+static const value_string profinet_port2_status_vals[] = {
 	{ 0,	"OFF" },
 	{ 1,	"SYNCDATA_LOADED" },
 	{ 2,	"RTCLASS2_UP" },
@@ -468,7 +491,7 @@ const value_string profinet_port2_status_vals[] = {
 	{ 0,	NULL }
 };
 
-const value_string profinet_port3_status_vals[] = {
+static const value_string profinet_port3_status_vals[] = {
 	{ 0,	"OFF" },
 	{ 1,	"IRDATA_LOADED" },
 	{ 2,	"RTCLASS3_UP" },
@@ -478,7 +501,7 @@ const value_string profinet_port3_status_vals[] = {
 	{ 0,	NULL }
 };
 
-const value_string profinet_mrrt_port_status_vals[] = {
+static const value_string profinet_mrrt_port_status_vals[] = {
 	{ 0,	"OFF" },
 	{ 1,	"MRRT_CONFIGURED" },
 	{ 2,	"MRRT_UP" },
@@ -553,8 +576,6 @@ dissect_lldp_chassis_id(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gui
 	guint32 tempLen = 0;
 	const char *strPtr=NULL;
 	guint8 incorrectLen = 0;	/* incorrect length if 1 */
-
-	const guint8 *mac_addr = NULL;
 	guint32 ip_addr = 0;
 	struct e_in6_addr ip6_addr;
 	guint8 addr_family = 0;
@@ -607,8 +628,7 @@ dissect_lldp_chassis_id(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gui
 			break;
 		}
 
-		mac_addr=tvb_get_ptr(tvb, (offset+3), 6);
-		strPtr = ether_to_str(mac_addr);
+		strPtr = tvb_ether_to_str(tvb, offset+3);
 
 		break;
 	}
@@ -717,7 +737,7 @@ dissect_lldp_chassis_id(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gui
 		switch (tempType)
 		{
 		case 4:	/* MAC address */
-			proto_tree_add_ether(chassis_tree, hf_chassis_id_mac, tvb, (offset+3), 6, mac_addr);
+			proto_tree_add_item(chassis_tree, hf_chassis_id_mac, tvb, (offset+3), 6, ENC_NA);
 			proto_item_append_text(tf, ", Id: %s", strPtr);
 			break;
 		case 5: /* Network address */
@@ -758,7 +778,6 @@ dissect_lldp_port_id(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, guint3
 	guint16 tempShort;
 	guint32 tempLen = 0;
 	const char *strPtr;
-	const guint8 *mac_addr = NULL;
 	guint32 ip_addr = 0;
 	struct e_in6_addr ip6_addr;
 	guint8 addr_family = 0;
@@ -784,8 +803,7 @@ dissect_lldp_port_id(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, guint3
 		if (tempLen != 7)
 			return -1; 	/* Invalid port id */
 
-		mac_addr=tvb_get_ptr(tvb, (offset+3), 6);
-		strPtr = ether_to_str(mac_addr);
+		strPtr = tvb_ether_to_str(tvb, offset+3);
 
 		break;
 	}
@@ -846,7 +864,7 @@ dissect_lldp_port_id(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, guint3
 		switch (tempType)
 		{
 		case 3:	/* MAC address */
-			proto_tree_add_ether(port_tree, hf_port_id_mac, tvb, (offset+3), 6, mac_addr);
+			proto_tree_add_item(port_tree, hf_port_id_mac, tvb, (offset+3), 6, ENC_NA);
 			break;
 		case 4: /* Network address */
 			/* Network address
@@ -1339,9 +1357,124 @@ dissect_ieee_802_1_tlv(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree,
 	return;
 }
 
+/* Dissect IEEE 802.1Qbg TLVs */
+static void
+dissect_ieee_802_1qbg_tlv(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree, guint32 offset)
+{
+	guint8 subType;
+	guint8 tempByte;
+	guint16 tempShort;
+	guint32 tempOffset = offset;
+
+	proto_tree *evb_capabilities_subtree = NULL;
+
+	proto_item *tf = NULL;
+
+	subType = tvb_get_guint8(tvb, tempOffset);
+
+	if (tree)
+		proto_tree_add_item(tree, hf_ieee_802_1qbg_subtype, tvb, tempOffset, 1, FALSE);
+
+	tempOffset++;
+
+	switch (subType) {
+		case 0x00:
+			/* Get EVB capabilities */
+			tempShort = tvb_get_ntohs(tvb, tempOffset);
+			if (tree) {
+				tf = proto_tree_add_text(tree, tvb, tempOffset, 2, "supported capabilities: 0x%04X", tempShort);
+				evb_capabilities_subtree = proto_item_add_subtree(tf, ett_802_1qbg_capabilities_flags);
+
+				if (tempShort & EVB_CAPA_STD)
+					proto_tree_add_text(evb_capabilities_subtree, tvb, (offset+2), 2, "%s",
+								decode_boolean_bitfield(tempShort, EVB_CAPA_STD,
+								16, "standard bridging (STD)", ""));
+
+				if (tempShort & EVB_CAPA_RR)
+					proto_tree_add_text(evb_capabilities_subtree, tvb, (offset+2), 2, "%s",
+								decode_boolean_bitfield(tempShort, EVB_CAPA_RR,
+								16, "reflective relay (RR)", ""));
+
+				if (tempShort & EVB_CAPA_RTE)
+					proto_tree_add_text(evb_capabilities_subtree, tvb, (offset+2), 2, "%s",
+								decode_boolean_bitfield(tempShort, EVB_CAPA_RTE,
+								16, "retransmission timer exponent (RTE)", ""));
+
+				if (tempShort & EVB_CAPA_ECP)
+					proto_tree_add_text(evb_capabilities_subtree, tvb, (offset+2), 2, "%s",
+								decode_boolean_bitfield(tempShort, EVB_CAPA_ECP,
+								16, "edge control protocol (ECP)", ""));
+
+				if (tempShort & EVB_CAPA_VDP)
+					proto_tree_add_text(evb_capabilities_subtree, tvb, (offset+2), 2, "%s",
+								decode_boolean_bitfield(tempShort, EVB_CAPA_VDP,
+								16, "VSI discovery protocol (VDP)", ""));
+			}
+
+			tempOffset += 2;
+
+			tempShort = tvb_get_ntohs(tvb, tempOffset);
+			if (tree) {
+				tf = proto_tree_add_text(tree, tvb, tempOffset, 2, "configured capabilities: 0x%04X", tempShort);
+				evb_capabilities_subtree = proto_item_add_subtree(tf, ett_802_1qbg_capabilities_flags);
+
+				if (tempShort & EVB_CAPA_STD)
+					proto_tree_add_text(evb_capabilities_subtree, tvb, (offset+2), 2, "%s",
+								decode_boolean_bitfield(tempShort, EVB_CAPA_STD,
+								16, "standard bridging (STD)", ""));
+
+				if (tempShort & EVB_CAPA_RR)
+					proto_tree_add_text(evb_capabilities_subtree, tvb, (offset+2), 2, "%s",
+								decode_boolean_bitfield(tempShort, EVB_CAPA_RR,
+								16, "reflective relay (RR)", ""));
+
+				if (tempShort & EVB_CAPA_RTE)
+					proto_tree_add_text(evb_capabilities_subtree, tvb, (offset+2), 2, "%s",
+								decode_boolean_bitfield(tempShort, EVB_CAPA_RTE,
+								16, "retransmission timer exponent (RTE)", ""));
+
+				if (tempShort & EVB_CAPA_ECP)
+					proto_tree_add_text(evb_capabilities_subtree, tvb, (offset+2), 2, "%s",
+								decode_boolean_bitfield(tempShort, EVB_CAPA_ECP,
+								16, "edge control protocol (ECP)", ""));
+
+				if (tempShort & EVB_CAPA_VDP)
+					proto_tree_add_text(evb_capabilities_subtree, tvb, (offset+2), 2, "%s",
+								decode_boolean_bitfield(tempShort, EVB_CAPA_VDP,
+								16, "VSI discovery protocol (VDP)", ""));
+			}
+
+			tempOffset += 2;
+
+			tempShort = tvb_get_ntohs(tvb, tempOffset);
+			if (tree) {
+				proto_tree_add_text(tree, tvb, tempOffset, 2, "supported no. of VSIs: %04u", tempShort);
+			}
+
+			tempOffset += 2;
+
+			tempShort = tvb_get_ntohs(tvb, tempOffset);
+			if (tree) {
+				proto_tree_add_text(tree, tvb, tempOffset, 2, "configured no. of VSIs: %04u", tempShort);
+			}
+
+			tempOffset += 2;
+
+			tempByte= tvb_get_guint8(tvb, tempOffset);
+			if (tree) {
+				proto_tree_add_text(tree, tvb, tempOffset, 1, "retransmission timer exponent: %02u", tempByte);
+			}
+
+			break;
+	}
+
+	return;
+}
+
+
 /* Dissect IEEE 802.3 TLVs */
 static void
-dissect_ieee_802_3_tlv(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree, guint32 offset)
+dissect_ieee_802_3_tlv(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree, guint32 offset, guint16 tlvLen)
 {
 	guint8 subType;
 	guint8 tempByte;
@@ -1619,6 +1752,9 @@ dissect_ieee_802_3_tlv(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree,
 
 		tempOffset++;
 
+		if (tlvLen == 4)
+			break;
+
 		/* Get first byte */
 		tempByte = tvb_get_guint8(tvb, tempOffset);
 
@@ -1669,14 +1805,14 @@ dissect_ieee_802_3_tlv(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree,
 
 		tempOffset++;
 
-		/* Power Value: 1 to 255 expected  */
+		/* Power Value: 1 to 510 expected  */
 		tempShort = tvb_get_ntohs(tvb, tempOffset);
 		if (tree)
 			proto_tree_add_text(tree, tvb, tempOffset, 2, "PD Requested Power Value: %u.%u Watt", tempShort/10, tempShort%10);
 
 		tempOffset+=2;
 
-		/* Power Value: 1 to 255 expected */
+		/* Power Value: 1 to 510 expected */
 		tempShort = tvb_get_ntohs(tvb, tempOffset);
 		if (tree)
 			proto_tree_add_text(tree, tvb, tempOffset, 2, "PSE Allocated Power Value: %u.%u Watt", tempShort/10, tempShort%10);
@@ -2289,7 +2425,6 @@ dissect_profinet_tlv(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree, gu
 	guint32 port_tx_delay_local;
 	guint32 port_tx_delay_remote;
 	guint32 cable_delay_local;
-	const guint8 *mac_addr = NULL;
 	e_guid_t * uuid;
 	guint16 mrrt_PortStatus;
 
@@ -2372,16 +2507,14 @@ dissect_profinet_tlv(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree, gu
 	}
     case 5:     /* LLDP_PNIO_CHASSIS_MAC */
     {
-	mac_addr=tvb_get_ptr(tvb, offset, 6);
-	proto_tree_add_ether(tree, hf_profinet_cm_mac, tvb, offset, 6, mac_addr);
+	proto_tree_add_item(tree, hf_profinet_cm_mac, tvb, offset, 6, ENC_NA);
         offset += 6;
         break;
     }
     case 6:     /* LLDP_PNIO_PTCPSTATUS */
     {
 	    /* MasterSourceAddress */
-	    mac_addr=tvb_get_ptr(tvb, offset, 6);
-	    proto_tree_add_ether(tree, hf_profinet_master_source_address, tvb, offset, 6, mac_addr);
+	    proto_tree_add_item(tree, hf_profinet_master_source_address, tvb, offset, 6, ENC_NA);
 	    offset += 6;
 	    /* SubdomainUUID */
 	    tvb_get_ntohguid (tvb, offset, (e_guid_t *) &uuid);
@@ -2410,6 +2543,58 @@ dissect_profinet_tlv(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree, gu
 	}
 }
 
+/* Dissect Cisco OUI TLVs */
+static void
+dissect_cisco_tlv(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree, guint32 offset)
+{
+	guint8 subType;
+	guint8 tempByte;
+	guint32 tempOffset = offset;
+
+	proto_tree *fourwire_data = NULL;
+	proto_item *tf = NULL;
+
+	/* Get subtype */
+	subType = tvb_get_guint8(tvb, tempOffset);
+
+	proto_tree_add_item(tree, hf_cisco_subtype, tvb, tempOffset, 1, FALSE);
+
+	tempOffset++;
+
+	switch (subType)
+	{
+	case 0x01: /* Four-Wire Power-via-MDI TLV */
+	  tempByte = tvb_get_guint8(tvb, tempOffset);
+	  if (tree) {
+	    tf = proto_tree_add_text(tree, tvb, tempOffset, 1, "Four-Wire Power-via-MDI: 0x%02x", tempByte);
+	    fourwire_data = proto_item_add_subtree(tf, ett_cisco_fourwire_tlv);
+
+	    proto_tree_add_text(fourwire_data, tvb, tempOffset, 1, "%s",
+				decode_boolean_bitfield(tempByte, 0x01, 8,
+							"PSE Four-Wire PoE Supported",
+							"PSE Four-Wire PoE Not Supported"));
+
+	    proto_tree_add_text(fourwire_data, tvb, tempOffset, 1, "%s",
+				decode_boolean_bitfield(tempByte, 0x02, 8,
+							"PD  Spare Pair Architecture Shared",
+							"PD  Spare Pair Architecture Independent"));
+
+	    proto_tree_add_text(fourwire_data, tvb, tempOffset, 1, "%s",
+				decode_boolean_bitfield(tempByte, 0x04, 8,
+							"PD  Request Spare Pair PoE On",
+							"PD  Request Spare Pair PoE Off"));
+
+	    proto_tree_add_text(fourwire_data, tvb, tempOffset, 1, "%s",
+				decode_boolean_bitfield(tempByte, 0x08, 8,
+							"PSE Spare Pair PoE On",
+							"PSE Spare Pair PoE Off"));
+	  }
+	  break;
+	default:
+	  proto_tree_add_item(tree, hf_unknown_subtype, tvb, offset, 1, FALSE);
+	  break;
+	}
+}
 
 /* Dissect Organizational Specific TLV */
 static gint32
@@ -2450,6 +2635,12 @@ dissect_organizational_specific_tlv(tvbuff_t *tvb, packet_info *pinfo, proto_tre
 	case OUI_PROFINET:
 		subTypeStr = val_to_str(subType, profinet_subtypes, "Reserved (0x%x)");
 		break;
+	case OUI_CISCO_2:
+		subTypeStr = val_to_str(subType, cisco_subtypes, "Unknown subtype (0x%x)");
+		break;
+	case OUI_IEEE_802_1QBG:
+		subTypeStr = val_to_str(subType, ieee_802_1qbg_subtypes, "Unknown subtype 0x%x");
+		break;
 	default:
 		subTypeStr = "Unknown";
 		break;
@@ -2485,7 +2676,7 @@ dissect_organizational_specific_tlv(tvbuff_t *tvb, packet_info *pinfo, proto_tre
 		dissect_ieee_802_1_tlv(tvb, pinfo, org_tlv_tree, (offset+5));
 		break;
 	case OUI_IEEE_802_3:
-		dissect_ieee_802_3_tlv(tvb, pinfo, org_tlv_tree, (offset+5));
+		dissect_ieee_802_3_tlv(tvb, pinfo, org_tlv_tree, (offset+5), (guint16) (tempLen-3));
 		break;
 	case OUI_MEDIA_ENDPOINT:
 		dissect_media_tlv(tvb, pinfo, org_tlv_tree, (offset+5), (guint16) (tempLen-3));
@@ -2493,6 +2684,12 @@ dissect_organizational_specific_tlv(tvbuff_t *tvb, packet_info *pinfo, proto_tre
 	case OUI_PROFINET:
 		dissect_profinet_tlv(tvb, pinfo, org_tlv_tree, (offset+5), (guint16) (tempLen-3));
 		break;
+	case OUI_CISCO_2:
+		dissect_cisco_tlv(tvb, pinfo, org_tlv_tree, (offset+5));
+		break;
+	case OUI_IEEE_802_1QBG:
+		dissect_ieee_802_1qbg_tlv(tvb, pinfo, org_tlv_tree, (offset+5));
+		break;
 	default:
 		proto_tree_add_item(org_tlv_tree, hf_unknown_subtype, tvb, (offset+5), (guint16) (tempLen-3), FALSE);
 	}
@@ -2716,16 +2913,20 @@ proto_register_lldp(void)
 			NULL, 0, NULL, HFILL }
 		},
 		{ &hf_org_spc_oui,
-			{ "Organization Unique Code",	"lldp.orgtlv.oui", FT_UINT24, BASE_HEX,
-	   		VALS(tlv_oui_subtype_vals), 0x0, NULL, HFILL }
+			{ "Organization Unique Code", "lldp.orgtlv.oui", FT_UINT24, BASE_HEX,
+			VALS(tlv_oui_subtype_vals), 0x0, NULL, HFILL }
 		},
 		{ &hf_ieee_802_1_subtype,
-			{ "IEEE 802.1 Subtype",	"lldp.ieee.802_1.subtype", FT_UINT8, BASE_HEX,
-	   		VALS(ieee_802_1_subtypes), 0x0, NULL, HFILL }
+			{ "IEEE 802.1 Subtype", "lldp.ieee.802_1.subtype", FT_UINT8, BASE_HEX,
+			VALS(ieee_802_1_subtypes), 0x0, NULL, HFILL }
 		},
 		{ &hf_ieee_802_3_subtype,
-			{ "IEEE 802.3 Subtype",	"lldp.ieee.802_3.subtype", FT_UINT8, BASE_HEX,
-	   		VALS(ieee_802_3_subtypes), 0x0, NULL, HFILL }
+			{ "IEEE 802.3 Subtype", "lldp.ieee.802_3.subtype", FT_UINT8, BASE_HEX,
+			VALS(ieee_802_3_subtypes), 0x0, NULL, HFILL }
+		},
+		{ &hf_ieee_802_1qbg_subtype,
+			{ "IEEE 802.1Qbg Subtype", "lldp.ieee.802_1qbg.subtype", FT_UINT8, BASE_HEX,
+			VALS(ieee_802_1qbg_subtypes), 0x0, NULL, HFILL }
 		},
 		{ &hf_media_tlv_subtype,
 			{ "Media Subtype",	"lldp.media.subtype", FT_UINT8, BASE_HEX,
@@ -2819,6 +3020,10 @@ proto_register_lldp(void)
 			{ "GreenPeriodBegin.Offset",	"lldp.profinet.green_period_begin_offset", FT_UINT32, BASE_DEC,
 	   		NULL, 0x7FFFFFFF, "Unrestricted period, offset to cycle begin in nanoseconds", HFILL }
 		},
+		{ &hf_cisco_subtype,
+			{ "Cisco Subtype",	"lldp.cisco.subtype", FT_UINT8, BASE_HEX,
+	   		VALS(cisco_subtypes), 0x0, NULL, HFILL }
+		},
 		{ &hf_unknown_subtype,
 			{ "Unknown Subtype Content","lldp.unknown_subtype", FT_BYTES, BASE_NONE,
 	   		NULL, 0x0, NULL, HFILL }
@@ -2845,8 +3050,10 @@ proto_register_lldp(void)
 		&ett_802_3_autoneg_advertised,
 		&ett_802_3_power,
 		&ett_802_3_aggregation,
+		&ett_802_1qbg_capabilities_flags,
 		&ett_media_capabilities,
-		&ett_profinet_period
+		&ett_profinet_period,
+		&ett_cisco_fourwire_tlv
 	};
 
 	/* Register the protocol name and description */
@@ -2863,5 +3070,5 @@ proto_reg_handoff_lldp(void)
 	dissector_handle_t lldp_handle;
 
 	lldp_handle = create_dissector_handle(dissect_lldp,proto_lldp);
-	dissector_add("ethertype", ETHERTYPE_LLDP, lldp_handle);
+	dissector_add_uint("ethertype", ETHERTYPE_LLDP, lldp_handle);
 }
