@@ -2,7 +2,7 @@
  * Routines for Message Session Relay Protocol(MSRP) dissection
  * Copyright 2005, Anders Broman <anders.broman[at]ericsson.com>
  *
- * $Id: packet-msrp.c 27254 2009-01-17 17:33:50Z wmeier $
+ * $Id: packet-msrp.c 36996 2011-05-05 06:55:12Z darkjames $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -30,7 +30,6 @@
 # include "config.h"
 #endif
 
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
@@ -43,7 +42,7 @@
 #include <epan/strutil.h>
 #include <epan/packet.h>
 #include <epan/emem.h>
-#include "prefs.h"
+#include <epan/prefs.h>
 
 #include "packet-msrp.h"
 
@@ -204,12 +203,7 @@ void msrp_add_address( packet_info *pinfo,
 	 */
 	if (!p_conv_data) {
 		/* Create conversation data */
-		p_conv_data = se_alloc(sizeof(struct _msrp_conversation_info));
-		if (!p_conv_data)
-		{
-			return;
-		}
-		memset(p_conv_data, 0, sizeof(struct _msrp_conversation_info));
+		p_conv_data = se_alloc0(sizeof(struct _msrp_conversation_info));
 		conversation_add_proto_data(p_conv, proto_msrp, p_conv_data);
 	}
 
@@ -249,12 +243,7 @@ void show_setup_info(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			if (p_conv_data)
 			{
 				/* Save this conversation info into packet info */
-				p_conv_packet_data = se_alloc(sizeof(struct _msrp_conversation_info));
-				if (!p_conv_packet_data)
-				{
-					return;
-				}
-				memcpy(p_conv_packet_data, p_conv_data,
+				p_conv_packet_data = se_memdup(p_conv_data,
 				       sizeof(struct _msrp_conversation_info));
 
 				p_add_proto_data(pinfo->fd, proto_msrp, p_conv_packet_data);
@@ -367,6 +356,10 @@ check_msrp_header(tvbuff_t *tvb)
 	 * "tvb_get_ptr()" calls below won't throw exceptions.	 *
 	 */
 	offset = 0;
+	if(tvb_length(tvb) < 4 ||  tvb_get_ntohl(tvb, 0) != 0x4d535250 /* MSRP */){
+		return FALSE;
+	}
+
 	linelen = tvb_find_line_end(tvb, 0, -1, &next_offset, FALSE);
 	/* Find the first SP */
 	space_offset = tvb_find_guint8(tvb, 0, -1, ' ');
@@ -395,7 +388,7 @@ check_msrp_header(tvbuff_t *tvb)
 	/*
 	 * Is the first token "MSRP"?
 	 */
-	if (token_1_len == MSRP_HDR_LEN && tvb_strneql(tvb, 0, MSRP_HDR, MSRP_HDR_LEN) == 0){
+	if (token_1_len == MSRP_HDR_LEN) { /*  && tvb_strneql(tvb, 0, MSRP_HDR, MSRP_HDR_LEN) == 0){ */
 		/* This check can be made more strict but accept we do have MSRP for now */
 		return TRUE;
 
@@ -483,7 +476,7 @@ dissect_msrp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	gint hf_index;
 	gint value_offset;
 	guchar c;
-	size_t value_len;
+	gint value_len;
 	char *value;
 	gboolean have_body = FALSE;
 	gboolean found_match = FALSE;
@@ -544,31 +537,26 @@ dissect_msrp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	}
 
 	/* Make entries in Protocol column and Info column on summary display */
-	if (check_col(pinfo->cinfo, COL_PROTOCOL))
-		col_set_str(pinfo->cinfo, COL_PROTOCOL, "MSRP");
+	col_set_str(pinfo->cinfo, COL_PROTOCOL, "MSRP");
 	if (is_msrp_response){
-		if (check_col(pinfo->cinfo, COL_INFO)) {
-			col_add_fstr(pinfo->cinfo, COL_INFO, "Response: %s ",
-					tvb_format_text(tvb, token_3_start, token_3_len));
+		col_add_fstr(pinfo->cinfo, COL_INFO, "Response: %s ",
+				tvb_format_text(tvb, token_3_start, token_3_len));
 
-			if (token_4_len )
-				col_append_fstr(pinfo->cinfo, COL_INFO, "%s ",
-					tvb_format_text(tvb, token_4_start, token_4_len));
+		if (token_4_len )
+			col_append_fstr(pinfo->cinfo, COL_INFO, "%s ",
+				tvb_format_text(tvb, token_4_start, token_4_len));
 
-			col_append_fstr(pinfo->cinfo, COL_INFO, "Transaction ID: %s",
-					tvb_format_text(tvb, token_2_start, token_2_len));
-		}
+		col_append_fstr(pinfo->cinfo, COL_INFO, "Transaction ID: %s",
+				tvb_format_text(tvb, token_2_start, token_2_len));
 	}else{
-		if (check_col(pinfo->cinfo, COL_INFO)) {
-			proto_tree_add_text(tree, tvb, token_3_start, token_3_len,
-					"Col %s L=%u", tvb_format_text(tvb, token_3_start, token_3_len),token_3_len);
+		proto_tree_add_text(tree, tvb, token_3_start, token_3_len,
+				"Col %s L=%u", tvb_format_text(tvb, token_3_start, token_3_len),token_3_len);
 
-			col_add_fstr(pinfo->cinfo, COL_INFO, "Request: %s ",
-					tvb_format_text(tvb, token_3_start, token_3_len));
+		col_add_fstr(pinfo->cinfo, COL_INFO, "Request: %s ",
+				tvb_format_text(tvb, token_3_start, token_3_len));
 
-			col_append_fstr(pinfo->cinfo, COL_INFO, "Transaction ID: %s",
-					tvb_format_text(tvb, token_2_start, token_2_len));
-		}
+		col_append_fstr(pinfo->cinfo, COL_INFO, "Transaction ID: %s",
+				tvb_format_text(tvb, token_2_start, token_2_len));
 	}
 
 	/* Find the end line to be able to process the headers
@@ -708,7 +696,7 @@ dissect_msrp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			proto_item_set_end(msrp_headers_item, tvb, next_offset);
 
 			/* Create new tree & tvb for data */
-			next_tvb = tvb_new_subset(tvb, next_offset, -1, -1);
+			next_tvb = tvb_new_subset_remaining(tvb, next_offset);
 			ti = proto_tree_add_item(msrp_tree, hf_msrp_data, tvb,
 			                         next_offset, -1, FALSE);
 			msrp_data_tree = proto_item_add_subtree(ti, ett_msrp_data);
@@ -797,122 +785,122 @@ proto_register_msrp(void)
 		{ &hf_msrp_request_line,
 			{ "Request Line", 		"msrp.request.line",
 			FT_STRING, BASE_NONE,NULL,0x0,
-			"Request Line", HFILL }
+			NULL, HFILL }
 		},
 		{ &hf_msrp_response_line,
 			{ "Response Line", 		"msrp.response.line",
 			FT_STRING, BASE_NONE,NULL,0x0,
-			"Response Line", HFILL }
+			NULL, HFILL }
 		},
 		{ &hf_msrp_transactionID,
 			{ "Transaction Id", 		"msrp.transaction.id",
 			FT_STRING, BASE_NONE,NULL,0x0,
-			"Transaction Id", HFILL }
+			NULL, HFILL }
 		},
 		{ &hf_msrp_method,
 			{ "Method", 				"msrp.method",
 			FT_STRING, BASE_NONE,NULL,0x0,
-			"Method", HFILL }
+			NULL, HFILL }
 		},
 		{ &hf_msrp_status_code,
 			{ "Status code", 		"msrp.status.code",
 			FT_UINT16, BASE_DEC,NULL,0x0,
-			"Status code", HFILL }
+			NULL, HFILL }
 		},
 		{ &hf_msrp_msg_hdr,
 			{ "Message Header", 		"msrp.msg.hdr",
-			FT_NONE, 0,NULL,0x0,
-			"Message Header", HFILL }
+			FT_NONE, BASE_NONE,NULL,0x0,
+			NULL, HFILL }
 		},
 		{ &hf_msrp_end_line,
 			{ "End Line", 		"msrp.end.line",
 			FT_STRING, BASE_NONE,NULL,0x0,
-			"End Line", HFILL }
+			NULL, HFILL }
 		},
 		{ &hf_msrp_cnt_flg,
 			{ "Continuation-flag", 		"msrp.cnt.flg",
 			FT_STRING, BASE_NONE,NULL,0x0,
-			"Continuation-flag", HFILL }
+			NULL, HFILL }
 		},
 		{ &hf_header_array[MSRP_FROM_PATH],
 			{ "From Path", 		"msrp.from.path",
 			FT_STRING, BASE_NONE,NULL,0x0,
-			"From Path", HFILL }
+			NULL, HFILL }
 		},
 		{ &hf_header_array[MSRP_TO_PATH],
 			{ "To Path", 		"msrp.to.path",
 			FT_STRING, BASE_NONE,NULL,0x0,
-			"To Path", HFILL }
+			NULL, HFILL }
 		},
 		{ &hf_header_array[MSRP_MESSAGE_ID],
 			{ "Message ID", 		"msrp.messageid",
 			FT_STRING, BASE_NONE,NULL,0x0,
-			"Message ID", HFILL }
+			NULL, HFILL }
 		},
 		{ &hf_header_array[MSRP_SUCCESS_REPORT],
 			{ "Success Report", 		"msrp.success.report",
 			FT_STRING, BASE_NONE,NULL,0x0,
-			"Success Report", HFILL }
+			NULL, HFILL }
 		},
 		{ &hf_header_array[MSRP_FAILURE_REPORT],
 			{ "Failure Report", 		"msrp.failure.report",
 			FT_STRING, BASE_NONE,NULL,0x0,
-			"Failure Report", HFILL }
+			NULL, HFILL }
 		},
 		{ &hf_header_array[MSRP_BYTE_RANGE],
 			{ "Byte Range", 		"msrp.byte.range",
 			FT_STRING, BASE_NONE,NULL,0x0,
-			"Byte Range", HFILL }
+			NULL, HFILL }
 		},
 		{ &hf_header_array[MSRP_STATUS],
 			{ "Status", 		"msrp.status",
 			FT_STRING, BASE_NONE,NULL,0x0,
-			"Status", HFILL }
+			NULL, HFILL }
 		},
 		{ &hf_header_array[MSRP_CONTENT_TYPE],
 			{ "Content-Type", 		"msrp.content.type",
 			FT_STRING, BASE_NONE,NULL,0x0,
-			"Content-Type", HFILL }
+			NULL, HFILL }
 		},
 		{ &hf_header_array[MSRP_CONTENT_ID],
 			{ "Content-ID", 		"msrp.content.id",
 			FT_STRING, BASE_NONE,NULL,0x0,
-			"Content-ID", HFILL }
+			NULL, HFILL }
 		},
 		{ &hf_header_array[MSRP_CONTENT_DISCRIPTION],
 			{ "Content-Description", 		"msrp.content.description",
 			FT_STRING, BASE_NONE,NULL,0x0,
-			"Content-Description", HFILL }
+			NULL, HFILL }
 		},
 		{ &hf_header_array[MSRP_CONTENT_DISPOSITION],
 			{ "Content-Disposition", 		"msrp.content.disposition",
 			FT_STRING, BASE_NONE,NULL,0x0,
-			"Content-Disposition", HFILL }
+			NULL, HFILL }
 		},
 		{ &hf_header_array[MSRP_USE_PATH],
 			{ "Use-Path", 		"msrp.use.path",
 			FT_STRING, BASE_NONE,NULL,0x0,
-			"Use-Path", HFILL }
+			NULL, HFILL }
 		},
 		{ &hf_header_array[MSRP_WWW_AUTHENTICATE],
 			{ "WWW-Authenticate", 		"msrp.www.authenticate",
 			FT_STRING, BASE_NONE,NULL,0x0,
-			"WWW-Authenticate", HFILL }
+			NULL, HFILL }
 		},
 		{ &hf_header_array[MSRP_AUTHORIZATION],
 			{ "Authorization", 		"msrp.authorization",
 			FT_STRING, BASE_NONE,NULL,0x0,
-			"Authorization", HFILL }
+			NULL, HFILL }
 		},
 		{ &hf_header_array[MSRP_AUTHENTICATION_INFO],
 			{ "Authentication-Info", 		"msrp.authentication.info",
 			FT_STRING, BASE_NONE,NULL,0x0,
-			"Authentication-Info", HFILL }
+			NULL, HFILL }
 		},
 		{ &hf_msrp_data,
 			{ "Data", "msrp.data",
 			FT_STRING, BASE_NONE, NULL, 0x0,
-			"Data", HFILL}
+			NULL, HFILL}
         },
 		{ &hf_msrp_setup,
 			{ "Stream setup", "msrp.setup",
