@@ -9,7 +9,7 @@
  * Updated to Olsr.org and NRLOLSR packages by Henning Rogge <rogge@fgan.de>
  * http://www.ietf.org/rfc/rfc3626.txt
  *
- * $Id: packet-olsr.c 29344 2009-08-09 07:36:13Z krj $
+ * $Id: packet-olsr.c 35896 2011-02-10 16:31:00Z morriss $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -34,7 +34,6 @@
 # include "config.h"
 #endif
 
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -161,19 +160,19 @@ static const value_string nameservice_type_vals[] = {
 static gboolean global_olsr_olsrorg = TRUE;
 static gboolean global_olsr_nrlolsr = TRUE;
 
-static double getOlsrTime(guint8 time) {
+static double getOlsrTime(guint8 timeval) {
   int high_bits, low_bits;
 
-  high_bits = ((time & 0xF0) >> 4);
-  low_bits = (time & 0x0F);
+  high_bits = ((timeval & 0xF0) >> 4);
+  low_bits = (timeval & 0x0F);
   return ((1 << low_bits) / 16.0) * (1 + (high_bits / 16.0));
 }
 
 /*------------------------- TC Dissecting Code-------------------------*/
 static int dissect_olsr_tc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *olsr_tree, int offset, int message_end) {
   if (message_end - offset < 4) {
-    proto_tree_add_bytes_format(olsr_tree, hf_olsr_ansn, tvb, offset, message_end - offset, tvb_get_ptr(tvb, offset,
-        message_end - offset), "Not enough bytes for TC");
+    proto_tree_add_bytes_format(olsr_tree, hf_olsr_ansn, tvb, offset, message_end - offset,
+        NULL, "Not enough bytes for TC");
     return message_end;
   }
 
@@ -182,8 +181,8 @@ static int dissect_olsr_tc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *olsr_t
 
   while (offset < message_end) {
     if (message_end - offset < pinfo->src.len) {
-      proto_tree_add_bytes_format(olsr_tree, hf_olsr_neighbor_addr, tvb, offset, message_end - offset, tvb_get_ptr(tvb,
-          offset, message_end - offset), "Not enough bytes for last neighbor");
+      proto_tree_add_bytes_format(olsr_tree, hf_olsr_neighbor_addr, tvb, offset, message_end - offset,
+          NULL, "Not enough bytes for last neighbor");
       return message_end;
     }
     if (pinfo->src.type == AT_IPv4) {
@@ -202,8 +201,8 @@ static int dissect_olsr_tc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *olsr_t
 static int dissect_olsrorg_lq_tc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *olsr_tree, int offset, int message_end) {
 
   if (message_end - offset < 4) {
-    proto_tree_add_bytes_format(olsr_tree, hf_olsr_ansn, tvb, offset, message_end - offset, tvb_get_ptr(tvb, offset,
-        message_end - offset), "Not enough bytes for Olsr.org LQ-TC");
+    proto_tree_add_bytes_format(olsr_tree, hf_olsr_ansn, tvb, offset, message_end - offset,
+        NULL, "Not enough bytes for Olsr.org LQ-TC");
     return message_end;
   }
   proto_tree_add_item(olsr_tree, hf_olsr_ansn, tvb, offset, 2, FALSE);
@@ -212,21 +211,19 @@ static int dissect_olsrorg_lq_tc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *
   while (offset < message_end) {
     proto_item *address_group;
     proto_tree *address_tree;
-    const guint8 *ptr;
     guint8 lq, nlq;
 
     if (pinfo->src.type == AT_IPv4) {
       if (message_end - offset < 8) {
-        proto_tree_add_bytes_format(olsr_tree, hf_olsr_neighbor, tvb, offset, message_end - offset, tvb_get_ptr(tvb,
-            offset, message_end - offset), "Not enough bytes for last entry (need 8 bytes)");
+        proto_tree_add_bytes_format(olsr_tree, hf_olsr_neighbor, tvb, offset, message_end - offset,
+            NULL, "Not enough bytes for last entry (need 8 bytes)");
         return message_end;
       }
-      ptr = tvb_get_ptr(tvb, offset, 8);
       lq = tvb_get_guint8(tvb, offset + 4);
       nlq = tvb_get_guint8(tvb, offset + 5);
 
-      address_group = proto_tree_add_bytes_format(olsr_tree, hf_olsr_neighbor, tvb, offset, 8, ptr,
-          "Neighbor Address: %s (%d/%d)", ip_to_str(ptr), lq, nlq);
+      address_group = proto_tree_add_bytes_format(olsr_tree, hf_olsr_neighbor, tvb, offset, 8,
+          NULL, "Neighbor Address: %s (%d/%d)", tvb_ip_to_str(tvb, offset), lq, nlq);
 
       address_tree = proto_item_add_subtree(address_group, ett_olsr_message_neigh);
 
@@ -234,16 +231,15 @@ static int dissect_olsrorg_lq_tc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *
       offset += 4;
     } else if (pinfo->src.type == AT_IPv6) {
       if (message_end - offset < 20) {
-        proto_tree_add_bytes_format(olsr_tree, hf_olsr_neighbor, tvb, offset, message_end - offset, tvb_get_ptr(tvb,
-            offset, message_end - offset), "Not enough bytes for last entry (need 20 bytes)");
+        proto_tree_add_bytes_format(olsr_tree, hf_olsr_neighbor, tvb, offset, message_end - offset,
+            NULL, "Not enough bytes for last entry (need 20 bytes)");
         return message_end;
       }
-      ptr = tvb_get_ptr(tvb, offset, 20);
       lq = tvb_get_guint8(tvb, offset + 16);
       nlq = tvb_get_guint8(tvb, offset + 17);
 
-      address_group = proto_tree_add_bytes_format(olsr_tree, hf_olsr_neighbor, tvb, offset, 20, ptr,
-          "Neighbor Address: %s (%d/%d)", ip6_to_str((const struct e_in6_addr *) ptr), lq, nlq);
+      address_group = proto_tree_add_bytes_format(olsr_tree, hf_olsr_neighbor, tvb, offset, 20,
+          NULL, "Neighbor Address: %s (%d/%d)", tvb_ip6_to_str(tvb, offset), lq, nlq);
 
       address_tree = proto_item_add_subtree(address_group, ett_olsr_message_neigh);
 
@@ -263,8 +259,8 @@ static int dissect_olsrorg_lq_tc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *
 static int dissect_nrlolsr_tc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *olsr_tree, int offset, int message_end) {
   int field1Ptr, field2Ptr, saneEnd;
   if (message_end - offset < 4) {
-    proto_tree_add_bytes_format(olsr_tree, hf_olsr_ansn, tvb, offset, message_end - offset, tvb_get_ptr(tvb, offset,
-        message_end - offset), "Not enough bytes for NRLOLSR LQ-TC");
+    proto_tree_add_bytes_format(olsr_tree, hf_olsr_ansn, tvb, offset, message_end - offset,
+        NULL, "Not enough bytes for NRLOLSR LQ-TC");
     return message_end;
   }
 
@@ -302,8 +298,8 @@ static int dissect_olsr_hello(tvbuff_t *tvb, packet_info *pinfo, proto_tree *ols
   guint16 message_size = 0;
 
   if (message_end - offset < 4) {
-    proto_tree_add_bytes_format(olsr_tree, hf_olsr_htime, tvb, offset, message_end - offset, tvb_get_ptr(tvb, offset,
-        message_end - offset), "Not enough bytes for Hello");
+    proto_tree_add_bytes_format(olsr_tree, hf_olsr_htime, tvb, offset, message_end - offset,
+        NULL, "Not enough bytes for Hello");
     return message_end;
   }
 
@@ -320,8 +316,8 @@ static int dissect_olsr_hello(tvbuff_t *tvb, packet_info *pinfo, proto_tree *ols
 
   while (offset < message_end) {
     if (message_end - offset < 4) {
-      proto_tree_add_bytes_format(olsr_tree, hf_olsr_link_type, tvb, offset, message_end - offset, tvb_get_ptr(tvb,
-          offset, message_end - offset), "Not enough bytes for last Hello entry");
+      proto_tree_add_bytes_format(olsr_tree, hf_olsr_link_type, tvb, offset, message_end - offset,
+          NULL, "Not enough bytes for last Hello entry");
       return message_end;
     }
 
@@ -351,8 +347,8 @@ static int handle_olsr_hello_rfc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *
   /*-------------------Dissect Neighbor Addresses--------------------*/
   while (offset < link_message_end) {
     if (link_message_end - offset < pinfo->src.len) {
-      proto_tree_add_bytes_format(olsr_tree, hf_olsr_neighbor, tvb, offset, link_message_end - offset, tvb_get_ptr(tvb,
-          offset, link_message_end - offset), "Not enough bytes for last Hello entry");
+      proto_tree_add_bytes_format(olsr_tree, hf_olsr_neighbor, tvb, offset, link_message_end - offset,
+          NULL, "Not enough bytes for last Hello entry");
       return link_message_end;
     }
     if (pinfo->src.type == AT_IPv4) {
@@ -375,34 +371,31 @@ static int handle_olsr_hello_olsrorg(tvbuff_t *tvb, packet_info *pinfo, proto_tr
   while (offset < link_message_end) {
     proto_item *address_group;
     proto_tree *address_tree;
-    const guint8 *ptr;
     guint8 lq, nlq;
 
     if (link_message_end - offset < pinfo->src.len + 4) {
-      proto_tree_add_bytes_format(olsr_tree, hf_olsr_neighbor, tvb, offset, link_message_end - offset, tvb_get_ptr(tvb,
-          offset, link_message_end - offset), "Not enough bytes for last Olsr.org LQ-Hello entry");
+      proto_tree_add_bytes_format(olsr_tree, hf_olsr_neighbor, tvb, offset, link_message_end - offset,
+          NULL, "Not enough bytes for last Olsr.org LQ-Hello entry");
       return link_message_end;
     }
 
     if (pinfo->src.type == AT_IPv4) {
-      ptr = tvb_get_ptr(tvb, offset, 8);
       lq = tvb_get_guint8(tvb, offset + 4);
       nlq = tvb_get_guint8(tvb, offset + 5);
 
-      address_group = proto_tree_add_bytes_format(olsr_tree, hf_olsr_neighbor, tvb, offset, 8, ptr,
-          "Neighbor Address: %s (%d/%d)", ip_to_str(ptr), lq, nlq);
+      address_group = proto_tree_add_bytes_format(olsr_tree, hf_olsr_neighbor, tvb, offset, 8,
+           NULL, "Neighbor Address: %s (%d/%d)", tvb_ip_to_str(tvb, offset), lq, nlq);
 
       address_tree = proto_item_add_subtree(address_group, ett_olsr_message_neigh);
 
       proto_tree_add_item(address_tree, hf_olsr_neighbor_addr, tvb, offset, 4, FALSE);
       offset += 4;
     } else if (pinfo->src.type == AT_IPv6) {
-      ptr = tvb_get_ptr(tvb, offset, 20);
       lq = tvb_get_guint8(tvb, offset + 16);
       nlq = tvb_get_guint8(tvb, offset + 17);
 
-      address_group = proto_tree_add_bytes_format(olsr_tree, hf_olsr_neighbor, tvb, offset, 20, ptr,
-          "Neighbor Address: %s (%d/%d)", ip6_to_str((const struct e_in6_addr *) ptr), lq, nlq);
+      address_group = proto_tree_add_bytes_format(olsr_tree, hf_olsr_neighbor, tvb, offset, 20,
+          NULL, "Neighbor Address: %s (%d/%d)", tvb_ip6_to_str(tvb, offset), lq, nlq);
 
       address_tree = proto_item_add_subtree(address_group, ett_olsr_message_neigh);
 
@@ -423,8 +416,8 @@ static int handle_olsr_hello_olsrorg(tvbuff_t *tvb, packet_info *pinfo, proto_tr
 static int dissect_olsr_mid(tvbuff_t *tvb, packet_info *pinfo, proto_tree *olsr_tree, int offset, int message_end) {
   while (offset < message_end) {
     if (message_end - offset < pinfo->src.len) {
-      proto_tree_add_bytes_format(olsr_tree, hf_olsr_interface_addr, tvb, offset, message_end - offset, tvb_get_ptr(
-          tvb, offset, message_end - offset), "Not enough bytes for last MID entry");
+      proto_tree_add_bytes_format(olsr_tree, hf_olsr_interface_addr, tvb, offset, message_end - offset,
+          NULL, "Not enough bytes for last MID entry");
       return message_end;
     }
     if (pinfo->src.type == AT_IPv4) {
@@ -444,8 +437,8 @@ static int dissect_olsr_mid(tvbuff_t *tvb, packet_info *pinfo, proto_tree *olsr_
 static int dissect_olsr_hna(tvbuff_t *tvb, packet_info *pinfo, proto_tree *olsr_tree, int offset, int message_end) {
   while (offset < message_end) {
     if (message_end - offset < pinfo->src.len * 2) {
-      proto_tree_add_bytes_format(olsr_tree, hf_olsr_network_addr, tvb, offset, message_end - offset, tvb_get_ptr(tvb,
-          offset, message_end - offset), "Not enough bytes for last HNA entry");
+      proto_tree_add_bytes_format(olsr_tree, hf_olsr_network_addr, tvb, offset, message_end - offset,
+          NULL, "Not enough bytes for last HNA entry");
       return message_end;
     }
 
@@ -475,8 +468,8 @@ static int dissect_olsrorg_nameservice(tvbuff_t *tvb, packet_info *pinfo, proto_
   proto_tree *olsr_ns_tree;
 
   if (message_end - offset < 4) {
-    proto_tree_add_bytes_format(olsr_tree, hf_olsrorg_ns_version, tvb, offset, message_end - offset, tvb_get_ptr(tvb,
-        offset, message_end - offset), "Not enough bytes for Olsr.org Nameservice message");
+    proto_tree_add_bytes_format(olsr_tree, hf_olsrorg_ns_version, tvb, offset, message_end - offset,
+        NULL, "Not enough bytes for Olsr.org Nameservice message");
     return message_end;
   }
 
@@ -489,8 +482,8 @@ static int dissect_olsrorg_nameservice(tvbuff_t *tvb, packet_info *pinfo, proto_
   offset += 4;
 
   if (version != 1) {
-    proto_tree_add_bytes_format(olsr_tree, hf_olsr_data, tvb, offset, message_end - offset, tvb_get_ptr(tvb, offset,
-        message_end - offset), "Unkown nameservice protocol version %d", version);
+    proto_tree_add_bytes_format(olsr_tree, hf_olsr_data, tvb, offset, message_end - offset,
+        NULL, "Unkown nameservice protocol version %d", version);
     return message_end;
   }
 
@@ -499,8 +492,8 @@ static int dissect_olsrorg_nameservice(tvbuff_t *tvb, packet_info *pinfo, proto_
     int total_length;
 
     if (message_end - offset < 20) {
-      proto_tree_add_bytes_format(olsr_tree, hf_olsrorg_ns, tvb, offset, message_end - offset, tvb_get_ptr(tvb, offset,
-          message_end - offset), "Not enough bytes for last nameservice entry");
+      proto_tree_add_bytes_format(olsr_tree, hf_olsrorg_ns, tvb, offset, message_end - offset,
+          NULL, "Not enough bytes for last nameservice entry");
       return message_end;
     }
 
@@ -509,8 +502,8 @@ static int dissect_olsrorg_nameservice(tvbuff_t *tvb, packet_info *pinfo, proto_
 
     total_length = 4 + 16 + ((length - 1) | 3) + 1;
 
-    olsr_ns_item = proto_tree_add_bytes_format(olsr_tree, hf_olsrorg_ns, tvb, offset, total_length, tvb_get_ptr(tvb,
-        offset, total_length), "Nameservice: %s (%d)", val_to_str(type, nameservice_type_vals, "UNKNOWN"), type);
+    olsr_ns_item = proto_tree_add_bytes_format(olsr_tree, hf_olsrorg_ns, tvb, offset, total_length,
+        NULL, "Nameservice: %s (%d)", val_to_str(type, nameservice_type_vals, "UNKNOWN"), type);
 
     olsr_ns_tree = proto_item_add_subtree(olsr_ns_item, ett_olsr_message_ns);
 
@@ -526,8 +519,8 @@ static int dissect_olsrorg_nameservice(tvbuff_t *tvb, packet_info *pinfo, proto_
     }
 
     if (message_end - offset < total_length) {
-      proto_tree_add_bytes_format(olsr_tree, hf_olsrorg_ns, tvb, offset, message_end - offset, tvb_get_ptr(tvb, offset,
-          message_end - offset), "Not enough bytes for content of last nameservice entry");
+      proto_tree_add_bytes_format(olsr_tree, hf_olsrorg_ns, tvb, offset, message_end - offset,
+          NULL, "Not enough bytes for content of last nameservice entry");
       return message_end;
     }
     proto_tree_add_item(olsr_ns_tree, hf_olsrorg_ns_content, tvb, offset + 20, length, FALSE);
@@ -594,7 +587,7 @@ static int dissect_olsr(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
 
       if (packet_len - offset < 4) {
         proto_tree_add_bytes_format(olsr_tree, hf_olsr_message, tvb, offset, packet_len - offset,
-            tvb_get_ptr(tvb, offset, packet_len - offset), "Message too short !");
+            NULL, "Message too short !");
         break;
       }
 
@@ -602,8 +595,8 @@ static int dissect_olsr(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
       vTime = getOlsrTime(tvb_get_guint8(tvb, offset + 1));
       message_len = tvb_get_ntohs(tvb, offset + 2);
 
-      message_item = proto_tree_add_bytes_format(olsr_tree, hf_olsr_message, tvb, offset, message_len, tvb_get_ptr(tvb,
-          offset, message_len), "Message: %s (%d)", val_to_str(message_type, message_type_vals, "UNKNOWN"),
+      message_item = proto_tree_add_bytes_format(olsr_tree, hf_olsr_message, tvb, offset, message_len,
+          NULL, "Message: %s (%d)", val_to_str(message_type, message_type_vals, "UNKNOWN"),
           message_type);
       message_tree = proto_item_add_subtree(message_item, ett_olsr_message[message_type]);
 
@@ -684,12 +677,12 @@ static int dissect_olsr(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
         /*-----------------------------Undefined message types-----------------------------*/
         else {
           if ((message_len - 12) % 4) {
-            proto_tree_add_bytes_format(message_tree, hf_olsr_data, tvb, offset, message_len - 12, tvb_get_ptr(tvb,
-                offset, message_len - 12), "Data (%u bytes) (must be aligned on 32 bits)", message_len - 12);
+            proto_tree_add_bytes_format(message_tree, hf_olsr_data, tvb, offset, message_len - 12,
+                NULL, "Data (%u bytes) (must be aligned on 32 bits)", message_len - 12);
             break;
           }
-          proto_tree_add_bytes_format(message_tree, hf_olsr_data, tvb, offset, message_len - 12, tvb_get_ptr(tvb,
-              offset, message_len - 12), "Data (%u bytes)", message_len - 12);
+          proto_tree_add_bytes_format(message_tree, hf_olsr_data, tvb, offset, message_len - 12,
+              NULL, "Data (%u bytes)", message_len - 12);
           offset += message_len - 12;
         } /* end if for undefined message types */
       }
@@ -976,37 +969,35 @@ void proto_register_olsr(void) {
     &ett_olsr_message_ns
   };
 
-  static gint *ett[array_length(ett_base) + (G_MAXUINT8+1)];
+  gint *ett[array_length(ett_base) + (G_MAXUINT8+1)];
 
   module_t *olsr_module;
   int i,j;
 
-  if (proto_olsr == -1) {
-    memcpy(ett, ett_base, sizeof(ett_base));
-    j = array_length(ett_base);
-    for (i=0; i<G_MAXUINT8+1; i++) {
-      ett_olsr_message[i] = -1;
-      ett[j++] = &ett_olsr_message[i];
-    }
-
-    proto_olsr = proto_register_protocol("Optimized Link State Routing Protocol", "OLSR", "olsr");
-
-    proto_register_field_array(proto_olsr, hf, array_length(hf));
-    proto_register_subtree_array(ett, array_length(ett));
-
-    olsr_module = prefs_register_protocol(proto_olsr, NULL);
-    prefs_register_bool_preference(olsr_module, "ff_olsrorg",
-        "Dissect olsr.org messages","Dissect custom olsr.org message types (compatible with rfc routing agents)",
-        &global_olsr_olsrorg);
-    prefs_register_bool_preference(olsr_module, "nrlolsr",
-        "Dissect NRL-Olsr TC messages", "Dissect custom nrlolsr tc message (incompatible with rfc routing agents)",
-        &global_olsr_nrlolsr);
+  memcpy(ett, ett_base, sizeof(ett_base));
+  j = array_length(ett_base);
+  for (i=0; i<G_MAXUINT8+1; i++) {
+    ett_olsr_message[i] = -1;
+    ett[j++] = &ett_olsr_message[i];
   }
+
+  proto_olsr = proto_register_protocol("Optimized Link State Routing Protocol", "OLSR", "olsr");
+
+  proto_register_field_array(proto_olsr, hf, array_length(hf));
+  proto_register_subtree_array(ett, array_length(ett));
+
+  olsr_module = prefs_register_protocol(proto_olsr, NULL);
+  prefs_register_bool_preference(olsr_module, "ff_olsrorg",
+                                 "Dissect olsr.org messages","Dissect custom olsr.org message types (compatible with rfc routing agents)",
+                                 &global_olsr_olsrorg);
+  prefs_register_bool_preference(olsr_module, "nrlolsr",
+                                 "Dissect NRL-Olsr TC messages", "Dissect custom nrlolsr tc message (incompatible with rfc routing agents)",
+                                 &global_olsr_nrlolsr);
 }
 
 void proto_reg_handoff_olsr(void) {
   dissector_handle_t olsr_handle;
 
   olsr_handle = new_create_dissector_handle(dissect_olsr, proto_olsr);
-  dissector_add("udp.port", UDP_PORT_OLSR, olsr_handle);
+  dissector_add_uint("udp.port", UDP_PORT_OLSR, olsr_handle);
 }
