@@ -1,7 +1,7 @@
 /* packet.c
  * Routines for packet disassembly
  *
- * $Id: packet.c 30527 2009-10-12 01:31:01Z wmeier $
+ * $Id: packet.c 37398 2011-05-25 21:19:56Z gerald $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -65,9 +65,9 @@ static dissector_handle_t data_handle = NULL;
 void
 packet_init(void)
 {
-  frame_handle = find_dissector("frame");
-  data_handle = find_dissector("data");
-  proto_malformed = proto_get_id_by_filter_name("malformed");
+	frame_handle = find_dissector("frame");
+	data_handle = find_dissector("data");
+	proto_malformed = proto_get_id_by_filter_name("malformed");
 }
 
 void
@@ -81,7 +81,7 @@ packet_cleanup(void)
  * of the tvbuff to reflect the specified length.
  */
 void
-set_actual_length(tvbuff_t *tvb, guint specified_len)
+set_actual_length(tvbuff_t *tvb, const guint specified_len)
 {
 	if (specified_len < tvb_reported_length(tvb)) {
 		/* Adjust the length of this tvbuff to include only the specified
@@ -225,20 +225,6 @@ add_new_data_source(packet_info *pinfo, tvbuff_t *tvb, const char *name)
 	pinfo->data_src = g_slist_append(pinfo->data_src, src);
 }
 
-/*
- * This should only add a data source to the list of data sources for
- * a frame if the data sources are being used.  Note that they can
- * be used even if the protocol tree isn't being built or isn't visible,
- * e.g. if you run tshark with -x but without -V or anything else to
- * cause the protocol tree to be built.
- */
-void
-packet_add_new_data_source(packet_info *pinfo, proto_tree *tree _U_,
-			   tvbuff_t *tvb, const char *name)
-{
-	add_new_data_source(pinfo, tvb, name);
-}
-
 const char*
 get_data_source_name(data_source *src)
 {
@@ -322,12 +308,15 @@ dissect_packet(epan_dissect_t *edt, union wtap_pseudo_header *pseudo_header,
 	edt->pi.dcerpc_procedure_name="";
 	edt->pi.link_dir = LINK_DIR_UNKNOWN;
 
+	/* to enable decode as for ethertype=0x0000 (fix for bug 4721) */
+	edt->pi.ethertype = G_MAXINT;
+
 	EP_CHECK_CANARY(("before dissecting frame %d",fd->num));
 
 	TRY {
 		edt->tvb = tvb_new_real_data(pd, fd->cap_len, fd->pkt_len);
 		/* Add this tvbuffer into the data_src list */
-		packet_add_new_data_source(&edt->pi, edt->tree, edt->tvb, "Frame");
+		add_new_data_source(&edt->pi, edt->tvb, "Frame");
 
 		/* Even though dissect_frame() catches all the exceptions a
 		 * sub-dissector can throw, dissect_frame() itself may throw
@@ -361,7 +350,7 @@ dissect_packet(epan_dissect_t *edt, union wtap_pseudo_header *pseudo_header,
 /*********************** code added for sub-dissector lookup *********************/
 
 /*
- * An dissector handle.
+ * A dissector handle.
  */
 struct dissector_handle {
 	const char	*name;		/* dissector name */
@@ -634,19 +623,21 @@ struct dtbl_entry {
  *
  * "hash_table" is a hash table, indexed by port number, supplying
  * a "struct dtbl_entry"; it records what dissector is assigned to
- * that port number in that table.
+ * that uint or string value in that table.
  *
  * "dissector_handles" is a list of all dissectors that *could* be
  * used in that table; not all of them are necessarily in the table,
- * as they may be for protocols that don't have a fixed port number.
+ * as they may be for protocols that don't have a fixed uint value,
+ * e.g. for TCP or UDP port number tables and protocols with no fixed
+ * port number.
  *
  * "ui_name" is the name the dissector table has in the user interface.
  *
- * "type" is a field type giving the width of the port number for that
- * dissector table.
+ * "type" is a field type giving the width of the uint value for that
+ * dissector table, if it's a uint dissector table.
  *
- * "base" is the base in which to display the port number for that
- * dissector table.
+ * "base" is the base in which to display the uint value for that
+ * dissector table, if it's a uint dissector table.
  */
 struct dissector_table {
 	GHashTable	*hash_table;
@@ -668,7 +659,7 @@ find_dissector_table(const char *name)
 
 /* Find an entry in a uint dissector table. */
 static dtbl_entry_t *
-find_uint_dtbl_entry(dissector_table_t sub_dissectors, guint32 pattern)
+find_uint_dtbl_entry(dissector_table_t sub_dissectors, const guint32 pattern)
 {
 	switch (sub_dissectors->type) {
 
@@ -677,13 +668,13 @@ find_uint_dtbl_entry(dissector_table_t sub_dissectors, guint32 pattern)
 	case FT_UINT24:
 	case FT_UINT32:
 		/*
-		 * You can do a port lookup in these tables.
+		 * You can do a uint lookup in these tables.
 		 */
 		break;
 
 	default:
 		/*
-		 * But you can't do a port lookup in any other types
+		 * But you can't do a uint lookup in any other types
 		 * of tables.
 		 */
 		g_assert_not_reached();
@@ -698,7 +689,7 @@ find_uint_dtbl_entry(dissector_table_t sub_dissectors, guint32 pattern)
 
 #if 0
 static void
-dissector_add_sanity_check(const char *name, guint32 pattern, dissector_handle_t handle, dissector_table_t sub_dissectors)
+dissector_add_uint_sanity_check(const char *name, guint32 pattern, dissector_handle_t handle, dissector_table_t sub_dissectors)
 {
 	dtbl_entry_t *dtbl_entry;
 
@@ -718,14 +709,15 @@ dissector_add_sanity_check(const char *name, guint32 pattern, dissector_handle_t
 
 /* Add an entry to a uint dissector table. */
 void
-dissector_add(const char *name, guint32 pattern, dissector_handle_t handle)
+dissector_add_uint(const char *name, const guint32 pattern, dissector_handle_t handle)
 {
 	dissector_table_t sub_dissectors;
 	dtbl_entry_t *dtbl_entry;
 
 	sub_dissectors = find_dissector_table(name);
-/* sanity checks */
+	/* sanity checks */
 	g_assert(sub_dissectors);
+	g_assert(handle!=NULL);
 	switch (sub_dissectors->type) {
 
 	case FT_UINT8:
@@ -733,20 +725,20 @@ dissector_add(const char *name, guint32 pattern, dissector_handle_t handle)
 	case FT_UINT24:
 	case FT_UINT32:
 		/*
-		 * You can do a port lookup in these tables.
+		 * You can do a uint lookup in these tables.
 		 */
 		break;
 
 	default:
 		/*
-		 * But you can't do a port lookup in any other types
+		 * But you can't do a uint lookup in any other types
 		 * of tables.
 		 */
 		g_assert_not_reached();
 	}
 
 #if 0
-	dissector_add_sanity_check(name, pattern, handle, sub_dissectors);
+	dissector_add_uint_sanity_check(name, pattern, handle, sub_dissectors);
 #endif
 
 	dtbl_entry = g_malloc(sizeof (dtbl_entry_t));
@@ -768,12 +760,12 @@ dissector_add(const char *name, guint32 pattern, dissector_handle_t handle)
    with a particular pattern. */
 
 /* NOTE: this doesn't use the dissector call variable. It is included to */
-/*	be consistant with the dissector_add and more importantly to be used */
+/*	be consistant with the dissector_add_uint and more importantly to be used */
 /*	if the technique of adding a temporary dissector is implemented.  */
 /*	If temporary dissectors are deleted, then the original dissector must */
 /*	be available. */
 void
-dissector_delete(const char *name, guint32 pattern,
+dissector_delete_uint(const char *name, const guint32 pattern,
 	dissector_handle_t handle _U_)
 {
 	dissector_table_t sub_dissectors = find_dissector_table( name);
@@ -804,7 +796,7 @@ dissector_delete(const char *name, guint32 pattern,
 /* Change the entry for a dissector in a uint dissector table
    with a particular pattern to use a new dissector handle. */
 void
-dissector_change(const char *name, guint32 pattern, dissector_handle_t handle)
+dissector_change_uint(const char *name, const guint32 pattern, dissector_handle_t handle)
 {
 	dissector_table_t sub_dissectors = find_dissector_table( name);
 	dtbl_entry_t *dtbl_entry;
@@ -840,7 +832,7 @@ dissector_change(const char *name, guint32 pattern, dissector_handle_t handle)
 
 /* Reset an entry in a uint dissector table to its initial value. */
 void
-dissector_reset(const char *name, guint32 pattern)
+dissector_reset_uint(const char *name, const guint32 pattern)
 {
 	dissector_table_t sub_dissectors = find_dissector_table( name);
 	dtbl_entry_t *dtbl_entry;
@@ -873,15 +865,15 @@ dissector_reset(const char *name, guint32 pattern)
    otherwise return FALSE. */
 
 gboolean
-dissector_try_port_new(dissector_table_t sub_dissectors, guint32 port,
-		       tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gboolean add_proto_name)
+dissector_try_uint_new(dissector_table_t sub_dissectors, const guint32 uint_val,
+		       tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, const gboolean add_proto_name)
 {
 	dtbl_entry_t *dtbl_entry;
 	struct dissector_handle *handle;
-	guint32 saved_match_port;
+	guint32 saved_match_uint;
 	int ret;
 
-	dtbl_entry = find_uint_dtbl_entry(sub_dissectors, port);
+	dtbl_entry = find_uint_dtbl_entry(sub_dissectors, uint_val);
 	if (dtbl_entry != NULL) {
 		/*
 		 * Is there currently a dissector handle for this entry?
@@ -897,14 +889,14 @@ dissector_try_port_new(dissector_table_t sub_dissectors, guint32 port,
 		}
 
 		/*
-		 * Save the current value of "pinfo->match_port",
-		 * set it to the port that matched, call the
-		 * dissector, and restore "pinfo->match_port".
+		 * Save the current value of "pinfo->match_uint",
+		 * set it to the uint_val that matched, call the
+		 * dissector, and restore "pinfo->match_uint".
 		 */
-		saved_match_port = pinfo->match_port;
-		pinfo->match_port = port;
+		saved_match_uint = pinfo->match_uint;
+		pinfo->match_uint = uint_val;
 		ret = call_dissector_work(handle, tvb, pinfo, tree, add_proto_name);
-		pinfo->match_port = saved_match_port;
+		pinfo->match_uint = saved_match_uint;
 
 		/*
 		 * If a new-style dissector returned 0, it means that
@@ -925,20 +917,21 @@ dissector_try_port_new(dissector_table_t sub_dissectors, guint32 port,
 }
 
 gboolean
-dissector_try_port(dissector_table_t sub_dissectors, guint32 port,
+dissector_try_uint(dissector_table_t sub_dissectors, const guint32 uint_val,
 		   tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
 
-	return dissector_try_port_new(sub_dissectors, port, tvb, pinfo, tree, TRUE);
+	return dissector_try_uint_new(sub_dissectors, uint_val, tvb, pinfo, tree, TRUE);
 }
+
 /* Look for a given value in a given uint dissector table and, if found,
    return the dissector handle for that value. */
 dissector_handle_t
-dissector_get_port_handle(dissector_table_t sub_dissectors, guint32 port)
+dissector_get_uint_handle(dissector_table_t const sub_dissectors, const guint32 uint_val)
 {
 	dtbl_entry_t *dtbl_entry;
 
-	dtbl_entry = find_uint_dtbl_entry(sub_dissectors, port);
+	dtbl_entry = find_uint_dtbl_entry(sub_dissectors, uint_val);
 	if (dtbl_entry != NULL)
 		return dtbl_entry->current;
 	else
@@ -947,7 +940,7 @@ dissector_get_port_handle(dissector_table_t sub_dissectors, guint32 port)
 
 /* Find an entry in a string dissector table. */
 static dtbl_entry_t *
-find_string_dtbl_entry(dissector_table_t sub_dissectors, const gchar *pattern)
+find_string_dtbl_entry(dissector_table_t const sub_dissectors, const gchar *pattern)
 {
 	switch (sub_dissectors->type) {
 
@@ -1055,7 +1048,7 @@ dissector_delete_string(const char *name, const gchar *pattern,
 /* Change the entry for a dissector in a string dissector table
    with a particular pattern to use a new dissector handle. */
 void
-dissector_change_string(const char *name, gchar *pattern,
+dissector_change_string(const char *name, const gchar *pattern,
 			dissector_handle_t handle)
 {
 	dissector_table_t sub_dissectors = find_dissector_table( name);
@@ -1086,7 +1079,7 @@ dissector_change_string(const char *name, gchar *pattern,
 	dtbl_entry->current = handle;
 
 /* do the table insertion */
-	g_hash_table_insert( sub_dissectors->hash_table, pattern,
+	g_hash_table_insert( sub_dissectors->hash_table, (gpointer)pattern,
 			     (gpointer)dtbl_entry);
 }
 
@@ -1235,11 +1228,11 @@ dtbl_entry_get_initial_handle (dtbl_entry_t *dtbl_entry)
 /**************************************************/
 
 typedef struct dissector_foreach_info {
-  gpointer     caller_data;
-  DATFunc      caller_func;
-  GHFunc       next_func;
-  const gchar  *table_name;
-  ftenum_t     selector_type;
+	gpointer     caller_data;
+	DATFunc      caller_func;
+	GHFunc       next_func;
+	const gchar  *table_name;
+	ftenum_t     selector_type;
 } dissector_foreach_info_t;
 
 /*
@@ -1413,7 +1406,7 @@ typedef struct dissector_foreach_table_info {
  * Called for each entry in the table of all dissector tables.
  */
 static void
-dissector_all_tables_foreach_table_func (gpointer key, gpointer value, gpointer user_data)
+dissector_all_tables_foreach_table_func (gpointer key, const gpointer value, const gpointer user_data)
 {
 	dissector_table_t table;
 	dissector_foreach_table_info_t *info;
@@ -1439,8 +1432,8 @@ dissector_all_tables_foreach_table (DATFunc_table func,
 }
 
 dissector_table_t
-register_dissector_table(const char *name, const char *ui_name, ftenum_t type,
-			 int base)
+register_dissector_table(const char *name, const char *ui_name, const ftenum_t type,
+			 const int base)
 {
 	dissector_table_t	sub_dissectors;
 
@@ -1529,7 +1522,7 @@ find_heur_dissector_list(const char *name)
 }
 
 void
-heur_dissector_add(const char *name, heur_dissector_t dissector, int proto)
+heur_dissector_add(const char *name, heur_dissector_t dissector, const int proto)
 {
 	heur_dissector_list_t *sub_dissectors = find_heur_dissector_list(name);
 	heur_dtbl_entry_t *dtbl_entry;
@@ -1554,7 +1547,7 @@ static int find_matching_heur_dissector( gconstpointer a, gconstpointer b) {
 		(dtbl_entry_a->protocol == dtbl_entry_b->protocol) ? 0 : 1;
 }
 
-void heur_dissector_delete(const char *name, heur_dissector_t dissector, int proto) {
+void heur_dissector_delete(const char *name, heur_dissector_t dissector, const int proto) {
 	heur_dissector_list_t *sub_dissectors = find_heur_dissector_list(name);
 	heur_dtbl_entry_t dtbl_entry;
 	GSList* found_entry;
@@ -1642,7 +1635,7 @@ dissector_try_heuristic(heur_dissector_list_t sub_dissectors,
 			status = TRUE;
 			break;
 		} else {
-			EP_CHECK_CANARY(("after heuristic dissector for protocol: %s has returned true",
+			EP_CHECK_CANARY(("after heuristic dissector for protocol: %s has returned false",
 					 proto_get_protocol_filter_name(proto_get_id(dtbl_entry->protocol))));
 
 			/*
@@ -1689,10 +1682,21 @@ register_heur_dissector_list(const char *name, heur_dissector_list_t *sub_dissec
  */
 static GHashTable *registered_dissectors = NULL;
 
+/* Get the long name of the protocol for a dissector handle, if it has
+   a protocol. */
+const char *
+dissector_handle_get_long_name(const dissector_handle_t handle)
+{
+	if (handle == NULL || handle->protocol == NULL) {
+		return NULL;
+	}
+	return proto_get_protocol_long_name(handle->protocol);
+}
+
 /* Get the short name of the protocol for a dissector handle, if it has
    a protocol. */
 const char *
-dissector_handle_get_short_name(dissector_handle_t handle)
+dissector_handle_get_short_name(const dissector_handle_t handle)
 {
 	if (handle->protocol == NULL) {
 		/*
@@ -1709,7 +1713,7 @@ dissector_handle_get_short_name(dissector_handle_t handle)
 /* Get the index of the protocol for a dissector handle, if it has
    a protocol. */
 int
-dissector_handle_get_protocol_index(dissector_handle_t handle)
+dissector_handle_get_protocol_index(const dissector_handle_t handle)
 {
 	if (handle->protocol == NULL) {
 		/*
@@ -1733,7 +1737,7 @@ find_dissector(const char *name)
 
 /* Create an anonymous handle for a dissector. */
 dissector_handle_t
-create_dissector_handle(dissector_t dissector, int proto)
+create_dissector_handle(dissector_t dissector, const int proto)
 {
 	struct dissector_handle *handle;
 
@@ -1747,7 +1751,7 @@ create_dissector_handle(dissector_t dissector, int proto)
 }
 
 dissector_handle_t
-new_create_dissector_handle(new_dissector_t dissector, int proto)
+new_create_dissector_handle(new_dissector_t dissector, const int proto)
 {
 	struct dissector_handle *handle;
 
@@ -1762,7 +1766,7 @@ new_create_dissector_handle(new_dissector_t dissector, int proto)
 
 /* Register a dissector by name. */
 void
-register_dissector(const char *name, dissector_t dissector, int proto)
+register_dissector(const char *name, dissector_t dissector, const int proto)
 {
 	struct dissector_handle *handle;
 
@@ -1786,7 +1790,7 @@ register_dissector(const char *name, dissector_t dissector, int proto)
 }
 
 void
-new_register_dissector(const char *name, new_dissector_t dissector, int proto)
+new_register_dissector(const char *name, new_dissector_t dissector, const int proto)
 {
 	struct dissector_handle *handle;
 
@@ -1860,7 +1864,7 @@ call_dissector(dissector_handle_t handle, tvbuff_t *tvb,
 
 static void
 dissector_dump_decodes_display(const gchar *table_name,
-			       ftenum_t selector_type _U_, gpointer key, gpointer value,
+			       ftenum_t selector_type _U_, const gpointer key, const gpointer value,
 			       gpointer user_data _U_)
 {
 	guint32 selector = (guint32)(unsigned long) key;
@@ -1898,7 +1902,7 @@ dissector_dump_decodes_display(const gchar *table_name,
 }
 
 void
-dissector_dump_decodes()
+dissector_dump_decodes(void)
 {
 	dissector_all_tables_foreach(dissector_dump_decodes_display, NULL);
 }
@@ -1917,7 +1921,7 @@ register_postdissector(dissector_handle_t handle)
 }
 
 gboolean
-have_postdissector()
+have_postdissector(void)
 {
 	guint i;
 	dissector_handle_t handle;
