@@ -1,7 +1,7 @@
 /* prefs.h
  * Definitions for preference handling routines
  *
- * $Id: prefs.h 30136 2009-09-25 05:19:18Z etxrab $
+ * $Id: prefs.h 36875 2011-04-26 21:59:11Z gerald $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -119,6 +119,8 @@ typedef struct _e_prefs {
   gchar   *gui_font_name;
   color_t  gui_marked_fg;
   color_t  gui_marked_bg;
+  color_t  gui_ignored_fg;
+  color_t  gui_ignored_bg;
   gchar   *gui_colorized_fg;
   gchar   *gui_colorized_bg;
   gboolean gui_geometry_save_position;
@@ -145,10 +147,13 @@ typedef struct _e_prefs {
   gint     console_log_level;
   guint32  name_resolve;
   gint     name_resolve_concurrency;
+  gboolean load_smi_modules;
+  gboolean suppress_smi_errors;
   gchar   *capture_device;
   gchar   *capture_devices_linktypes;
   gchar   *capture_devices_descr;
   gchar   *capture_devices_hide;
+  gchar   *capture_devices_monitor_mode;
   gboolean capture_prom_mode;
   gboolean capture_pcap_ng;
   gboolean capture_real_time;
@@ -296,7 +301,7 @@ extern const char *prefs_get_title_by_name(const char *name);
  */
 extern module_t *prefs_find_module(const char *name);
 
-/** Given a module name, and a preference name return a pointer to the given 
+/** Given a module name, and a preference name return a pointer to the given
  * module's given preference or NULL if it's not found.
  *
  * @param module The preference module name.  Usually the same as the protocol
@@ -368,6 +373,7 @@ extern void prefs_register_uat_preference(module_t *module,
 extern void prefs_register_obsolete_preference(module_t *module,
     const char *name);
 
+
 typedef guint (*pref_cb)(pref_t *pref, gpointer user_data);
 
 /*
@@ -438,9 +444,23 @@ typedef enum {
 
 extern prefs_set_pref_e prefs_set_pref(char *prefarg);
 
+/*
+ * Get or set a preference's obsolete status. These can be used to make a
+ * preference obsolete after startup so that we can fetch its value but
+ * keep it from showing up in the prefrences dialog.
+ */
+extern gboolean prefs_get_preference_obsolete(pref_t *pref);
+extern prefs_set_pref_e prefs_set_preference_obsolete(pref_t *pref);
+
+
 /*
  * Returns TRUE if the given device is hidden
  */
 extern gboolean prefs_is_capture_device_hidden(const char *name);
 
+/*
+ * Returns TRUE if the given device should capture in monitor mode by default
+ */
+extern gboolean prefs_capture_device_monitor_mode(const char *name);
+
 #endif /* prefs.h */
