@@ -3,7 +3,7 @@
  *
  * (c) 2006, Luis E. Garcia Ontanon <luis@ontanon.org>
  *
- * $Id: wslua_gui.c 28700 2009-06-11 13:08:06Z stig $
+ * $Id: wslua_gui.c 37029 2011-05-09 12:49:39Z darkjames $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -24,6 +24,10 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "wslua.h"
 
 /* WSLUA_MODULE Gui GUI support */
@@ -46,7 +50,7 @@ WSLUA_FUNCTION wslua_gui_enabled(lua_State* L) { /* Checks whether the GUI facil
     WSLUA_RETURN(1); /* A boolean: true if it is enabled, false if it isn't. */
 }
 
-void lua_menu_callback(gpointer data) {
+static void lua_menu_callback(gpointer data) {
     struct _lua_menu_data* md = data;
 	lua_State* L = md->L;
 
@@ -160,7 +164,7 @@ struct _close_cb_data {
 };
 
 
-int text_win_close_cb_error_handler(lua_State* L) {
+static int text_win_close_cb_error_handler(lua_State* L) {
     const gchar* error =  lua_tostring(L,1);
     report_failure("Lua: Error During execution of TextWindow close callback:\n %s",error);
     return 0;
@@ -171,7 +175,7 @@ static void text_win_close_cb(void* data) {
     lua_State* L = cbd->L;
 
     if (cbd->L) { /* close function is set */
-	
+
         lua_settop(L,0);
         lua_pushcfunction(L,text_win_close_cb_error_handler);
         lua_rawgeti(L, LUA_REGISTRYINDEX, cbd->func_ref);
@@ -189,7 +193,7 @@ static void text_win_close_cb(void* data) {
                 break;
         }
     }
-    
+
     if (cbd->wslua_tw->expired) {
         g_free(cbd->wslua_tw);
     } else {
@@ -216,17 +220,14 @@ WSLUA_FUNCTION wslua_new_dialog(lua_State* L) { /* Pops up a new dialog */
 
     if (! (title  = luaL_checkstring(L,WSLUA_ARG_new_dialog_TITLE)) ) {
         WSLUA_ARG_ERROR(new_dialog,TITLE,"Must be a string");
-        return 0;
     }
 
     if (! lua_isfunction(L,WSLUA_ARG_new_dialog_ACTION)) {
         WSLUA_ARG_ERROR(new_dialog,ACTION,"Must be a function");
-        return 0;
     }
 
     if (top < 3) {
         WSLUA_ERROR(new_dialog,"At least one field required");
-        return 0;
     }
 
 
@@ -273,13 +274,13 @@ WSLUA_CONSTRUCTOR ProgDlg_new(lua_State* L) { /* Creates a new TextWindow. */
     pd->title = g_strdup(luaL_optstring(L,WSLUA_OPTARG_ProgDlg_new_TITLE,"Progress"));
     pd->task = g_strdup(luaL_optstring(L,WSLUA_OPTARG_ProgDlg_new_TASK,""));
     pd->stopped = FALSE;
-    
+
     if (ops->new_progress_window) {
         pd->pw = ops->new_progress_window(pd->title,pd->task,TRUE,&(pd->stopped));
     }
-    
+
     pushProgDlg(L,pd);
-    
+
 	WSLUA_RETURN(1); /* The newly created TextWindow object. */
 }
 
@@ -289,32 +290,33 @@ WSLUA_METHOD ProgDlg_update(lua_State* L) { /* Appends text */
     ProgDlg pd = checkProgDlg(L,1);
     double pr = lua_tonumber(L,WSLUA_ARG_ProgDlg_update_PROGRESS);
     const gchar* task = luaL_optstring(L,WSLUA_OPTARG_ProgDlg_update_TASK,"");
-    
+
     g_free(pd->task);
     pd->task = g_strdup(task);
 
+	/* XXX, dead code: pd already dereferenced. should it be: !pd->task?
 	if (!pd) {
 		WSLUA_ERROR(ProgDlg_update,"Cannot be called for something not a ProgDlg");
-    }
-    
+    } */
+
     if (pr >= 0.0 || pr <= 1.0) {
         ops->update_progress(pd->pw, (float) pr, task);
     } else {
         WSLUA_ERROR(ProgDlg_update,"Progress value out of range (must be between 0.0 and 1.0)");
     }
-    
+
     return 0;
 }
 
 WSLUA_METHOD ProgDlg_stopped(lua_State* L) { /* Checks wheher the user has pressed the stop button.  */
     ProgDlg pd = checkProgDlg(L,1);
-    
+
 	if (!pd) {
 		WSLUA_ERROR(ProgDlg_stopped,"Cannot be called for something not a ProgDlg");
     }
-    
+
     lua_pushboolean(L,pd->stopped);
-    
+
 	WSLUA_RETURN(1); /* true if the user has asked to stop the progress. */
 }
 
@@ -322,7 +324,7 @@ WSLUA_METHOD ProgDlg_stopped(lua_State* L) { /* Checks wheher the user has press
 
 WSLUA_METHOD ProgDlg_close(lua_State* L) { /* Appends text */
     ProgDlg pd = checkProgDlg(L,1);
-    
+
 	if (!pd) {
 		WSLUA_ERROR(ProgDlg_update,"Cannot be called for something not a ProgDlg");
     }
@@ -335,29 +337,29 @@ WSLUA_METHOD ProgDlg_close(lua_State* L) { /* Appends text */
 }
 
 
-static int ProgDlg__tostring(lua_State* L) { 
+static int ProgDlg__tostring(lua_State* L) {
     ProgDlg pd = checkProgDlg(L,1);
-    
+
     if (pd) {
         lua_pushstring(L,ep_strdup_printf("%sstopped",pd->stopped?"":"not "));
     } else {
         luaL_error(L, "ProgDlg__tostring has being passed something else!");
     }
-	
+
     return 0;
 }
 
-static int ProgDlg__gc(lua_State* L) { 
+static int ProgDlg__gc(lua_State* L) {
     ProgDlg pd = checkProgDlg(L,1);
-    
+
     if (pd) {
         if (pd->pw) ops->destroy_progress_window(pd->pw);
-        
+
         g_free(pd);
     } else {
         luaL_error(L, "ProgDlg__gc has being passed something else!");
     }
-	
+
     return 0;
 }
 
@@ -377,11 +379,11 @@ WSLUA_META ProgDlg_meta[] = {
 };
 
 int ProgDlg_register(lua_State* L) {
-    
+
     ops = funnel_get_funnel_ops();
-    
+
 	WSLUA_REGISTER_CLASS(ProgDlg);
-    
+
     return 1;
 }
 
@@ -390,7 +392,7 @@ int ProgDlg_register(lua_State* L) {
 WSLUA_CLASS_DEFINE(TextWindow,NOP,NOP); /* Manages a text window. */
 
 /* XXX: button and close callback data is being leaked */
-/* XXX: lua callback function and TextWindow are not garbage collected because 
+/* XXX: lua callback function and TextWindow are not garbage collected because
    they stay in LUA_REGISTRYINDEX forever */
 
 WSLUA_CONSTRUCTOR TextWindow_new(lua_State* L) { /* Creates a new TextWindow. */
@@ -404,7 +406,7 @@ WSLUA_CONSTRUCTOR TextWindow_new(lua_State* L) { /* Creates a new TextWindow. */
     tw = g_malloc(sizeof(struct _wslua_tw));
     tw->expired = FALSE;
     tw->ws_tw = ops->new_text_window(title);
-    
+
     default_cbd = g_malloc(sizeof(struct _close_cb_data));
 
     default_cbd->L = NULL;
@@ -454,7 +456,7 @@ WSLUA_METHOD TextWindow_set(lua_State* L) { /* Sets the text. */
 
     if (tw->expired)
 		WSLUA_ARG_ERROR(TextWindow_set,TEXT,"Expired TextWindow");
-    
+
     if (!text)
 		WSLUA_ARG_ERROR(TextWindow_set,TEXT,"Must be a string");
 
@@ -473,7 +475,7 @@ WSLUA_METHOD TextWindow_append(lua_State* L) { /* Appends text */
 
     if (tw->expired)
 		WSLUA_ARG_ERROR(TextWindow_set,TEXT,"Expired TextWindow");
-    
+
 	if (!text)
 		WSLUA_ARG_ERROR(TextWindow_append,TEXT,"Must be a string");
 
@@ -492,7 +494,7 @@ WSLUA_METHOD TextWindow_prepend(lua_State* L) { /* Prepends text */
 
     if (tw->expired)
 		WSLUA_ARG_ERROR(TextWindow_set,TEXT,"Expired TextWindow");
-    
+
  	if (!text)
 		WSLUA_ARG_ERROR(TextWindow_prepend,TEXT,"Must be a string");
 
@@ -509,7 +511,7 @@ WSLUA_METHOD TextWindow_clear(lua_State* L) { /* Erases all text in the window.
 
     if (tw->expired)
 		WSLUA_ARG_ERROR(TextWindow_set,TEXT,"Expired TextWindow");
-    
+
     ops->clear_text(tw->ws_tw);
 
 	WSLUA_RETURN(1); /* The TextWindow object. */
@@ -524,7 +526,7 @@ WSLUA_METHOD TextWindow_get_text(lua_State* L) { /* Get the text of the window *
 
     if (tw->expired)
 		WSLUA_ARG_ERROR(TextWindow_set,TEXT,"Expired TextWindow");
-    
+
 	text = ops->get_text(tw->ws_tw);
 
     lua_pushstring(L,text);
@@ -536,7 +538,7 @@ static int TextWindow__gc(lua_State* L) {
 
 	if (!tw)
 		return 0;
-	
+
 	if (!tw->expired) {
 		tw->expired = TRUE;
 		ops->destroy_text_window(tw->ws_tw);
@@ -544,7 +546,7 @@ static int TextWindow__gc(lua_State* L) {
 		g_free(tw);
 	}
 
-	
+
     return 0;
 }
 
@@ -559,7 +561,7 @@ WSLUA_METHOD TextWindow_set_editable(lua_State* L) { /* Make this window editabl
 
     if (tw->expired)
 		WSLUA_ARG_ERROR(TextWindow_set,TEXT,"Expired TextWindow");
-    
+
 	if (ops->set_editable)
 		ops->set_editable(tw->ws_tw,editable);
 
@@ -613,7 +615,7 @@ WSLUA_METHOD TextWindow_add_button(lua_State* L) {
 
     if (tw->expired)
 		WSLUA_ARG_ERROR(TextWindow_set,TEXT,"Expired TextWindow");
-    
+
 	if (! lua_isfunction(L,WSLUA_ARG_TextWindow_add_button_FUNCTION) )
 		WSLUA_ARG_ERROR(TextWindow_add_button,FUNCTION,"must be a function");
 
