@@ -2,7 +2,7 @@
  * camel message counter for Wireshark
  * Copyright 2006 Florent Drouin (based on h225_counter.c from Lars Roland)
  *
- * $Id: camel_counter.c 26641 2008-10-31 09:53:56Z stig $
+ * $Id: camel_counter.c 34017 2010-08-30 16:10:01Z morriss $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -42,9 +42,7 @@
 #include <epan/asn1.h>
 #include <epan/camel-persistentdata.h>
 
-#include "../globals.h"
 #include "../stat_menu.h"
-#include "../register.h"
 #include "../timestats.h"
 #include "../simple_dialog.h"
 
@@ -72,7 +70,7 @@ struct camelcounter_t {
   GtkWidget *vbox;
   char *filter;
   GtkWidget *scrolled_window;
-  GtkCList *table;
+  GtkTreeView *table;
   guint32 camel_msg[camel_MAX_NUM_OPR_CODES];
 };
 
@@ -108,26 +106,27 @@ static void gtk_camelcounter_draw(void *phs)
 {
   struct camelcounter_t *p_counter=(struct camelcounter_t *)phs;
   int i;
-  char *str[2];
+  char str[256];
+  GtkListStore *store;
+  GtkTreeIter iter;
 
-  for(i=0;i<2;i++) {
-    str[i]=g_malloc(sizeof(char[256]));
-  }
   /* Now print Message and Reason Counter Table */
   /* clear list before printing */
-  gtk_clist_clear(p_counter->table);
+  store = GTK_LIST_STORE(gtk_tree_view_get_model(p_counter->table));
+  gtk_list_store_clear(store);
 
   for(i=0;i<camel_MAX_NUM_OPR_CODES;i++) {
     /* Message counter */
     if(p_counter->camel_msg[i]!=0) {
-      g_snprintf(str[0], sizeof(char[256]),
-		 "Request %s", val_to_str(i,camel_opr_code_strings,"Unknown message "));
-      g_snprintf(str[1], sizeof(char[256]),
-		 "%d", p_counter->camel_msg[i]);
-      gtk_clist_append(p_counter->table, str);
+      g_snprintf(str, 256, "Request %s", val_to_str(i,camel_opr_code_strings,"Unknown message "));
+
+      gtk_list_store_append(store, &iter);
+      gtk_list_store_set(store, &iter,
+				   0, str,
+				   1, p_counter->camel_msg[i],
+				   -1);
     }
   } /* Message Type */
-  gtk_widget_show(GTK_WIDGET(p_counter->table));
 }
 
 static void win_destroy_cb(GtkWindow *win _U_, gpointer data)
@@ -145,15 +144,15 @@ static void win_destroy_cb(GtkWindow *win _U_, gpointer data)
   g_free(hs);
 }
 
-static const gchar *titles[]={
-  "Message Type or Reason",
-  "Count" };
+static const stat_column titles[]={
+  {G_TYPE_STRING, LEFT, "Message Type or Reason"},
+  {G_TYPE_UINT, RIGHT, "Count" }
+};
 
 static void gtk_camelcounter_init(const char *optarg, void *userdata _U_)
 {
   struct camelcounter_t *p_camelcounter;
   const char *filter=NULL;
-  const char *emptyfilter="";
   GString *error_string;
   GtkWidget *bbox;
   GtkWidget *close_bt;
@@ -169,8 +168,11 @@ static void gtk_camelcounter_init(const char *optarg, void *userdata _U_)
 
   gtk_camelcounter_reset(p_camelcounter);
 
-  p_camelcounter->win=window_new(GTK_WINDOW_TOPLEVEL, "Wireshark: CAMEL counters");
-  gtk_window_set_default_size(GTK_WINDOW(p_camelcounter->win), 500, 300);
+  /* transient_for top_level */
+  p_camelcounter->win=dlg_window_new("Wireshark: CAMEL counters");
+  gtk_window_set_destroy_with_parent (GTK_WINDOW(p_camelcounter->win), TRUE);
+
+	gtk_window_set_default_size(GTK_WINDOW(p_camelcounter->win), 500, 300);
 
   p_camelcounter->vbox=gtk_vbox_new(FALSE, 3);
   gtk_container_set_border_width(GTK_CONTAINER(p_camelcounter->vbox), 12);
@@ -182,17 +184,10 @@ static void gtk_camelcounter_init(const char *optarg, void *userdata _U_)
 
   p_camelcounter->table = create_stat_table(p_camelcounter->scrolled_window, p_camelcounter->vbox, 2, titles);
 
-  if (filter) {
-    error_string=register_tap_listener("CAMEL", p_camelcounter, filter,
+  error_string=register_tap_listener("CAMEL", p_camelcounter, filter, 0,
 				       gtk_camelcounter_reset,
 				       gtk_camelcounter_packet,
 				       gtk_camelcounter_draw);
-  } else {
-    error_string=register_tap_listener("CAMEL", p_camelcounter, emptyfilter,
-				       gtk_camelcounter_reset,
-				       gtk_camelcounter_packet,
-				       gtk_camelcounter_draw);
-  }
 
   if(error_string){
     simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "%s", error_string->str);
@@ -214,7 +209,7 @@ static void gtk_camelcounter_init(const char *optarg, void *userdata _U_)
   gtk_widget_show_all(p_camelcounter->win);
   window_present(p_camelcounter->win);
 
-  cf_retap_packets(&cfile, FALSE);
+  cf_retap_packets(&cfile);
   gdk_window_raise(p_camelcounter->win->window);
 }
 
@@ -228,7 +223,7 @@ static tap_dfilter_dlg camel_counter_dlg = {
 void  /* Next line mandatory */
 register_tap_listener_gtk_camelcounter(void)
 {
-  register_dfilter_stat(&camel_counter_dlg, "GSM/CAMEL",
+  register_dfilter_stat(&camel_counter_dlg, "_GSM/CAMEL",
   			REGISTER_STAT_GROUP_TELEPHONY);
 
 }
