@@ -1,7 +1,7 @@
 /* follow_stream.c
  * Common routines for following data streams
  *
- * $Id: follow_stream.c 30323 2009-10-05 07:25:33Z stig $
+ * $Id: follow_stream.c 35849 2011-02-07 18:41:38Z sfisher $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -59,7 +59,7 @@
 
 #ifdef _WIN32
 #include "../tempfile.h"
-#include "gtk/print_win32.h"
+#include "win32/print_win32.h"
 #endif
 
 /* static variable declarations to speed up the performance
@@ -79,19 +79,19 @@ GList *follow_infos = NULL;
 
 static frs_return_t
 follow_read_stream(follow_info_t *follow_info,
-		   gboolean (*print_line)(char *, size_t, gboolean, void *),
+		   gboolean (*print_line_fcn_p)(char *, size_t, gboolean, void *),
 		   void *arg)
 {
 	switch(follow_info->follow_type) {
 
 	case FOLLOW_TCP :
-		return follow_read_tcp_stream(follow_info, print_line, arg);
+		return follow_read_tcp_stream(follow_info, print_line_fcn_p, arg);
 
 	case FOLLOW_UDP :
-		return follow_read_udp_stream(follow_info, print_line, arg);
+		return follow_read_udp_stream(follow_info, print_line_fcn_p, arg);
 
 	case FOLLOW_SSL :
-		return follow_read_ssl_stream(follow_info, print_line, arg);
+		return follow_read_ssl_stream(follow_info, print_line_fcn_p, arg);
 
 	default :
 		g_assert_not_reached();
@@ -192,7 +192,7 @@ follow_charset_toggle_cb(GtkWidget * w _U_, gpointer data)
 	 * delivered.  We only want to reprocess the display once,
 	 * so we do it only when the button goes on.
 	 */
-	if (GTK_TOGGLE_BUTTON(w)->active) {
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))) {
 		if (w == follow_info->ebcdic_bt)
 			follow_info->show_type = SHOW_EBCDIC;
 		else if (w == follow_info->hexdump_bt)
@@ -719,7 +719,7 @@ follow_stream(gchar *title, follow_info_t *follow_info,
 {
 	GtkWidget	*streamwindow, *vbox, *txt_scrollw, *text;
 	GtkWidget	*hbox, *bbox, *button, *radio_bt;
-	GtkWidget	*stream_fr, *stream_vb;
+	GtkWidget	*stream_fr, *stream_vb, *direction_hbox;
 	GtkWidget	*stream_cmb;
 	GtkTooltips	*tooltips;
 	follow_stats_t stats;
@@ -769,32 +769,9 @@ follow_stream(gchar *title, follow_info_t *follow_info,
 	gtk_container_add(GTK_CONTAINER(txt_scrollw), text);
 	follow_info->text = text;
 
-	/* stream hbox */
-	hbox = gtk_hbox_new(FALSE, 1);
-	gtk_box_pack_start(GTK_BOX(stream_vb), hbox, FALSE, FALSE, 0);
-
-	/* Create Find Button */
-	button = gtk_button_new_from_stock(GTK_STOCK_FIND);
-	g_signal_connect(button, "clicked", G_CALLBACK(follow_find_cb), follow_info);
-	gtk_tooltips_set_tip (tooltips, button, "Find text in the displayed content", NULL);
-	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
-
-	/* Create Save As Button */
-	button = gtk_button_new_from_stock(GTK_STOCK_SAVE_AS);
-	g_signal_connect(button, "clicked", G_CALLBACK(follow_save_as_cmd_cb), follow_info);
-	gtk_tooltips_set_tip (tooltips, button, "Save the content as currently displayed", NULL);
-	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
-
-	/* Create Print Button */
-        button = gtk_button_new_from_stock(GTK_STOCK_PRINT);
-        g_signal_connect(button, "clicked", G_CALLBACK(follow_print_stream), follow_info);
-        gtk_tooltips_set_tip(tooltips, button, "Print the content as currently displayed", NULL);
-        gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
-
-	/* Stream to show */
-	follow_stats(&stats);
-
-	follow_info->is_ipv6 = stats.is_ipv6;
+	/* direction hbox */
+	direction_hbox = gtk_hbox_new(FALSE, 1);
+	gtk_box_pack_start(GTK_BOX(stream_vb), direction_hbox, FALSE, FALSE, 0);
 
 	stream_cmb = gtk_combo_box_new_text();
 
@@ -817,14 +794,41 @@ follow_stream(gchar *title, follow_info_t *follow_info,
 
 	gtk_tooltips_set_tip (tooltips, stream_cmb,
 			      "Select the stream direction to display", NULL);
-	gtk_box_pack_start(GTK_BOX(hbox), stream_cmb, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(direction_hbox), stream_cmb, TRUE, TRUE, 0);
+
+	/* stream hbox */
+	hbox = gtk_hbox_new(FALSE, 1);
+	gtk_box_pack_start(GTK_BOX(stream_vb), hbox, FALSE, FALSE, 0);
+
+	/* Create Find Button */
+	button = gtk_button_new_from_stock(GTK_STOCK_FIND);
+	g_signal_connect(button, "clicked", G_CALLBACK(follow_find_cb), follow_info);
+	gtk_tooltips_set_tip (tooltips, button, "Find text in the displayed content", NULL);
+	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+
+	/* Create Save As Button */
+	button = gtk_button_new_from_stock(GTK_STOCK_SAVE_AS);
+	g_signal_connect(button, "clicked", G_CALLBACK(follow_save_as_cmd_cb), follow_info);
+	gtk_tooltips_set_tip (tooltips, button, "Save the content as currently displayed", NULL);
+	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+
+	/* Create Print Button */
+        button = gtk_button_new_from_stock(GTK_STOCK_PRINT);
+        g_signal_connect(button, "clicked", G_CALLBACK(follow_print_stream), follow_info);
+        gtk_tooltips_set_tip(tooltips, button, "Print the content as currently displayed", NULL);
+        gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+
+	/* Stream to show */
+	follow_stats(&stats);
+
+	follow_info->is_ipv6 = stats.is_ipv6;
 
 	/* ASCII radio button */
 	radio_bt = gtk_radio_button_new_with_label(NULL, "ASCII");
 	gtk_tooltips_set_tip (tooltips, radio_bt, "Stream data output in \"ASCII\" format", NULL);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_bt),
 		IS_SHOW_TYPE(SHOW_ASCII));
-	gtk_box_pack_start(GTK_BOX(hbox), radio_bt, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), radio_bt, TRUE, TRUE, 0);
 	g_signal_connect(radio_bt, "toggled", G_CALLBACK(follow_charset_toggle_cb),
                        follow_info);
 	follow_info->ascii_bt = radio_bt;
@@ -836,7 +840,7 @@ follow_stream(gchar *title, follow_info_t *follow_info,
 	gtk_tooltips_set_tip (tooltips, radio_bt, "Stream data output in \"EBCDIC\" format", NULL);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_bt),
 		IS_SHOW_TYPE(SHOW_EBCDIC));
-	gtk_box_pack_start(GTK_BOX(hbox), radio_bt, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), radio_bt, TRUE, TRUE, 0);
 	g_signal_connect(radio_bt, "toggled", G_CALLBACK(follow_charset_toggle_cb),
                        follow_info);
 	follow_info->ebcdic_bt = radio_bt;
@@ -848,7 +852,7 @@ follow_stream(gchar *title, follow_info_t *follow_info,
 	gtk_tooltips_set_tip (tooltips, radio_bt, "Stream data output in \"Hexdump\" format", NULL);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_bt),
 		IS_SHOW_TYPE(SHOW_HEXDUMP));
-	gtk_box_pack_start(GTK_BOX(hbox), radio_bt, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), radio_bt, TRUE, TRUE, 0);
 	g_signal_connect(radio_bt, "toggled", G_CALLBACK(follow_charset_toggle_cb),
                        follow_info);
 	follow_info->hexdump_bt = radio_bt;
@@ -860,7 +864,7 @@ follow_stream(gchar *title, follow_info_t *follow_info,
 	gtk_tooltips_set_tip (tooltips, radio_bt, "Stream data output in \"C Array\" format", NULL);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_bt),
 		IS_SHOW_TYPE(SHOW_CARRAY));
-	gtk_box_pack_start(GTK_BOX(hbox), radio_bt, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), radio_bt, TRUE, TRUE, 0);
 	g_signal_connect(radio_bt, "toggled", G_CALLBACK(follow_charset_toggle_cb),
                        follow_info);
 	follow_info->carray_bt = radio_bt;
@@ -872,7 +876,7 @@ follow_stream(gchar *title, follow_info_t *follow_info,
 	gtk_tooltips_set_tip (tooltips, radio_bt, "Stream data output in \"Raw\" (binary) format. As this contains non printable characters, the screen output will be in ASCII format", NULL);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_bt),
 		IS_SHOW_TYPE(SHOW_RAW));
-	gtk_box_pack_start(GTK_BOX(hbox), radio_bt, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), radio_bt, TRUE, TRUE, 0);
 	g_signal_connect(radio_bt, "toggled", G_CALLBACK(follow_charset_toggle_cb),
                        follow_info);
 	follow_info->raw_bt = radio_bt;
@@ -976,21 +980,21 @@ follow_destroy_cb(GtkWidget *w, gpointer data _U_)
 
 frs_return_t
 follow_show(follow_info_t *follow_info,
-	    gboolean (*print_line)(char *, size_t, gboolean, void *),
+	    gboolean (*print_line_fcn_p)(char *, size_t, gboolean, void *),
 	    char *buffer, size_t nchars, gboolean is_server, void *arg,
 	    guint32 *global_pos, guint32 *server_packet_count,
 	    guint32 *client_packet_count)
 {
 	gchar initbuf[256];
 	guint32 current_pos;
-	static const gchar hexchars[16] = "0123456789abcdef";
+	static const gchar hexchars[16] = {'0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f'};
 
 	switch (follow_info->show_type) {
 
 	case SHOW_EBCDIC:
 		/* If our native arch is ASCII, call: */
 		EBCDIC_to_ASCII(buffer, (guint) nchars);
-		if (!(*print_line) (buffer, nchars, is_server, arg))
+		if (!(*print_line_fcn_p) (buffer, nchars, is_server, arg))
 			return FRS_PRINT_ERROR;
 		break;
 
@@ -998,7 +1002,7 @@ follow_show(follow_info_t *follow_info,
                 /* If our native arch is EBCDIC, call:
                  * ASCII_TO_EBCDIC(buffer, nchars);
                  */
-                if (!(*print_line) (buffer, nchars, is_server, arg))
+                if (!(*print_line_fcn_p) (buffer, nchars, is_server, arg))
 			return FRS_PRINT_ERROR;
                 break;
 
@@ -1006,7 +1010,7 @@ follow_show(follow_info_t *follow_info,
                 /* Don't translate, no matter what the native arch
                  * is.
                  */
-                if (!(*print_line) (buffer, nchars, is_server, arg))
+                if (!(*print_line_fcn_p) (buffer, nchars, is_server, arg))
 			return FRS_PRINT_ERROR;
                 break;
 
@@ -1053,7 +1057,7 @@ follow_show(follow_info_t *follow_info,
 			(*global_pos) += i;
 			*cur++ = '\n';
 			*cur = 0;
-			if (!(*print_line) (hexbuf, strlen(hexbuf), is_server, arg))
+			if (!(*print_line_fcn_p) (hexbuf, strlen(hexbuf), is_server, arg))
 				return FRS_PRINT_ERROR;
                 }
                 break;
@@ -1063,7 +1067,7 @@ follow_show(follow_info_t *follow_info,
                 g_snprintf(initbuf, sizeof(initbuf), "char peer%d_%d[] = {\n",
 			   is_server ? 1 : 0,
 			   is_server ? (*server_packet_count)++ : (*client_packet_count)++);
-                if (!(*print_line) (initbuf, strlen(initbuf), is_server, arg))
+                if (!(*print_line_fcn_p) (initbuf, strlen(initbuf), is_server, arg))
 			return FRS_PRINT_ERROR;
 
                 while (current_pos < nchars) {
@@ -1097,7 +1101,7 @@ follow_show(follow_info_t *follow_info,
 			(*global_pos) += i;
 			hexbuf[cur++] = '\n';
 			hexbuf[cur] = 0;
-			if (!(*print_line) (hexbuf, strlen(hexbuf), is_server, arg))
+			if (!(*print_line_fcn_p) (hexbuf, strlen(hexbuf), is_server, arg))
 				return FRS_PRINT_ERROR;
                 }
                 break;
