@@ -1,6 +1,6 @@
 /* main_welcome.c
  *
- * $Id: main_welcome.c 29949 2009-09-16 20:29:00Z gerald $
+ * $Id: main_welcome.c 37398 2011-05-25 21:19:56Z gerald $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -35,10 +35,12 @@
 #include <epan/prefs.h>
 
 #include "../color.h"
+#ifdef HAVE_LIBPCAP
 #include "capture.h"
 #include "capture-pcap-util.h"
 #include "capture_opts.h"
 #include "capture_ui_utils.h"
+#endif
 #include "simple_dialog.h"
 #include <wsutil/file_util.h>
 
@@ -49,13 +51,16 @@
 #include "gtk/main.h"
 #include "gtk/menus.h"
 #include "gtk/main_welcome.h"
+#include "gtk/help_dlg.h"
+#include "gtk/capture_file_dlg.h"
+#include "gtk/stock_icons.h"
+#include "gtk/utf8_entities.h"
+#ifdef HAVE_LIBPCAP
 #include "gtk/capture_dlg.h"
 #include "gtk/capture_if_dlg.h"
-#include "gtk/capture_file_dlg.h"
-#include "gtk/help_dlg.h"
-#include "gtk/stock_icons.h"
 #include "gtk/capture_globals.h"
-#include "../image/wssplash-dev.xpm"
+#endif
+#include "../image/wssplash.xpm"
 #include "../version_info.h"
 
 #ifdef _WIN32
@@ -74,11 +79,13 @@ extern gint if_list_comparator_alph (const void *first_arg, const void *second_a
 
 static GtkWidget *welcome_hb = NULL;
 static GtkWidget *header_lb = NULL;
-static GdkColor header_bar_bg;
-static GdkColor topic_header_bg;
-static GdkColor topic_content_bg;
+/* Foreground colors are set using Pango markup */
+static GdkColor welcome_bg = { 0, 0xe6e6, 0xe6e6, 0xe6e6 };
+static GdkColor header_bar_bg = { 0, 0x1818, 0x5c5c, 0xcaca };
+static GdkColor topic_header_bg = { 0, 0x0101, 0x3939, 0xbebe };
+static GdkColor topic_content_bg = { 0, 0xffff, 0xffff, 0xffff };
 static GdkColor topic_item_idle_bg;
-static GdkColor topic_item_entered_bg;
+static GdkColor topic_item_entered_bg = { 0, 0xd3d3, 0xd8d8, 0xdada };
 
 static GtkWidget *welcome_file_panel_vb = NULL;
 #ifdef HAVE_LIBPCAP
@@ -115,6 +122,7 @@ scroll_box_dynamic_new(GtkBox *child_box, guint max_childs, guint scrollw_y_size
     return parent_box;
 }
 
+
 static GtkWidget *
 scroll_box_dynamic_add(GtkWidget *parent_box)
 {
@@ -125,7 +133,6 @@ scroll_box_dynamic_add(GtkWidget *parent_box)
     guint scrollw_y_size;
     GList *childs;
 
-
     child_box = g_object_get_data(G_OBJECT(parent_box), SCROLL_BOX_CHILD_BOX);
     max_cnt = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(parent_box), SCROLL_BOX_MAX_CHILDS));
 
@@ -140,7 +147,7 @@ scroll_box_dynamic_add(GtkWidget *parent_box)
         /* XXX - there's no way to get rid of the shadow frame - except for creating an own widget :-( */
         scrollw = scrolled_window_new(NULL, NULL);
         scrollw_y_size = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(parent_box), SCROLL_BOX_SCROLLW_Y_SIZE));
-	    gtk_widget_set_size_request(scrollw, -1, scrollw_y_size);
+        gtk_widget_set_size_request(scrollw, -1, scrollw_y_size);
 
         g_object_set_data(G_OBJECT(parent_box), SCROLL_BOX_SCROLLW, scrollw);
         gtk_box_pack_start(GTK_BOX(parent_box), scrollw, TRUE, TRUE, 0);
@@ -156,6 +163,7 @@ scroll_box_dynamic_add(GtkWidget *parent_box)
     return child_box;
 }
 
+
 static GtkWidget *
 scroll_box_dynamic_reset(GtkWidget *parent_box)
 {
@@ -177,8 +185,6 @@ scroll_box_dynamic_reset(GtkWidget *parent_box)
 }
 
 
-
-
 /* mouse entered this widget - change background color */
 static gboolean
 welcome_item_enter_cb(GtkWidget *eb, GdkEventCrossing *event _U_, gpointer user_data _U_)
@@ -188,9 +194,10 @@ welcome_item_enter_cb(GtkWidget *eb, GdkEventCrossing *event _U_, gpointer user_
     return FALSE;
 }
 
+
 /* mouse has left this widget - change background color  */
 static gboolean
-welcome_item_leave_cb(GtkWidget *eb, GdkEvent *event _U_, gpointer user_data _U_)
+welcome_item_leave_cb(GtkWidget *eb, GdkEventCrossing *event _U_, gpointer user_data _U_)
 {
     gtk_widget_modify_bg(eb, GTK_STATE_NORMAL, &topic_item_idle_bg);
 
@@ -198,11 +205,15 @@ welcome_item_leave_cb(GtkWidget *eb, GdkEvent *event _U_, gpointer user_data _U_
 }
 
 
+typedef gboolean (*welcome_button_callback_t)  (GtkWidget      *widget,
+                                                GdkEventButton *event,
+                                                gpointer        user_data);
+
 /* create a "button widget" */
 static GtkWidget *
 welcome_button(const gchar *stock_item,
-               const gchar * title, const gchar * subtitle, const gchar *tooltip,
-			   GtkSignalFunc callback, void *callback_data)
+               const gchar *title, const gchar *subtitle, const gchar *tooltip,
+               welcome_button_callback_t welcome_button_callback, gpointer welcome_button_callback_data)
 {
     GtkWidget *eb, *w, *item_hb, *text_vb;
     gchar *formatted_text;
@@ -223,7 +234,7 @@ welcome_button(const gchar *stock_item,
 
     g_signal_connect(eb, "enter-notify-event", G_CALLBACK(welcome_item_enter_cb), NULL);
     g_signal_connect(eb, "leave-notify-event", G_CALLBACK(welcome_item_leave_cb), NULL);
-    g_signal_connect(eb, "button-press-event", G_CALLBACK(callback), callback_data);
+    g_signal_connect(eb, "button-release-event", G_CALLBACK(welcome_button_callback), welcome_button_callback_data);
 
     /* icon */
     w = gtk_image_new_from_stock(stock_item, GTK_ICON_SIZE_LARGE_TOOLBAR);
@@ -252,27 +263,41 @@ welcome_button(const gchar *stock_item,
     return eb;
 }
 
-static void
+
+/* Hack to handle welcome-button "button-release-event" callback   */
+/*  1. Dispatch to desired actual callback                         */
+/*  2. Return TRUE for the event callback.                         */
+/* user_data: actual (no arg) callback fcn to be invoked.          */
+static gboolean
+welcome_button_callback_helper(GtkWidget *w, GdkEventButton *event _U_, gpointer user_data)
+{
+    void (*funct)(GtkWidget *, gpointer) = user_data;
+    (*funct)(w, NULL);
+    return TRUE;
+}
+
+
+void
 welcome_header_set_message(gchar *msg) {
     GString *message;
     time_t secs = time(NULL);
     struct tm *now = localtime(&secs);
-    
-    message = g_string_new("<span weight=\"bold\" size=\"x-large\" foreground=\"black\">");
-    
+
+    message = g_string_new("<span weight=\"bold\" size=\"x-large\" foreground=\"white\">");
+
     if (msg) {
-	g_string_append(message, msg);
+        g_string_append(message, msg);
     } else { /* Use our default header */
-	if ((now->tm_mon == 3 && now->tm_mday == 1) || (now->tm_mon == 6 && now->tm_mday == 14)) {
-	    g_string_append(message, "Sniffing the glue that holds the Internet together");
-	} else {
-	    g_string_append(message, prefs.gui_start_title);
-	}
-    
-	if (prefs.gui_version_in_start_page) {
-	    g_string_append_printf(message, "</span>\n<span size=\"large\" foreground=\"black\">Version " VERSION "%s",
-				   wireshark_svnversion);
-	}
+        if ((now->tm_mon == 3 && now->tm_mday == 1) || (now->tm_mon == 6 && now->tm_mday == 14)) {
+            g_string_append(message, "Sniffing the glue that holds the Internet together");
+        } else {
+            g_string_append(message, prefs.gui_start_title);
+        }
+
+        if (prefs.gui_version_in_start_page) {
+            g_string_append_printf(message, "</span>\n<span size=\"large\" foreground=\"white\">Version " VERSION "%s",
+                                   wireshark_svnversion);
+        }
     }
 
     g_string_append(message, "</span>");
@@ -281,6 +306,7 @@ welcome_header_set_message(gchar *msg) {
     g_string_free(message, TRUE);
 }
 
+
 /* create the banner "above our heads" */
 static GtkWidget *
 welcome_header_new(void)
@@ -313,37 +339,40 @@ welcome_header_new(void)
     return eb;
 }
 
+
 void
-welcome_header_push_msg(gchar *msg) {
+welcome_header_push_msg(const gchar *msg) {
     gchar *msg_copy = g_strdup(msg);
-    
+
     status_messages = g_slist_append(status_messages, msg_copy);
-    
+
     welcome_header_set_message(msg_copy);
-    
+
     gtk_widget_hide(welcome_hb);
 }
 
+
 void
 welcome_header_pop_msg(void) {
-    gchar *msg = NULL;    
+    gchar *msg = NULL;
 
     if (status_messages) {
-	g_free(status_messages->data);
+        g_free(status_messages->data);
         status_messages = g_slist_delete_link(status_messages, status_messages);
     }
-    
+
     if (status_messages) {
-	msg = status_messages->data;
+        msg = status_messages->data;
     }
 
     welcome_header_set_message(msg);
-    
+
     if (!status_messages) {
         gtk_widget_show(welcome_hb);
     }
 }
 
+
 /* create a "topic header widget" */
 static GtkWidget *
 welcome_topic_header_new(const char *header)
@@ -354,7 +383,7 @@ welcome_topic_header_new(const char *header)
 
 
     w = gtk_label_new(header);
-    formatted_message = g_strdup_printf("<span weight=\"bold\" size=\"x-large\" foreground=\"black\">%s</span>", header);
+    formatted_message = g_strdup_printf("<span weight=\"bold\" size=\"x-large\" foreground=\"white\">%s</span>", header);
     gtk_label_set_markup(GTK_LABEL(w), formatted_message);
     g_free(formatted_message);
 
@@ -398,7 +427,7 @@ welcome_topic_new(const char *header, GtkWidget **to_fill)
 
 /* a file link was pressed */
 static gboolean
-welcome_filename_link_press_cb(GtkWidget *widget _U_, GdkEvent *event _U_, gpointer data)
+welcome_filename_link_press_cb(GtkWidget *widget _U_, GdkEventButton *event _U_, gpointer data)
 {
     menu_open_filename(data);
 
@@ -410,13 +439,15 @@ welcome_filename_link_press_cb(GtkWidget *widget _U_, GdkEvent *event _U_, gpoin
 static GtkWidget *
 welcome_filename_link_new(const gchar *filename, GtkWidget **label)
 {
-    GtkWidget	*w;
-    GtkWidget	*eb;
-    GString	*str;
-    gchar	*str_escaped;
-    const unsigned int max = 60;
-    int		err;
-    struct stat stat_buf;
+    GtkWidget   *w;
+    GtkWidget   *eb;
+    GString     *str;
+    gchar       *str_escaped;
+    glong        uni_len;
+    gsize        uni_start, uni_end;
+    const glong  max = 60;
+    int          err;
+    ws_statb64   stat_buf;
     GtkTooltips *tooltips;
 
 
@@ -424,11 +455,14 @@ welcome_filename_link_new(const gchar *filename, GtkWidget **label)
 
     /* filename */
     str = g_string_new(filename);
+    uni_len = g_utf8_strlen(str->str, str->len);
 
     /* cut max filename length */
-    if( (str->len > max) && (str->len-(max) > 5) ) {
-        g_string_erase(str, 20, str->len-(max+5));
-        g_string_insert(str, 20, " ... ");
+    if (uni_len > max) {
+        uni_start = g_utf8_offset_to_pointer(str->str, 20) - str->str;
+        uni_end = g_utf8_offset_to_pointer(str->str, uni_len - max) - str->str;
+        g_string_erase(str, uni_start, uni_end);
+        g_string_insert(str, uni_start, " " UTF8_HORIZONTAL_ELLIPSIS " ");
     }
 
     /* escape the possibly shortened filename before adding pango language */
@@ -441,11 +475,11 @@ welcome_filename_link_new(const gchar *filename, GtkWidget **label)
      * Add file size. We use binary prefixes instead of IEC because that's what
      * most OSes use.
      */
-    err = ws_stat(filename, &stat_buf);
+    err = ws_stat64(filename, &stat_buf);
     if(err == 0) {
         if (stat_buf.st_size/1024/1024/1024 > 10) {
             g_string_append_printf(str, " (%" G_GINT64_MODIFIER "d GB)", (gint64) (stat_buf.st_size/1024/1024/1024));
-	} else if (stat_buf.st_size/1024/1024 > 10) {
+        } else if (stat_buf.st_size/1024/1024 > 10) {
             g_string_append_printf(str, " (%" G_GINT64_MODIFIER "d MB)", (gint64) (stat_buf.st_size/1024/1024));
         } else if (stat_buf.st_size/1024 > 10) {
             g_string_append_printf(str, " (%" G_GINT64_MODIFIER "d KB)", (gint64) (stat_buf.st_size/1024));
@@ -532,8 +566,10 @@ main_welcome_add_recent_capture_files(const char *widget_cf_name)
 #ifdef HAVE_LIBPCAP
 /* user clicked on an interface button */
 static gboolean
-welcome_if_press_cb(GtkWidget *widget _U_, GdkEvent *event _U_, gpointer data)
+welcome_if_press_cb(GtkWidget *widget _U_, GdkEventButton *event _U_, gpointer data)
 {
+    cap_settings_t cap_settings;
+
     g_free(global_capture_opts.iface);
     g_free(global_capture_opts.iface_descr);
 
@@ -541,6 +577,9 @@ welcome_if_press_cb(GtkWidget *widget _U_, GdkEvent *event _U_, gpointer data)
     global_capture_opts.iface_descr = NULL;
     /* XXX - fix this */
     /*global_capture_opts.iface_descr = get_interface_descriptive_name(global_capture_opts.iface);*/
+    cap_settings = capture_get_cap_settings (global_capture_opts.iface);;
+    global_capture_opts.monitor_mode = cap_settings.monitor_mode;
+    global_capture_opts.linktype = cap_settings.linktype;
 
     /* XXX - remove this? */
     if (global_capture_opts.save_file) {
@@ -610,63 +649,69 @@ welcome_if_new(const if_info_t *if_info, const gchar *user_descr, GdkColor *topi
     return eb;
 }
 
-
-/* load the list of interfaces */
-static void
-welcome_if_panel_load(void)
+/* load the sorted list of interfaces */
+static GList *
+welcome_if_panel_get_if_list(void)
 {
-  GtkWidget *child_box;
-  GtkWidget *interface_hb;
+    GList *if_list;
+    int err;
+    gchar *err_str = NULL;
 
-  if_info_t     *if_info;
-  GList         *if_list;
-  int err;
-  gchar         *err_str;
-  int           ifs;
-  GList         *curr;
-  gchar         *user_descr;
+    /* LOAD THE INTERFACES */
+    if_list = capture_interface_list(&err, &err_str);
+    if_list = g_list_sort (if_list, if_list_comparator_alph);
+    if (if_list == NULL && err == CANT_GET_INTERFACE_LIST) {
+        simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "%s", err_str);
+        g_free(err_str);
+    } else if (err_str) {
+        g_free(err_str);
+    }
+    return if_list;
+}
+
+/* list the interfaces */
+static void
+welcome_if_panel_list(GList *if_list)
+{
+    GtkWidget *child_box;
+    GtkWidget *interface_hb;
+
+    if_info_t     *if_info;
+    int           ifs;
+    GList         *curr;
+    gchar         *user_descr;
 
 
-  /* LOAD THE INTERFACES */
-  if_list = capture_interface_list(&err, &err_str);
-  if_list = g_list_sort (if_list, if_list_comparator_alph);
-  if (if_list == NULL && err == CANT_GET_INTERFACE_LIST) {
-    simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "%s", err_str);
-    g_free(err_str);
-    return;
-  }
+    /* List the interfaces */
+    for(ifs = 0; (curr = g_list_nth(if_list, ifs)); ifs++) {
+        /*g_string_assign(if_tool_str, "");*/
+        if_info = curr->data;
 
-  /* List the interfaces */
-  for(ifs = 0; (curr = g_list_nth(if_list, ifs)); ifs++) {
-      /*g_string_assign(if_tool_str, "");*/
-      if_info = curr->data;
+        /* Continue if capture device is hidden */
+        if (prefs_is_capture_device_hidden(if_info->name)) {
+            continue;
+        }
 
-      /* Continue if capture device is hidden */
-      if (prefs_is_capture_device_hidden(if_info->name)) {
-          continue;
-      }
-
-      user_descr = capture_dev_user_descr_find(if_info->name);
-      if (user_descr) {
+        user_descr = capture_dev_user_descr_find(if_info->name);
+        if (user_descr) {
 #ifndef _WIN32
-        gchar *comment = user_descr;
-        user_descr = g_strdup_printf("%s (%s)", comment, if_info->name);
-        g_free (comment);
+            gchar *comment = user_descr;
+            user_descr = g_strdup_printf("%s (%s)", comment, if_info->name);
+            g_free (comment);
 #endif
-        interface_hb = welcome_if_new(if_info, user_descr, &topic_content_bg, g_strdup(if_info->name));
-        g_free (user_descr);
-      } else {
-        interface_hb = welcome_if_new(if_info, NULL, &topic_content_bg, g_strdup(if_info->name));
-      }
+            interface_hb = welcome_if_new(if_info, user_descr, &topic_content_bg, g_strdup(if_info->name));
+            g_free (user_descr);
+        } else {
+            interface_hb = welcome_if_new(if_info, NULL, &topic_content_bg, g_strdup(if_info->name));
+        }
 
-      child_box = scroll_box_dynamic_add(welcome_if_panel_vb);
-      gtk_box_pack_start(GTK_BOX(child_box), interface_hb, FALSE, FALSE, 1);
-  }
-
-  free_interface_list(if_list);
+        child_box = scroll_box_dynamic_add(welcome_if_panel_vb);
+        gtk_box_pack_start(GTK_BOX(child_box), interface_hb, FALSE, FALSE, 1);
+    }
 }
 #endif  /* HAVE_LIBPCAP */
 
+
 /* reload the list of interfaces */
 void
 welcome_if_panel_reload(void)
@@ -678,6 +723,7 @@ welcome_if_panel_reload(void)
 
 
     if(welcome_if_panel_vb) {
+        GList *if_list;
         child_box = scroll_box_dynamic_reset(welcome_if_panel_vb);
         child_list = gtk_container_get_children(GTK_CONTAINER(child_box));
         child_list_item = child_list;
@@ -689,8 +735,11 @@ welcome_if_panel_reload(void)
 
         g_list_free(child_list);
 
-	welcome_if_panel_load();
-	gtk_widget_show_all(welcome_if_panel_vb);
+        if_list = welcome_if_panel_get_if_list();
+        welcome_if_panel_list(if_list);
+        free_interface_list(if_list);
+
+        gtk_widget_show_all(welcome_if_panel_vb);
     }
 #endif  /* HAVE_LIBPCAP */
 }
@@ -701,6 +750,7 @@ GtkWidget *
 welcome_new(void)
 {
     GtkWidget *welcome_scrollw;
+    GtkWidget *welcome_eb;
     GtkWidget *welcome_vb;
     GtkWidget *column_vb;
     GtkWidget *item_hb;
@@ -710,6 +760,7 @@ welcome_new(void)
     GtkWidget *topic_to_fill;
 #ifdef HAVE_LIBPCAP
     GtkWidget *if_child_box;
+    GList *if_list;
 #endif  /* HAVE_LIBPCAP */
     GtkWidget *file_child_box;
     gchar *label_text;
@@ -720,60 +771,32 @@ welcome_new(void)
 #endif
 
     /* prepare colors */
+
+    /* "page" background */
+    get_color(&welcome_bg);
+
     /* header bar background color */
-    header_bar_bg.pixel = 0;
-    header_bar_bg.red = 154 * 255;
-    header_bar_bg.green = 210 * 255;
-    header_bar_bg.blue = 229 * 255;
     get_color(&header_bar_bg);
 
     /* topic header background color */
-    topic_header_bg.pixel = 0;
-    topic_header_bg.red = 24 * 255;
-    topic_header_bg.green = 151 * 255;
-    topic_header_bg.blue = 192 * 255;
     get_color(&topic_header_bg);
 
     /* topic content background color */
-    topic_content_bg.pixel = 0;
-    topic_content_bg.red = 221 * 255;
-    topic_content_bg.green = 226 * 255;
-    topic_content_bg.blue = 228 * 255;
     get_color(&topic_content_bg);
 
-    /* topic item idle background color */
-    /*topic_item_idle_bg.pixel = 0;
-    topic_item_idle_bg.red = 216 * 255;
-    topic_item_idle_bg.green = 221 * 255;
-    topic_item_idle_bg.blue = 223 * 255;
-    get_color(&topic_item_idle_bg);*/
-
     topic_item_idle_bg = topic_content_bg;
 
     /* topic item entered color */
-    topic_item_entered_bg.pixel = 0;
-    topic_item_entered_bg.red = 211 * 255;
-    topic_item_entered_bg.green = 216 * 255;
-    topic_item_entered_bg.blue = 218 * 255;
     get_color(&topic_item_entered_bg);
 
-    /*topic_item_entered_bg.pixel = 0;
-    topic_item_entered_bg.red = 216 * 255;
-    topic_item_entered_bg.green = 221 * 255;
-    topic_item_entered_bg.blue = 223 * 255;
-    get_color(&topic_item_entered_bg);*/
-
-    /*topic_item_entered_bg.pixel = 0;
-    topic_item_entered_bg.red = 154 * 255;
-    topic_item_entered_bg.green = 210 * 255;
-    topic_item_entered_bg.blue = 229 * 255;
-    get_color(&topic_item_entered_bg);*/
-
-
     welcome_scrollw = scrolled_window_new(NULL, NULL);
 
     welcome_vb = gtk_vbox_new(FALSE, 0);
 
+    welcome_eb = gtk_event_box_new();
+    gtk_container_add(GTK_CONTAINER(welcome_eb), welcome_vb);
+    gtk_widget_modify_bg(welcome_eb, GTK_STATE_NORMAL, &welcome_bg);
+
     /* header */
     header = welcome_header_new();
     gtk_box_pack_start(GTK_BOX(welcome_vb), header, FALSE, FALSE, 0);
@@ -786,6 +809,7 @@ welcome_new(void)
 
     /* column capture */
     column_vb = gtk_vbox_new(FALSE, 10);
+    gtk_widget_modify_bg(column_vb, GTK_STATE_NORMAL, &welcome_bg);
     gtk_box_pack_start(GTK_BOX(welcome_hb), column_vb, TRUE, TRUE, 0);
 
     /* capture topic */
@@ -793,65 +817,79 @@ welcome_new(void)
     gtk_box_pack_start(GTK_BOX(column_vb), topic_vb, TRUE, TRUE, 0);
 
 #ifdef HAVE_LIBPCAP
-    item_hb = welcome_button(WIRESHARK_STOCK_CAPTURE_INTERFACES,
-        "Interface List",
-		"Live list of the capture interfaces (counts incoming packets)",
-        "Same as Capture/Interfaces menu or toolbar item",
-        G_CALLBACK(capture_if_cb), NULL);
-    gtk_box_pack_start(GTK_BOX(topic_to_fill), item_hb, FALSE, FALSE, 5);
+    if_list = welcome_if_panel_get_if_list();
+    if (g_list_length(if_list) > 0) {
 
-    label_text =  g_strdup("<span foreground=\"black\">Start capture on interface:</span>");
-    w = gtk_label_new(label_text);
-    gtk_label_set_markup(GTK_LABEL(w), label_text);
-    g_free (label_text);
-    gtk_misc_set_alignment (GTK_MISC(w), 0.0f, 0.0f);
-    gtk_box_pack_start(GTK_BOX(topic_to_fill), w, FALSE, FALSE, 5);
+        item_hb = welcome_button(WIRESHARK_STOCK_CAPTURE_INTERFACES,
+            "Interface List",
+            "Live list of the capture interfaces\n(counts incoming packets)",
+            "Same as Capture/Interfaces menu or toolbar item",
+            welcome_button_callback_helper, capture_if_cb);
+        gtk_box_pack_start(GTK_BOX(topic_to_fill), item_hb, FALSE, FALSE, 5);
 
-    if_child_box = gtk_vbox_new(FALSE, 0);
-    /* 8 capture interfaces or 150 pixels height is about the size */
-    /* that still fits on a screen of about 1000*700 */
-    welcome_if_panel_vb = scroll_box_dynamic_new(GTK_BOX(if_child_box), 8, 150);
-    gtk_box_pack_start(GTK_BOX(topic_to_fill), welcome_if_panel_vb, FALSE, FALSE, 0);
-    welcome_if_panel_load();
+        label_text =  g_strdup("<span foreground=\"black\">Start capture on interface:</span>");
+        w = gtk_label_new(label_text);
+        gtk_label_set_markup(GTK_LABEL(w), label_text);
+        g_free (label_text);
+        gtk_misc_set_alignment (GTK_MISC(w), 0.0f, 0.0f);
+        gtk_box_pack_start(GTK_BOX(topic_to_fill), w, FALSE, FALSE, 5);
 
-    item_hb = welcome_button(WIRESHARK_STOCK_CAPTURE_OPTIONS,
-        "Capture Options",
-		"Start a capture with detailed options",
-        "Same as Capture/Options menu or toolbar item",
-        G_CALLBACK(capture_prep_cb), NULL);
-    gtk_box_pack_start(GTK_BOX(topic_to_fill), item_hb, FALSE, FALSE, 5);
+        if_child_box = gtk_vbox_new(FALSE, 0);
+        /* 8 capture interfaces or 150 pixels height is about the size */
+        /* that still fits on a screen of about 1000*700 */
+        welcome_if_panel_vb = scroll_box_dynamic_new(GTK_BOX(if_child_box), 8, 150);
+        gtk_box_pack_start(GTK_BOX(topic_to_fill), welcome_if_panel_vb, FALSE, FALSE, 0);
+        welcome_if_panel_list(if_list);
 
+        item_hb = welcome_button(WIRESHARK_STOCK_CAPTURE_OPTIONS,
+            "Capture Options",
+            "Start a capture with detailed options",
+            "Same as Capture/Options menu or toolbar item",
+            welcome_button_callback_helper, capture_prep_cb);
+        gtk_box_pack_start(GTK_BOX(topic_to_fill), item_hb, FALSE, FALSE, 5);
 #ifdef _WIN32
-    /* Check for chimney offloading */
-    reg_ret = RegQueryValueEx(HKEY_LOCAL_MACHINE,
-			      _T("SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters\\EnableTCPChimney"),
-			      NULL, NULL, (LPBYTE) &chimney_enabled, &ce_size);
-    if (reg_ret == ERROR_SUCCESS && chimney_enabled) {	
-	item_hb = welcome_button(WIRESHARK_STOCK_WIKI,
-		    "Offloading Detected",
-	    "TCP Chimney offloading is enabled. You \nmight not capture much data.",
-	    topic_online_url(ONLINEPAGE_CHIMNEY),
-	    G_CALLBACK(topic_menu_cb), GINT_TO_POINTER(ONLINEPAGE_CHIMNEY));
-	gtk_box_pack_start(GTK_BOX(topic_to_fill), item_hb, FALSE, FALSE, 5);
-    }
+        /* Check for chimney offloading */
+        reg_ret = RegQueryValueEx(HKEY_LOCAL_MACHINE,
+                                  _T("SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters\\EnableTCPChimney"),
+                                  NULL, NULL, (LPBYTE) &chimney_enabled, &ce_size);
+        if (reg_ret == ERROR_SUCCESS && chimney_enabled) {
+            item_hb = welcome_button(WIRESHARK_STOCK_WIKI,
+                "Offloading Detected",
+                "TCP Chimney offloading is enabled. You \nmight not capture much data.",
+                topic_online_url(ONLINEPAGE_CHIMNEY),
+                topic_menu_cb, GINT_TO_POINTER(ONLINEPAGE_CHIMNEY));
+            gtk_box_pack_start(GTK_BOX(topic_to_fill), item_hb, FALSE, FALSE, 5);
+        }
 #endif /* _WIN32 */
+    } else {
+        label_text =  g_strdup("No interface can be used for capturing in\n"
+                               "this system with the current configuration.\n\n"
+                               "See Capture Help for details.");
+        w = gtk_label_new(label_text);
+        gtk_label_set_markup(GTK_LABEL(w), label_text);
+        g_free (label_text);
+        gtk_misc_set_alignment (GTK_MISC(w), 0.0f, 0.0f);
+        gtk_box_pack_start(GTK_BOX(topic_to_fill), w, FALSE, FALSE, 5);
+    }
+
+    free_interface_list(if_list);
 
     /* capture help topic */
     topic_vb = welcome_topic_new("Capture Help", &topic_to_fill);
     gtk_box_pack_start(GTK_BOX(column_vb), topic_vb, TRUE, TRUE, 0);
 
     item_hb = welcome_button(WIRESHARK_STOCK_WIKI,
-		"How to Capture",
-		"Step by step to a successful capture setup",
+        "How to Capture",
+        "Step by step to a successful capture setup",
         topic_online_url(ONLINEPAGE_CAPTURE_SETUP),
-        G_CALLBACK(topic_menu_cb), GINT_TO_POINTER(ONLINEPAGE_CAPTURE_SETUP));
+        topic_menu_cb, GINT_TO_POINTER(ONLINEPAGE_CAPTURE_SETUP));
     gtk_box_pack_start(GTK_BOX(topic_to_fill), item_hb, FALSE, FALSE, 5);
 
     item_hb = welcome_button(WIRESHARK_STOCK_WIKI,
-		"Network Media",
-        "Specific information for capturing on: Ethernet, WLAN, ...",
+        "Network Media",
+        "Specific information for capturing on:\nEthernet, WLAN, ...",
         topic_online_url(ONLINEPAGE_NETWORK_MEDIA),
-        G_CALLBACK(topic_menu_cb), GINT_TO_POINTER(ONLINEPAGE_NETWORK_MEDIA));
+        topic_menu_cb, GINT_TO_POINTER(ONLINEPAGE_NETWORK_MEDIA));
     gtk_box_pack_start(GTK_BOX(topic_to_fill), item_hb, FALSE, FALSE, 5);
 #else
     label_text =  g_strdup("<span foreground=\"black\">Capturing is not compiled into this version of Wireshark!</span>");
@@ -873,9 +911,9 @@ welcome_new(void)
 
     item_hb = welcome_button(GTK_STOCK_OPEN,
         "Open",
-		"Open a previously captured file",
+        "Open a previously captured file",
         "Same as File/Open menu or toolbar item",
-        G_CALLBACK(file_open_cmd_cb), NULL);
+        welcome_button_callback_helper, file_open_cmd_cb);
     gtk_box_pack_start(GTK_BOX(topic_to_fill), item_hb, FALSE, FALSE, 5);
 
     /* prepare list of recent files (will be filled in later) */
@@ -894,9 +932,9 @@ welcome_new(void)
 
     item_hb = welcome_button(WIRESHARK_STOCK_WIKI,
         "Sample Captures",
-		"A rich assortment of example capture files on the wiki",
+        "A rich assortment of example capture files on the wiki",
         topic_online_url(ONLINEPAGE_SAMPLE_CAPTURES),
-        G_CALLBACK(topic_menu_cb), GINT_TO_POINTER(ONLINEPAGE_SAMPLE_CAPTURES));
+        topic_menu_cb, GINT_TO_POINTER(ONLINEPAGE_SAMPLE_CAPTURES));
     gtk_box_pack_start(GTK_BOX(topic_to_fill), item_hb, FALSE, FALSE, 5);
 
     /* fill bottom space */
@@ -914,23 +952,33 @@ welcome_new(void)
 
     item_hb = welcome_button(GTK_STOCK_HOME,
         "Website",
-		"Visit the project's website",
+        "Visit the project's website",
         topic_online_url(ONLINEPAGE_HOME),
-        G_CALLBACK(topic_menu_cb), GINT_TO_POINTER(ONLINEPAGE_HOME));
+        topic_menu_cb, GINT_TO_POINTER(ONLINEPAGE_HOME));
     gtk_box_pack_start(GTK_BOX(topic_to_fill), item_hb, FALSE, FALSE, 5);
 
+#ifdef HHC_DIR
     item_hb = welcome_button(GTK_STOCK_HELP,
         "User's Guide",
-		"The User's Guide (local version, if installed)",
+        "The User's Guide "
+        "(local version, if installed)",
         "Locally installed (if installed) otherwise online version",
-        G_CALLBACK(topic_menu_cb), GINT_TO_POINTER(HELP_CONTENT));
+        topic_menu_cb, GINT_TO_POINTER(HELP_CONTENT));
+#else
+    item_hb = welcome_button(GTK_STOCK_HELP,
+        "User's Guide",
+        "The User's Guide "
+        "(online version)",
+        topic_online_url(ONLINEPAGE_USERGUIDE),
+        topic_menu_cb, GINT_TO_POINTER(ONLINEPAGE_USERGUIDE));
+#endif
     gtk_box_pack_start(GTK_BOX(topic_to_fill), item_hb, FALSE, FALSE, 5);
 
     item_hb = welcome_button(WIRESHARK_STOCK_WIKI,
         "Security",
-		"Work with Wireshark as securely as possible",
+        "Work with Wireshark as securely as possible",
         topic_online_url(ONLINEPAGE_SECURITY),
-        G_CALLBACK(topic_menu_cb), GINT_TO_POINTER(ONLINEPAGE_SECURITY));
+        topic_menu_cb, GINT_TO_POINTER(ONLINEPAGE_SECURITY));
     gtk_box_pack_start(GTK_BOX(topic_to_fill), item_hb, FALSE, FALSE, 5);
 
 #if 0
@@ -948,10 +996,10 @@ welcome_new(void)
 
 
     /* the end */
-    gtk_widget_show_all(welcome_vb);
+    gtk_widget_show_all(welcome_eb);
 
     gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(welcome_scrollw),
-                                          welcome_vb);
+                                          welcome_eb);
     gtk_widget_show_all(welcome_scrollw);
 
     return welcome_scrollw;
