@@ -1,7 +1,7 @@
 /* capture_prefs.c
  * Dialog box for capture preferences
  *
- * $Id: prefs_capture.c 30918 2009-11-10 12:05:25Z stig $
+ * $Id: prefs_capture.c 37033 2011-05-09 19:29:40Z gerald $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -33,11 +33,9 @@
 
 #include <epan/prefs.h>
 
-#include "../globals.h"
 #include "../simple_dialog.h"
-#include "../capture-pcap-util.h"
+#include "../capture_ifinfo.h"
 #include "../capture_ui_utils.h"
-#include "../capture.h"
 
 #include "gtk/prefs_capture.h"
 #include "gtk/prefs_dlg.h"
@@ -62,12 +60,15 @@
 #define IFOPTS_CALLER_PTR_KEY	"ifopts_caller_ptr"
 #define IFOPTS_DIALOG_PTR_KEY	"ifopts_dialog_ptr"
 #define IFOPTS_TABLE_ROWS 2
-#define IFOPTS_LIST_COLS  5
+#define IFOPTS_LIST_TEXT_COLS  4
 #define IFOPTS_MAX_DESCR_LEN 128
 #define IFOPTS_IF_NOSEL -1
 
 /* interface options dialog */
-static GtkWidget *cur_list, *if_dev_lb, *if_name_lb, *if_linktype_cb, *if_descr_te, *if_hide_cb;
+static GtkWidget *cur_list, *if_dev_lb, *if_name_lb, *if_linktype_lb, *if_linktype_cb, *if_descr_te, *if_hide_cb;
+#ifdef HAVE_PCAP_CREATE
+static GtkWidget *if_monitor_lb, *if_monitor_cb;
+#endif
 static GtkTreeSelection *if_selection;	/* current interface row selected */
 static int num_linktypes;
 static gboolean interfaces_info_nochange;  /* TRUE to ignore Interface Options Properties */
@@ -77,12 +78,18 @@ static void ifopts_edit_cb(GtkWidget *w, gpointer data);
 static void ifopts_edit_ok_cb(GtkWidget *w, gpointer parent_w);
 static void ifopts_edit_destroy_cb(GtkWidget *win, gpointer data);
 static void ifopts_edit_ifsel_cb(GtkTreeSelection *selection, gpointer data);
+#ifdef HAVE_PCAP_CREATE
+static void ifopts_edit_monitor_changed_cb(GtkToggleButton *tbt, gpointer udata);
+#endif
 static void ifopts_edit_linktype_changed_cb(GtkComboBox *ed, gpointer udata);
 static void ifopts_edit_descr_changed_cb(GtkEditable *ed, gpointer udata);
 static void ifopts_edit_hide_changed_cb(GtkToggleButton *tbt, gpointer udata);
 static void ifopts_options_add(GtkListStore *list_store, if_info_t *if_info);
 static void ifopts_options_free(gchar *text[]);
 static void ifopts_if_liststore_add(void);
+#ifdef HAVE_PCAP_CREATE
+static void ifopts_write_new_monitor_mode(void);
+#endif
 static void ifopts_write_new_linklayer(void);
 static void ifopts_write_new_descr(void);
 static void ifopts_write_new_hide(void);
@@ -91,7 +98,7 @@ GtkWidget*
 capture_prefs_show(void)
 {
 	GtkWidget	*main_tb, *main_vb;
-	GtkWidget	*if_cb, *if_lb, *promisc_cb, *pcap_ng_cb, *sync_cb, *auto_scroll_cb, *show_info_cb;
+	GtkWidget	*if_cbxe, *if_lb, *promisc_cb, *pcap_ng_cb, *sync_cb, *auto_scroll_cb, *show_info_cb;
 	GtkWidget	*ifopts_lb, *ifopts_bt;
 	GList		*if_list, *combo_list;
 	int		err;
@@ -117,7 +124,7 @@ capture_prefs_show(void)
 	gtk_misc_set_alignment(GTK_MISC(if_lb), 1.0f, 0.5f);
 	gtk_widget_show(if_lb);
 
-	if_cb = gtk_combo_new();
+	if_cbxe = gtk_combo_box_entry_new_text();
 	/*
 	 * XXX - what if we can't get the list?
 	 */
@@ -125,18 +132,26 @@ capture_prefs_show(void)
 	combo_list = build_capture_combo_list(if_list, FALSE);
 	free_interface_list(if_list);
 	if (combo_list != NULL) {
-		gtk_combo_set_popdown_strings(GTK_COMBO(if_cb), combo_list);
-		free_capture_combo_list(combo_list);
+		GList *combo_entry;
+		for (combo_entry = combo_list; combo_entry != NULL; combo_entry = g_list_next(combo_entry)) {
+				gtk_combo_box_append_text(GTK_COMBO_BOX(if_cbxe), combo_entry->data);
+		}
 	}
-	if (prefs.capture_device)
-		gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(if_cb)->entry),
-		    prefs.capture_device);
-	gtk_table_attach_defaults(GTK_TABLE(main_tb), if_cb, 1, 2, row, row+1);
+	if (prefs.capture_device) {
+		gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(if_cbxe))),
+				   prefs.capture_device);
+	}
+	else if (combo_list != NULL) {
+		gtk_combo_box_set_active(GTK_COMBO_BOX(if_cbxe), 0);
+	}
+	free_capture_combo_list(combo_list);
+
+	gtk_table_attach_defaults(GTK_TABLE(main_tb), if_cbxe, 1, 2, row, row+1);
 	tooltips_text = "The default interface to be captured from.";
 	gtk_tooltips_set_tip(tooltips, if_lb, tooltips_text, NULL);
-	gtk_tooltips_set_tip(tooltips, GTK_COMBO(if_cb)->entry, tooltips_text, NULL);
-	gtk_widget_show(if_cb);
-	g_object_set_data(G_OBJECT(main_vb), DEVICE_KEY, if_cb);
+	gtk_tooltips_set_tip(tooltips, gtk_bin_get_child(GTK_BIN(if_cbxe)), tooltips_text, NULL);
+	gtk_widget_show(if_cbxe);
+	g_object_set_data(G_OBJECT(main_vb), DEVICE_KEY, if_cbxe);
 	row++;
 
 	/* Interface properties */
@@ -165,8 +180,7 @@ capture_prefs_show(void)
 	/* Pcap-NG format */
 	pcap_ng_cb = create_preference_check_button(main_tb, row++,
 	    "Capture packets in pcap-ng format:",
-	    "Capture packets in the next-generation capture file format. "
-	    "This is still experimental.",
+	    "Capture packets in the next-generation capture file format.",
 	    prefs.capture_pcap_ng);
 	g_object_set_data(G_OBJECT(main_vb), PCAP_NG_KEY, pcap_ng_cb);
 
@@ -201,13 +215,13 @@ capture_prefs_show(void)
 void
 capture_prefs_fetch(GtkWidget *w)
 {
-	GtkWidget *if_cb, *promisc_cb, *pcap_ng_cb, *sync_cb, *auto_scroll_cb, *show_info_cb;
+	GtkWidget *if_cbxe, *promisc_cb, *pcap_ng_cb, *sync_cb, *auto_scroll_cb, *show_info_cb;
 	gchar	*if_text;
 
-	if_cb = (GtkWidget *)g_object_get_data(G_OBJECT(w), DEVICE_KEY);
+	if_cbxe    = (GtkWidget *)g_object_get_data(G_OBJECT(w), DEVICE_KEY);
 	promisc_cb = (GtkWidget *)g_object_get_data(G_OBJECT(w), PROM_MODE_KEY);
 	pcap_ng_cb = (GtkWidget *)g_object_get_data(G_OBJECT(w), PCAP_NG_KEY);
-	sync_cb = (GtkWidget *)g_object_get_data(G_OBJECT(w), CAPTURE_REAL_TIME_KEY);
+	sync_cb    = (GtkWidget *)g_object_get_data(G_OBJECT(w), CAPTURE_REAL_TIME_KEY);
 	auto_scroll_cb = (GtkWidget *)g_object_get_data(G_OBJECT(w), AUTO_SCROLL_KEY);
         show_info_cb = (GtkWidget *)g_object_get_data(G_OBJECT(w), SHOW_INFO_KEY);
 
@@ -215,7 +229,7 @@ capture_prefs_fetch(GtkWidget *w)
 		g_free(prefs.capture_device);
 		prefs.capture_device = NULL;
 	}
-	if_text = g_strdup(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(if_cb)->entry)));
+	if_text = g_strdup(gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(if_cbxe)))));
 	/* Strip out white space */
 	g_strstrip(if_text);
 	/* If there was nothing but white space, treat that as an
@@ -227,15 +241,15 @@ capture_prefs_fetch(GtkWidget *w)
 	}
 	prefs.capture_device = if_text;
 
-	prefs.capture_prom_mode = GTK_TOGGLE_BUTTON (promisc_cb)->active;
+	prefs.capture_prom_mode = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(promisc_cb));
 
-	prefs.capture_pcap_ng = GTK_TOGGLE_BUTTON (pcap_ng_cb)->active;
+	prefs.capture_pcap_ng = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pcap_ng_cb));
 
-	prefs.capture_real_time = GTK_TOGGLE_BUTTON (sync_cb)->active;
+	prefs.capture_real_time = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sync_cb));
 
-	prefs.capture_auto_scroll = GTK_TOGGLE_BUTTON (auto_scroll_cb)->active;
+	prefs.capture_auto_scroll = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(auto_scroll_cb));
 
-	prefs.capture_show_info = !(GTK_TOGGLE_BUTTON (show_info_cb)->active);
+	prefs.capture_show_info = !(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(show_info_cb)));
 }
 
 void
@@ -266,6 +280,9 @@ enum
 {
 	DEVICE_COLUMN,
 	DESC_COLUMN,
+#ifdef HAVE_PCAP_CREATE
+	DEF_MONITOR_MODE_COLUMN,
+#endif
 	DEF_LINK_LAYER_COLUMN,
 	COMMENT_COLUMN,
 	HIDE_COLUMN,
@@ -278,7 +295,8 @@ ifopts_edit_cb(GtkWidget *w, gpointer data _U_)
 {
 	GtkWidget	  *ifopts_edit_dlg, *cur_scr_win, *main_hb, *main_tb,
 			  *cur_opts_fr, *ed_opts_fr, *main_vb,
-			  *if_linktype_lb, *if_descr_lb, *if_hide_lb,
+			  *if_descr_lb,
+			  *if_hide_lb,
 			  *bbox, *ok_bt, *cancel_bt, *help_bt;
 
 	GtkListStore	  *list_store;
@@ -304,7 +322,7 @@ ifopts_edit_cb(GtkWidget *w, gpointer data _U_)
 
 	/* create a new dialog */
 	ifopts_edit_dlg = dlg_conf_window_new("Wireshark: Preferences: Interface Options");
-	gtk_window_set_default_size(GTK_WINDOW(ifopts_edit_dlg), DEF_WIDTH, 340);
+	gtk_window_set_default_size(GTK_WINDOW(ifopts_edit_dlg), 1000, 440);
 
 	main_vb = gtk_vbox_new(FALSE, 1);
 	gtk_container_set_border_width(GTK_CONTAINER(main_vb), 5);
@@ -328,9 +346,12 @@ ifopts_edit_cb(GtkWidget *w, gpointer data _U_)
 	list_store = gtk_list_store_new(N_COLUMN,	/* Total number of columns XXX	*/
 					G_TYPE_STRING,	/* Device			*/
 					G_TYPE_STRING,	/* Description			*/
+#ifdef HAVE_PCAP_CREATE
+					G_TYPE_BOOLEAN,	/* Monitor mode		*/
+#endif
 					G_TYPE_STRING,	/* Default link-layer		*/
 					G_TYPE_STRING,	/* Comment			*/
-					G_TYPE_STRING,	/* Hide?			*/
+					G_TYPE_BOOLEAN,	/* Hide?			*/
 					G_TYPE_INT);	/* Dlt 				*/
 
 	list = gtk_tree_view_new_with_model (GTK_TREE_MODEL (list_store));
@@ -340,13 +361,54 @@ ifopts_edit_cb(GtkWidget *w, gpointer data _U_)
 	/* The view now holds a reference.  We can get rid of our own reference */
 	g_object_unref (G_OBJECT (list_store));
 
-	/* 
+	/*
 	 * Create the first column packet, associating the "text" attribute of the
-	 * cell_renderer to the first column of the model 
+	 * cell_renderer to the first column of the model
 	 */
 	renderer = gtk_cell_renderer_text_new ();
-	column = gtk_tree_view_column_new_with_attributes ("Device", renderer, 
-							   "text", DEVICE_COLUMN, 
+	column = gtk_tree_view_column_new_with_attributes ("Device", renderer,
+							   "text", DEVICE_COLUMN,
+							   NULL);
+
+	gtk_tree_view_column_set_resizable(column, TRUE);
+	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
+#ifdef _WIN32
+	gtk_tree_view_column_set_min_width(column, 230);
+#else
+	gtk_tree_view_column_set_min_width(column, 70);
+#endif
+	/* Add the column to the view. */
+	gtk_tree_view_append_column (list_view, column);
+
+	renderer = gtk_cell_renderer_text_new ();
+	column = gtk_tree_view_column_new_with_attributes ("Description", renderer,
+							   "text", DESC_COLUMN,
+							   NULL);
+
+	gtk_tree_view_column_set_resizable(column, TRUE);
+	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
+	gtk_tree_view_column_set_min_width(column, 260);
+	/* Add the column to the view. */
+	gtk_tree_view_append_column (list_view, column);
+
+#ifdef HAVE_PCAP_CREATE
+	/*
+	 * XXX - for some reason, this doesn't show up.
+	 */
+	renderer = gtk_cell_renderer_toggle_new ();
+	column = gtk_tree_view_column_new_with_attributes ("Default to monitor mode", renderer,
+							   "active", DEF_MONITOR_MODE_COLUMN,
+							   NULL);
+
+	gtk_tree_view_column_set_resizable(column, FALSE);
+	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
+	/* Add the column to the view. */
+	gtk_tree_view_append_column (list_view, column);
+#endif
+
+	renderer = gtk_cell_renderer_text_new ();
+	column = gtk_tree_view_column_new_with_attributes ("Default link-layer", renderer,
+							   "text", DEF_LINK_LAYER_COLUMN,
 							   NULL);
 
 	gtk_tree_view_column_set_resizable(column, TRUE);
@@ -355,28 +417,9 @@ ifopts_edit_cb(GtkWidget *w, gpointer data _U_)
 	/* Add the column to the view. */
 	gtk_tree_view_append_column (list_view, column);
 
-	column = gtk_tree_view_column_new_with_attributes ("Description", renderer, 
-							   "text", DESC_COLUMN, 
-							   NULL);
-
-	gtk_tree_view_column_set_resizable(column, TRUE);
-	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
-	gtk_tree_view_column_set_min_width(column, 260);
-	/* Add the column to the view. */
-	gtk_tree_view_append_column (list_view, column);
-
-	column = gtk_tree_view_column_new_with_attributes ("Default link-layer", renderer, 
-							   "text", DEF_LINK_LAYER_COLUMN, 
-							   NULL);
-
-	gtk_tree_view_column_set_resizable(column, TRUE);
-	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
-	gtk_tree_view_column_set_min_width(column, 260);
-	/* Add the column to the view. */
-	gtk_tree_view_append_column (list_view, column);
-
-	column = gtk_tree_view_column_new_with_attributes ("Comment", renderer, 
-							   "text", COMMENT_COLUMN, 
+	renderer = gtk_cell_renderer_text_new ();
+	column = gtk_tree_view_column_new_with_attributes ("Comment", renderer,
+							   "text", COMMENT_COLUMN,
 							   NULL);
 
 	gtk_tree_view_column_set_resizable(column, TRUE);
@@ -385,20 +428,21 @@ ifopts_edit_cb(GtkWidget *w, gpointer data _U_)
 	/* Add the column to the view. */
 	gtk_tree_view_append_column (list_view, column);
 
-	column = gtk_tree_view_column_new_with_attributes ("Hide?", renderer, 
-							   "text", HIDE_COLUMN, 
+	renderer = gtk_cell_renderer_toggle_new ();
+	column = gtk_tree_view_column_new_with_attributes ("Hide?", renderer,
+							   "active", HIDE_COLUMN,
 							   NULL);
 
-	gtk_tree_view_column_set_resizable(column, TRUE);
+	gtk_tree_view_column_set_resizable(column, FALSE);
 	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
-	gtk_tree_view_column_set_min_width(column, 40);
 	/* Add the column to the view. */
 	gtk_tree_view_append_column (list_view, column);
 
 #if 0
 	/* Don't show the DLT column */
-	column = gtk_tree_view_column_new_with_attributes ("DLT", renderer, 
-							   "text", DLT_COLUMN, 
+	renderer = gtk_cell_renderer_text_new ();
+	column = gtk_tree_view_column_new_with_attributes ("DLT", renderer,
+							   "text", DLT_COLUMN,
 							   NULL);
 
 	gtk_tree_view_column_set_resizable(column, TRUE);
@@ -464,6 +508,21 @@ ifopts_edit_cb(GtkWidget *w, gpointer data _U_)
 	gtk_widget_show(if_name_lb);
 	row++;
 
+#ifdef HAVE_PCAP_CREATE
+	/* create "monitor mode" label and button */
+	if_monitor_lb = gtk_label_new("Monitor mode:");
+	gtk_table_attach_defaults(GTK_TABLE(main_tb), if_monitor_lb, 0, 1, row, row+1);
+	gtk_misc_set_alignment(GTK_MISC(if_monitor_lb), 1.0f, 0.5f);
+	gtk_widget_show(if_monitor_lb);
+
+	if_monitor_cb = gtk_check_button_new();
+	g_signal_connect(if_monitor_cb, "toggled", G_CALLBACK(ifopts_edit_monitor_changed_cb),
+			cur_list);
+	gtk_table_attach_defaults(GTK_TABLE(main_tb), if_monitor_cb, 1, 2, row, row+1);
+	gtk_widget_show(if_monitor_cb);
+        row++;
+#endif
+
 	if_linktype_lb = gtk_label_new("Default link-layer header type:");
 	gtk_table_attach_defaults(GTK_TABLE(main_tb), if_linktype_lb, 0, 1, row, row+1);
 	gtk_misc_set_alignment(GTK_MISC(if_linktype_lb), 1.0f, 0.5f);
@@ -521,7 +580,7 @@ ifopts_edit_cb(GtkWidget *w, gpointer data _U_)
         window_set_cancel_button(ifopts_edit_dlg, cancel_bt, window_cancel_button_cb);
 
 	help_bt = g_object_get_data(G_OBJECT(bbox), GTK_STOCK_HELP);
-	g_signal_connect(help_bt, "clicked", G_CALLBACK(topic_cb), 
+	g_signal_connect(help_bt, "clicked", G_CALLBACK(topic_cb),
 			 (gpointer)HELP_CAPTURE_INTERFACE_OPTIONS_DIALOG);
 	gtk_tooltips_set_tip (tooltips, help_bt, "Show topic specific help", NULL);
 
@@ -550,6 +609,11 @@ static void
 ifopts_edit_ok_cb(GtkWidget *w _U_, gpointer parent_w)
 {
 	if (if_selection){ /* XXX: Cannot be NULL ?? */
+#ifdef HAVE_PCAP_CREATE
+		/* create/write new monitor-mode interfaces string */
+		ifopts_write_new_monitor_mode();
+#endif
+
 		/* create/write new interfaces link-layer string */
 		ifopts_write_new_linklayer();
 
@@ -585,30 +649,35 @@ ifopts_edit_destroy_cb(GtkWidget *win, gpointer data _U_)
 }
 
 static gint
-ifopts_description_to_val (const char *if_name, const char *descr) 
+ifopts_description_to_val (const char *if_name, gboolean monitor_mode,
+    const char *descr)
 {
-	GList *lt_list;
+	if_capabilities_t *caps;
 	int dlt = -1;
 
-	lt_list = capture_pcap_linktype_list(if_name, NULL);
-	if (lt_list != NULL) {
-		GList  *lt_entry;
-		/* XXX: Code skips first entry because that's the default ??? */
-		for (lt_entry = g_list_next(lt_list); lt_entry != NULL; lt_entry = g_list_next(lt_entry)) {
-			data_link_info_t *dli_p = lt_entry->data;
-			if (dli_p->description) {
-				if (strcmp(dli_p->description, descr) == 0) {
-					dlt = dli_p->dlt;
-					break;
-				}
-			} else {
-				if (strcmp(dli_p->name, descr) == 0) {
-					dlt = dli_p->dlt;
-					break;
+	caps = capture_get_if_capabilities(if_name, monitor_mode, NULL);
+	if (caps != NULL) {
+		if (caps->data_link_types != NULL) {
+			GList  *lt_entry;
+			/* XXX: Code skips first entry because that's the default ??? */
+			for (lt_entry = g_list_next(caps->data_link_types);
+			    lt_entry != NULL;
+			    lt_entry = g_list_next(lt_entry)) {
+				data_link_info_t *dli_p = lt_entry->data;
+				if (dli_p->description) {
+					if (strcmp(dli_p->description, descr) == 0) {
+						dlt = dli_p->dlt;
+						break;
+					}
+				} else {
+					if (strcmp(dli_p->name, descr) == 0) {
+						dlt = dli_p->dlt;
+						break;
+					}
 				}
 			}
 		}
-		free_pcap_linktype_list(lt_list);
+		free_if_capabilities(caps);
 	}
 	return dlt;
 }
@@ -620,23 +689,30 @@ static void
 ifopts_edit_ifsel_cb(GtkTreeSelection	*selection _U_,
 		     gpointer		 data _U_)
 {
-	GtkTreeIter       iter;
-	GtkTreeModel	 *model;
-	gchar            *desc, *comment, *hide, *text;
-	gchar            *if_name, *linktype;
-	GList 		 *lt_list;
-	gint              selected = 0;
+	GtkTreeIter         iter;
+	GtkTreeModel       *model;
+	gchar              *desc, *comment, *text;
+	gchar              *if_name, *linktype;
+#ifdef HAVE_PCAP_CREATE
+	gboolean            monitor_mode;
+#endif
+	gboolean            hide;
+	if_capabilities_t  *caps;
+	gint                selected = 0;
 
 	/* Get list_store data for currently selected interface */
 	if (!gtk_tree_selection_get_selected (if_selection, &model, &iter)){
 		return;
 	}
-	gtk_tree_model_get(model, &iter, 
-			   DEVICE_COLUMN,  &if_name,
-			   DESC_COLUMN,    &desc,
-			   DEF_LINK_LAYER_COLUMN, &linktype,
-			   COMMENT_COLUMN, &comment,
-			   HIDE_COLUMN,    &hide,
+	gtk_tree_model_get(model, &iter,
+			   DEVICE_COLUMN,             &if_name,
+			   DESC_COLUMN,               &desc,
+#ifdef HAVE_PCAP_CREATE
+			   DEF_MONITOR_MODE_COLUMN,   &monitor_mode,
+#endif
+			   DEF_LINK_LAYER_COLUMN,     &linktype,
+			   COMMENT_COLUMN,            &comment,
+			   HIDE_COLUMN,               &hide,
 			   -1);
 
 	/* display  the interface device from current interfaces selection */
@@ -655,32 +731,53 @@ ifopts_edit_ifsel_cb(GtkTreeSelection	*selection _U_,
 		gtk_combo_box_remove_text (GTK_COMBO_BOX(if_linktype_cb), num_linktypes);
 	}
 
-        /*  -- build and add to the ComboBox a linktype list for the current interfaces selection */
-	lt_list = capture_pcap_linktype_list(if_name, NULL);
-	if (lt_list != NULL) {
-		GList *lt_entry;
-		for (lt_entry = lt_list; lt_entry != NULL; lt_entry = g_list_next(lt_entry)) {
-			data_link_info_t *dli_p = lt_entry->data;
-			text = (dli_p->description != NULL) ? dli_p->description : dli_p->name;
-			if (strcmp(linktype, text) == 0) {
-				selected = num_linktypes;
+        /*
+	 * -- set the state and sensitivity of the monitor-mode checkbox,
+	 * and build and add to the ComboBox a linktype list, corresponding
+	 * to the interface capabilities of the selected interface
+	 */
+#ifdef HAVE_PCAP_CREATE
+	caps = capture_get_if_capabilities(if_name, monitor_mode, NULL);
+#else
+	caps = capture_get_if_capabilities(if_name, FALSE, NULL);
+#endif
+	if (caps != NULL) {
+#ifdef HAVE_PCAP_CREATE
+		gtk_widget_set_sensitive(if_monitor_lb, caps->can_set_rfmon);
+		gtk_widget_set_sensitive(if_monitor_cb, caps->can_set_rfmon);
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(if_monitor_cb), monitor_mode);
+#endif
+		if (caps->data_link_types != NULL) {
+			GList *lt_entry;
+			for (lt_entry = caps->data_link_types; lt_entry != NULL;
+			    lt_entry = g_list_next(lt_entry)) {
+				data_link_info_t *dli_p = lt_entry->data;
+				text = (dli_p->description != NULL) ? dli_p->description : dli_p->name;
+				if (strcmp(linktype, text) == 0) {
+					selected = num_linktypes;
+				}
+				gtk_combo_box_append_text(GTK_COMBO_BOX(if_linktype_cb), text);
+				num_linktypes++;
 			}
-			gtk_combo_box_append_text(GTK_COMBO_BOX(if_linktype_cb), text);
-			num_linktypes++;
+			gtk_widget_set_sensitive(if_linktype_lb, num_linktypes >= 2);
+			gtk_widget_set_sensitive(if_linktype_cb, num_linktypes >= 2);
+			gtk_combo_box_set_active(GTK_COMBO_BOX(if_linktype_cb), selected);
 		}
-		gtk_widget_set_sensitive(if_linktype_cb, num_linktypes >= 2);
-		gtk_combo_box_set_active(GTK_COMBO_BOX(if_linktype_cb), selected);
-		free_pcap_linktype_list(lt_list);
+		free_if_capabilities(caps);
 	}
+#ifdef HAVE_PCAP_CREATE
+	else {
+		gtk_widget_set_sensitive(if_monitor_lb, FALSE);
+		gtk_widget_set_sensitive(if_monitor_cb, FALSE);
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(if_monitor_cb), FALSE);
+	}
+#endif
 
 	/* display the interface description from current interfaces selection */
 	gtk_entry_set_text(GTK_ENTRY(if_descr_te), comment);
 
 	/* display the "hide interface" button state from current interfaces selection */
-	if (strcmp("Yes", hide) == 0)
-		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(if_hide_cb), TRUE);
-	else
-		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(if_hide_cb), FALSE);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(if_hide_cb), hide);
 
 	interfaces_info_nochange = FALSE;
 
@@ -688,9 +785,100 @@ ifopts_edit_ifsel_cb(GtkTreeSelection	*selection _U_,
 	g_free(desc);
 	g_free(linktype);
 	g_free(comment);
-	g_free(hide);
 }
 
+#ifdef HAVE_PCAP_CREATE
+/*
+ * Monitor-mode toggle button changed callback; update displayed widgets
+ * (the list of link-layer types might change) and list_store for currently
+ * selected interface.
+ */
+static void
+ifopts_edit_monitor_changed_cb(GtkToggleButton *tbt, gpointer udata)
+{
+	GtkTreeModel      *list_model;
+	GtkTreeIter        list_iter;
+	GtkListStore      *list_store;
+	gchar             *if_name, *text;
+	gboolean           monitor_mode;
+	if_capabilities_t *caps;
+
+	if (interfaces_info_nochange)
+		return;
+
+	if (if_selection == NULL) /* XXX: Cannot be NULL ?? */
+		return;
+
+	if (!gtk_tree_selection_get_selected (if_selection, &list_model, &list_iter)){
+		return;
+	}
+	gtk_tree_model_get(list_model, &list_iter,
+			   DEVICE_COLUMN,  &if_name,
+			   -1);
+
+	/* Ignore "changed" callbacks while we update the Properties widgets */
+	interfaces_info_nochange = TRUE;
+
+	/* display the link-layer header type from current interfaces selection */
+        /*  -- remove old linktype list (if any) from the ComboBox */
+	while (num_linktypes > 0) {
+		num_linktypes--;
+		gtk_combo_box_remove_text (GTK_COMBO_BOX(if_linktype_cb), num_linktypes);
+	}
+
+	list_store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW (udata))); /* Get store */
+
+#ifdef HAVE_PCAP_CREATE
+	/* get "monitor mode" button state and set status in list_store for currently selected interface */
+	monitor_mode = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tbt));
+	gtk_list_store_set  (list_store, &list_iter,
+			     DEF_MONITOR_MODE_COLUMN, monitor_mode,
+			     -1);
+	caps = capture_get_if_capabilities(if_name, monitor_mode, NULL);
+#else
+	/* no monitor-mode support */
+	caps = capture_get_if_capabilities(if_name, FALSE, NULL);
+#endif
+
+        /*
+	 * -- set the sensitivity of the monitor-mode checkbox, and
+	 * build and add to the ComboBox a linktype list for the current
+	 * interfaces selection, based on the interface capabilities
+	 */
+	if (caps != NULL) {
+#ifdef HAVE_PCAP_CREATE
+		gtk_widget_set_sensitive(if_monitor_lb, caps->can_set_rfmon);
+		gtk_widget_set_sensitive(if_monitor_cb, caps->can_set_rfmon);
+#endif
+		if (caps->data_link_types != NULL) {
+			GList *lt_entry;
+			for (lt_entry = caps->data_link_types; lt_entry != NULL;
+			    lt_entry = g_list_next(lt_entry)) {
+				data_link_info_t *dli_p = lt_entry->data;
+				text = (dli_p->description != NULL) ? dli_p->description : dli_p->name;
+				gtk_combo_box_append_text(GTK_COMBO_BOX(if_linktype_cb), text);
+				num_linktypes++;
+			}
+			gtk_widget_set_sensitive(if_linktype_lb, num_linktypes >= 2);
+			gtk_widget_set_sensitive(if_linktype_cb, num_linktypes >= 2);
+			gtk_combo_box_set_active(GTK_COMBO_BOX(if_linktype_cb), 0);
+		}
+		free_if_capabilities(caps);
+	}
+#ifdef HAVE_PCAP_CREATE
+	else {
+		gtk_widget_set_sensitive(if_monitor_lb, FALSE);
+		gtk_widget_set_sensitive(if_monitor_cb, FALSE);
+	}
+#endif
+
+	interfaces_info_nochange = FALSE;
+	g_signal_emit_by_name(if_linktype_cb, "changed");
+
+	g_free(if_name);
+}
+#endif
+
 /*
  * Link-layer entry changed callback; update list_store for currently selected interface.
  */
@@ -698,6 +886,9 @@ static void
 ifopts_edit_linktype_changed_cb(GtkComboBox *cb, gpointer udata)
 {
 	gchar        *ifnm, *text;
+#ifdef HAVE_PCAP_CREATE
+	gboolean      monitor_mode;
+#endif
 	gint          linktype;
 	GtkTreeModel *list_model;
 #if ! GTK_CHECK_VERSION(2,6,0)
@@ -716,9 +907,12 @@ ifopts_edit_linktype_changed_cb(GtkComboBox *cb, gpointer udata)
 	if (!gtk_tree_selection_get_selected (if_selection, &list_model, &list_iter)){
 		return;
 	}
-	
-	gtk_tree_model_get(list_model, &list_iter, 
-		DEVICE_COLUMN, &ifnm,
+
+	gtk_tree_model_get(list_model, &list_iter,
+		DEVICE_COLUMN,           &ifnm,
+#ifdef HAVE_PCAP_CREATE
+		DEF_MONITOR_MODE_COLUMN, &monitor_mode,
+#endif
 		-1);
 
 	/* get current description text and set value in list_store for currently selected interface */
@@ -730,7 +924,11 @@ ifopts_edit_linktype_changed_cb(GtkComboBox *cb, gpointer udata)
 		model = gtk_combo_box_get_model(GTK_COMBO_BOX(cb));
 		gtk_tree_model_get(model, &iter, 0, &text, -1);
 #endif
-		linktype = ifopts_description_to_val(ifnm, text);
+#ifdef HAVE_PCAP_CREATE
+		linktype = ifopts_description_to_val(ifnm, monitor_mode, text);
+#else
+		linktype = ifopts_description_to_val(ifnm, FALSE, text);
+#endif
 		list_store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW (udata))); /* Get store */
 		gtk_list_store_set  (list_store, &list_iter,
 				     DEF_LINK_LAYER_COLUMN, text,
@@ -795,14 +993,14 @@ ifopts_edit_hide_changed_cb(GtkToggleButton *tbt, gpointer udata)
 	}
 
 	list_store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW (udata))); /* Get store */
-	/* get "hide" button state and set text in list_store for currently selected interface */
+	/* get "hide" button state and set status in list_store for currently selected interface */
 	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tbt)) == TRUE)
 		gtk_list_store_set  (list_store, &list_iter,
-				     HIDE_COLUMN, "Yes",
+				     HIDE_COLUMN, TRUE,
 				     -1);
 	else
 		gtk_list_store_set  (list_store, &list_iter,
-				     HIDE_COLUMN, "No",
+				     HIDE_COLUMN, FALSE,
 				     -1);
 }
 
@@ -822,9 +1020,13 @@ ifopts_options_add(GtkListStore *list_store, if_info_t *if_info)
 	gchar	*ifnm;
 	gchar	*desc;
 	gchar	*pr_descr;
-	gchar	*text[] = { NULL, NULL, NULL, NULL, NULL };
-	GList   *lt_list;
+	gchar	*text[] = { NULL, NULL, NULL, NULL };
+	if_capabilities_t *caps;
+#ifdef HAVE_PCAP_CREATE
+	gboolean monitor_mode;
+#endif
 	gint     linktype;
+	gboolean hide;
 	GtkTreeIter  iter;
 
 	/* set device name text */
@@ -836,24 +1038,35 @@ ifopts_options_add(GtkListStore *list_store, if_info_t *if_info)
 	else
 		text[1] = g_strdup("");
 
+#ifdef HAVE_PCAP_CREATE
+	/* get default monitor mode setting */
+	monitor_mode = prefs_capture_device_monitor_mode(if_info->name);
+	caps = capture_get_if_capabilities(if_info->name, monitor_mode, NULL);
+#else
+	/* no monitor-mode support */
+	caps = capture_get_if_capabilities(if_info->name, FALSE, NULL);
+#endif
+
 	/* set default link-layer header type */
 	linktype = capture_dev_user_linktype_find(if_info->name);
-	lt_list = capture_pcap_linktype_list(if_info->name, NULL);
-	if (lt_list != NULL) {
-		GList  *lt_entry;
-		for (lt_entry = lt_list; lt_entry != NULL; lt_entry = g_list_next(lt_entry)) {
-			data_link_info_t *dli_p = lt_entry->data;
-			/* If we have no previous link-layer header type we use the first one */
-			if (linktype == -1 || linktype == dli_p->dlt) {
-				if (dli_p->description) {
-					text[2] = g_strdup(dli_p->description);
-				} else {
-					text[2] = g_strdup(dli_p->name);
+	if (caps != NULL) {
+		if (caps->data_link_types != NULL) {
+			GList  *lt_entry;
+			for (lt_entry = caps->data_link_types; lt_entry != NULL;
+			    lt_entry = g_list_next(lt_entry)) {
+				data_link_info_t *dli_p = lt_entry->data;
+				/* If we have no previous link-layer header type we use the first one */
+				if (linktype == -1 || linktype == dli_p->dlt) {
+					if (dli_p->description) {
+						text[2] = g_strdup(dli_p->description);
+					} else {
+						text[2] = g_strdup(dli_p->name);
+					}
+					break;
 				}
-				break;
 			}
 		}
-		free_pcap_linktype_list(lt_list);
+		free_if_capabilities(caps);
 	}
 	/* if we have no link-layer */
 	if (text[2] == NULL)
@@ -908,11 +1121,7 @@ ifopts_options_add(GtkListStore *list_store, if_info_t *if_info)
 		text[3] = g_strdup("");
 
 	/* check if interface is "hidden" */
-	if ((prefs.capture_devices_hide != NULL) &&
-	    (strstr(prefs.capture_devices_hide, if_info->name) != NULL))
-		text[4] = g_strdup("Yes");
-	else
-		text[4] = g_strdup("No");
+	hide = prefs_is_capture_device_hidden(if_info->name);
 
 	/* add row to ListStore */
 
@@ -922,12 +1131,15 @@ ifopts_options_add(GtkListStore *list_store, if_info_t *if_info)
 	gtk_list_store_append  (list_store, &iter);
 	gtk_list_store_set  (list_store, &iter,
 #endif
-			     DEVICE_COLUMN,  text[0],
-			     DESC_COLUMN,    text[1],
-			     DEF_LINK_LAYER_COLUMN, text[2],
-			     COMMENT_COLUMN, text[3],
-			     HIDE_COLUMN,    text[4],
-			     DLT_COLUMN,     linktype,
+			     DEVICE_COLUMN,           text[0],
+			     DESC_COLUMN,             text[1],
+#ifdef HAVE_PCAP_CREATE
+			     DEF_MONITOR_MODE_COLUMN, monitor_mode,
+#endif
+			     DEF_LINK_LAYER_COLUMN,   text[2],
+			     COMMENT_COLUMN,          text[3],
+			     HIDE_COLUMN,             hide,
+			     DLT_COLUMN,              linktype,
 			     -1);
 
 	ifopts_options_free(text);
@@ -938,7 +1150,7 @@ ifopts_options_free(gchar *text[])
 {
 	gint i;
 
-	for (i=0; i < IFOPTS_LIST_COLS; i++) {
+	for (i=0; i < IFOPTS_LIST_TEXT_COLS; i++) {
 		if (text[i] != NULL) {
 			g_free(text[i]);
 			text[i] = NULL;
@@ -978,6 +1190,69 @@ ifopts_if_liststore_add(void)
 	free_interface_list(if_list);
 }
 
+#ifdef HAVE_PCAP_CREATE
+/*
+ * Create/write new "monitor mode" interfaces string based on current CList.
+ * Put it into the preferences value.
+ */
+static void
+ifopts_write_new_monitor_mode(void)
+{
+	GtkListStore 	*store;
+	GtkTreeIter 	 iter;
+	GtkTreeModel 	*model;
+	gboolean	 more_items = TRUE;
+	gint		 first_if = TRUE;	/* flag to check if first in list */
+	gchar		*ifnm;
+	gboolean	 monitor_mode;
+	gchar		*new_monitor_mode;
+
+	/* new preferences "monitor mode" interfaces string */
+	new_monitor_mode = g_malloc0(MAX_VAL_LEN);
+
+	/* get "monitor mode" flag text for each row (interface) */
+	model = gtk_tree_view_get_model(GTK_TREE_VIEW(cur_list));
+	store = GTK_LIST_STORE(model);
+	if( gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter) ) {
+		while (more_items) {
+			gtk_tree_model_get(GTK_TREE_MODEL(store), &iter,
+					   DEVICE_COLUMN,           &ifnm,
+					   DEF_MONITOR_MODE_COLUMN, &monitor_mode,
+					   -1);
+
+			/* if flag text is "No", skip this interface */
+			if (!monitor_mode){
+				more_items = gtk_tree_model_iter_next (model,&iter);
+				continue;
+			}
+
+			/*
+			 * create/cat interface to new string
+			 */
+			if (first_if != TRUE)
+				g_strlcat (new_monitor_mode, ",", MAX_VAL_LEN);
+			g_strlcat (new_monitor_mode, ifnm, MAX_VAL_LEN);
+
+			/* set first-in-list flag to false */
+			first_if = FALSE;
+			more_items = gtk_tree_model_iter_next (model,&iter);
+		}
+
+		/* write new "hidden" string to preferences */
+		if (strlen(new_monitor_mode) > 0) {
+			g_free(prefs.capture_devices_monitor_mode);
+			prefs.capture_devices_monitor_mode = new_monitor_mode;
+		}
+		/* no "hidden" interfaces */
+		else {
+			g_free(prefs.capture_devices_monitor_mode);
+			g_free(new_monitor_mode);
+			prefs.capture_devices_monitor_mode = NULL;
+		}
+	}
+}
+#endif
+
 /*
  * Create/write new interfaces link-layer string based on current CList.
  * Put it into the preferences value.
@@ -1020,7 +1295,7 @@ ifopts_write_new_linklayer(void)
 			/*
 			 * create/cat interface link-layer to new string
 			 * (leave space for parens, comma and terminator)
-			 */			
+			 */
 			tmp_linklayer = g_strdup_printf("%s(%d)", ifnm, linktype);
 			g_strlcat(new_linklayer, tmp_linklayer, MAX_VAL_LEN);
 			g_free(tmp_linklayer);
@@ -1123,7 +1398,7 @@ ifopts_write_new_hide(void)
 	gboolean	 more_items = TRUE;
 	gint		 first_if = TRUE;	/* flag to check if first in list */
 	gchar		*ifnm;
-	gchar		*hide;
+	gboolean	 hide;
 	gchar		*new_hide;
 
 	/* new preferences "hidden" interfaces string */
@@ -1140,7 +1415,7 @@ ifopts_write_new_hide(void)
 					   -1);
 
 			/* if flag text is "No", skip this interface */
-			if (strcmp("No", hide) == 0){
+			if (!hide){
 				more_items = gtk_tree_model_iter_next (model,&iter);
 				continue;
 			}
