@@ -1,7 +1,7 @@
 /* prefs_dlg.c
  * Routines for handling preferences
  *
- * $Id: prefs_dlg.c 30087 2009-09-23 14:30:56Z etxrab $
+ * $Id: prefs_dlg.c 36161 2011-03-08 01:52:25Z sake $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -221,14 +221,14 @@ pref_show(pref_t *pref, gpointer user_data)
 
   case PREF_RANGE:
   {
-    char *range_string;
+    char *range_str_p;
 
     g_free(pref->saved_val.range);
     pref->saved_val.range = range_copy(*pref->varp.range);
-    range_string = range_convert_range(*pref->varp.range);
+    range_str_p = range_convert_range(*pref->varp.range);
     pref->control = create_preference_entry(main_tb, pref->ordinal,
                                             label_string, pref->description,
-                                            range_string);
+                                            range_str_p);
     break;
   }
 
@@ -409,7 +409,13 @@ prefs_nb_page_add(GtkWidget *notebook, const gchar *title, GtkWidget *page, cons
 
 /* show the dialog */
 void
-prefs_cb(GtkWidget *w _U_, gpointer dummy _U_)
+prefs_cb(GtkWidget *w, gpointer dummy)
+{
+  prefs_page_cb (w, dummy, PREFS_PAGE_USER_INTERFACE);
+}
+
+void
+prefs_page_cb(GtkWidget *w _U_, gpointer dummy _U_, PREFS_PAGE_E prefs_page)
 {
   GtkWidget         *top_hb, *bbox, *prefs_nb, *ct_sb,
                     *ok_bt, *apply_bt, *save_bt, *cancel_bt, *help_bt;
@@ -421,7 +427,8 @@ prefs_cb(GtkWidget *w _U_, gpointer dummy _U_)
   GtkCellRenderer   *renderer;
   GtkTreeViewColumn *column;
   gint              col_offset;
-  prefs_tree_iter   gui_iter;
+  prefs_tree_iter   gui_iter, layout_iter, columns_iter;
+  gint              layout_page, columns_page;
 
 
   if (prefs_w != NULL) {
@@ -509,14 +516,14 @@ prefs_cb(GtkWidget *w _U_, gpointer dummy _U_)
   /* GUI layout prefs */
   g_strlcpy(label_str, "Layout", MAX_TREE_NODE_NAME_LEN);
   prefs_nb_page_add(prefs_nb, label_str, layout_prefs_show(), E_GUI_LAYOUT_PAGE_KEY);
-  prefs_tree_page_add(label_str, cts.page, store, &gui_iter);
-  cts.page++;
+  layout_iter = prefs_tree_page_add(label_str, cts.page, store, &gui_iter);
+  layout_page = cts.page++;
 
   /* GUI Column prefs */
   g_strlcpy(label_str, "Columns", MAX_TREE_NODE_NAME_LEN);
   prefs_nb_page_add(prefs_nb, label_str, column_prefs_show(prefs_w), E_GUI_COLUMN_PAGE_KEY);
-  prefs_tree_page_add(label_str, cts.page, store, &gui_iter);
-  cts.page++;
+  columns_iter = prefs_tree_page_add(label_str, cts.page, store, &gui_iter);
+  columns_page = cts.page++;
 
   /* GUI Font prefs */
   g_strlcpy(label_str, "Font", MAX_TREE_NODE_NAME_LEN);
@@ -610,7 +617,7 @@ prefs_cb(GtkWidget *w _U_, gpointer dummy _U_)
   help_bt = g_object_get_data(G_OBJECT(bbox), GTK_STOCK_HELP);
   g_signal_connect(help_bt, "clicked", G_CALLBACK(topic_cb), (gpointer)HELP_PREFERENCES_DIALOG);
 
-  g_signal_connect(prefs_w, "delete_event", G_CALLBACK(prefs_main_delete_event_cb), prefs_w);
+  g_signal_connect(prefs_w, "delete_event", G_CALLBACK(prefs_main_delete_event_cb), NULL);
   g_signal_connect(prefs_w, "destroy", G_CALLBACK(prefs_main_destroy_cb), prefs_w);
 
   gtk_widget_show(prefs_w);
@@ -622,6 +629,20 @@ prefs_cb(GtkWidget *w _U_, gpointer dummy _U_)
 
   window_present(prefs_w);
 
+  switch (prefs_page) {
+  case PREFS_PAGE_LAYOUT:
+    gtk_tree_selection_select_iter(selection, &layout_iter);
+    gtk_notebook_set_current_page(g_object_get_data(G_OBJECT(prefs_w), E_PREFSW_NOTEBOOK_KEY), layout_page);
+    break;
+  case PREFS_PAGE_COLUMNS:
+    gtk_tree_selection_select_iter(selection, &columns_iter);
+    gtk_notebook_set_current_page(g_object_get_data(G_OBJECT(prefs_w), E_PREFSW_NOTEBOOK_KEY), columns_page);
+    break;
+  default:
+    /* Not implemented yet */
+    break;
+  }
+
   g_object_unref(G_OBJECT(store));
 }
 
@@ -676,7 +697,7 @@ create_preference_radio_buttons(GtkWidget *main_tb, int table_position,
   GtkTooltips *tooltips;
   GtkWidget *radio_button_hbox, *button = NULL;
   GSList *rb_group;
-  int index;
+  int idx;
   const enum_val_t *enum_valp;
   GtkWidget *event_box;
 
@@ -687,8 +708,8 @@ create_preference_radio_buttons(GtkWidget *main_tb, int table_position,
 
   radio_button_hbox = gtk_hbox_new(FALSE, 0);
   rb_group = NULL;
-  for (enum_valp = enumvals, index = 0; enum_valp->name != NULL;
-       enum_valp++, index++) {
+  for (enum_valp = enumvals, idx = 0; enum_valp->name != NULL;
+       enum_valp++, idx++) {
     button = gtk_radio_button_new_with_label(rb_group,
                                              enum_valp->description);
     gtk_widget_show(button);
@@ -756,12 +777,12 @@ fetch_preference_radio_buttons_val(GtkWidget *button,
   for (rb_entry = rb_group; rb_entry != NULL;
        rb_entry = g_slist_next(rb_entry)) {
     button = rb_entry->data;
-    if (GTK_TOGGLE_BUTTON(button)->active)
+    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button)))
       break;
   }
 
   /* OK, now return the value corresponding to that button's label. */
-  return label_to_enum_val(GTK_BIN(button)->child, enumvals);
+  return label_to_enum_val(gtk_bin_get_child(GTK_BIN(button)), enumvals);
 }
 
 GtkWidget *
@@ -771,7 +792,7 @@ create_preference_option_menu(GtkWidget *main_tb, int table_position,
 {
   GtkTooltips *tooltips;
   GtkWidget *menu_box, *combo_box;
-  int menu_index, index;
+  int menu_idx, idx;
   const enum_val_t *enum_valp;
   GtkWidget *event_box;
 
@@ -784,15 +805,15 @@ create_preference_option_menu(GtkWidget *main_tb, int table_position,
   combo_box = gtk_combo_box_new_text ();
   if (tooltip_text != NULL && tooltips != NULL)
     gtk_tooltips_set_tip(tooltips, combo_box, tooltip_text, NULL);
-  menu_index = 0;
-  for (enum_valp = enumvals, index = 0; enum_valp->name != NULL;
-       enum_valp++, index++) {
-	gtk_combo_box_append_text (GTK_COMBO_BOX (combo_box), enum_valp->description);
+  menu_idx = 0;
+  for (enum_valp = enumvals, idx = 0; enum_valp->name != NULL;
+       enum_valp++, idx++) {
+    gtk_combo_box_append_text (GTK_COMBO_BOX (combo_box), enum_valp->description);
     if (enum_valp->value == current_val)
-      menu_index = index;
+      menu_idx = idx;
   }
   /* Set the current value active */
-  gtk_combo_box_set_active(GTK_COMBO_BOX(combo_box), menu_index);
+  gtk_combo_box_set_active(GTK_COMBO_BOX(combo_box), menu_idx);
 
   /*
    * Put the combo box in an hbox, so that it's only as wide
@@ -822,9 +843,9 @@ fetch_preference_option_menu_val(GtkWidget *combo_box, const enum_val_t *enumval
    */
     int i;
 
-	i = gtk_combo_box_get_active (GTK_COMBO_BOX(combo_box));
+    i = gtk_combo_box_get_active (GTK_COMBO_BOX(combo_box));
 
-	return enumvals[i].value;
+    return enumvals[i].value;
 }
 
 GtkWidget *
@@ -993,7 +1014,7 @@ pref_fetch(pref_t *pref, gpointer user_data)
     break;
 
   case PREF_BOOL:
-    bval = GTK_TOGGLE_BUTTON(pref->control)->active;
+    bval = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pref->control));
     if (*pref->varp.boolp != bval) {
       *pref_changed_p = TRUE;
       *pref->varp.boolp = bval;
@@ -1619,10 +1640,10 @@ prefs_main_cancel_cb(GtkWidget *cancel_bt _U_, gpointer parent_w)
 
 /* Treat this as a cancel, by calling "prefs_main_cancel_cb()" */
 static gboolean
-prefs_main_delete_event_cb(GtkWidget *prefs_w, GdkEvent *event _U_,
-                           gpointer parent_w _U_)
+prefs_main_delete_event_cb(GtkWidget *prefs_w_lcl, GdkEvent *event _U_,
+                           gpointer user_data _U_)
 {
-  prefs_main_cancel_cb(NULL, prefs_w);
+  prefs_main_cancel_cb(NULL, prefs_w_lcl);
   return FALSE;
 }
 
@@ -1679,7 +1700,7 @@ tree_expand_row(GtkTreeModel *model, GtkTreeView *tree_view, GtkTreeIter *iter)
 /* select a node in the tree view */
 /* XXX - this is almost 100% copied from byte_view_select() in proto_draw.c,
  *       find a way to combine both to have a generic function for this */
-void
+static void
 tree_select_node(GtkWidget *tree, prefs_tree_iter *iter)
 {
   GtkTreeIter  local_iter = *iter;
