@@ -2,7 +2,7 @@
  *
  * Laurent Deniel <laurent.deniel@free.fr>
  *
- * $Id: supported_protos_dlg.h 24034 2008-01-08 22:54:51Z stig $
+ * $Id: supported_protos_dlg.h 33230 2010-06-15 21:18:31Z stig $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -24,8 +24,8 @@
  *
  */
 
-#ifndef __SUPPORTED_DLG_H__
-#define __SUPPORTED_DLG_H__
+#ifndef __SUPPORTED_PROTOS_DLG_H__
+#define __SUPPORTED_PROTOS_DLG_H__
 
 /** @file
  *  "Supported Protocols" dialog box.
@@ -42,4 +42,4 @@ void supported_cb(GtkWidget *widget, gpointer data);
 /** Redraw all the supported protos text widgets, to use a new font. */
 void supported_redraw(void);
 
-#endif
+#endif /* __SUPPORTED_PROTOS_DLG_H__ */
