@@ -1,6 +1,6 @@
 #!/bin/bash
 #
-# $Id: osx-app.sh 28548 2009-05-31 18:30:46Z stig $
+# $Id: osx-app.sh 37399 2011-05-25 21:32:13Z gerald $
 #
 # USAGE
 # osx-app [-s] [-l /path/to/libraries] -bp /path/to/wireshark/bin -p /path/to/Info.plist
@@ -229,6 +229,7 @@ done
 # ChmodBPF
 mkdir -p "$chmodbpf_dir"
 cp -v ChmodBPF/* "$chmodbpf_dir"
+chmod -R g-w "$chmodbpf_dir"
 
 # The rest of the Wireshark installation (we handled bin above)
 rsync -av \
@@ -276,11 +277,6 @@ mkdir -p $pkgetc/gtk-2.0
 sed -e "s,$LIBPREFIX,\${CWD},g" $LIBPREFIX/etc/gtk-2.0/gdk-pixbuf.loaders > $pkgetc/gtk-2.0/gdk-pixbuf.loaders
 sed -e "s,$LIBPREFIX,\${CWD},g" $LIBPREFIX/etc/gtk-2.0/gtk.immodules > $pkgetc/gtk-2.0/gtk.immodules
 
-for item in gnome-vfs-mime-magic gnome-vfs-2.0
-do
-	cp -r $LIBPREFIX/etc/$item $pkgetc/
-done
-
 pango_version=`pkg-config --variable=pango_module_version pango`
 mkdir -p $pkglib/pango/$pango_version/modules
 cp $LIBPREFIX/lib/pango/$pango_version/modules/*.so $pkglib/pango/$pango_version/modules/
@@ -289,8 +285,11 @@ gtk_version=`pkg-config --variable=gtk_binary_version gtk+-2.0`
 mkdir -p $pkglib/gtk-2.0/$gtk_version/{engines,immodules,loaders}
 cp -r $LIBPREFIX/lib/gtk-2.0/$gtk_version/* $pkglib/gtk-2.0/$gtk_version/
 
-mkdir -p $pkglib/gnome-vfs-2.0/modules
-cp $LIBPREFIX/lib/gnome-vfs-2.0/modules/*.so $pkglib/gnome-vfs-2.0/modules/
+gdk_pixbuf_version=`pkg-config --variable=gdk_pixbuf_binary_version gdk-pixbuf-2.0`
+if [ ! -z $gdk_pixbuf_version ]; then
+	mkdir -p $pkglib/gdk-pixbuf-2.0/$gdk_pixbuf_version/loaders
+	cp -r $LIBPREFIX/lib/gdk-pixbuf-2.0/$gdk_pixbuf_version/loaders/* $pkglib/gdk-pixbuf-2.0/$gdk_pixbuf_version/loaders
+fi
 
 # Find out libs we need from fink, darwinports, or from a custom install
 # (i.e. $LIBPREFIX), then loop until no changes.
@@ -302,7 +301,7 @@ lib_dep_search_list="
 	$pkglib/gtk-2.0/$gtk_version/immodules/*
 	$pkglib/gtk-2.0/$gtk_version/engines/*.so
 	$pkglib/pango/$pango_version/modules/*
-	$pkglib/gnome-vfs-2.0/modules/*
+	$pkglib/gdk-pixbuf-2.0/$gdk_pixbuf_version/loaders/*
 	$package/Contents/Resources/lib/*
 	$pkgbin/*-bin
 	"
