@@ -1,7 +1,7 @@
 /* print.c
  * Routines for printing packet analysis trees.
  *
- * $Id: print.c 30361 2009-10-06 13:23:07Z stig $
+ * $Id: print.c 37399 2011-05-25 21:32:13Z gerald $
  *
  * Gilbert Ramirez <gram@alumni.rice.edu>
  *
@@ -31,6 +31,8 @@
 #include <stdio.h>
 #include <string.h>
 
+#include <glib.h>
+
 #include <epan/epan.h>
 #include <epan/epan_dissect.h>
 #include <epan/tvbuff.h>
@@ -45,6 +47,7 @@
 #include <epan/charsets.h>
 #include <epan/dissectors/packet-data.h>
 #include <epan/dissectors/packet-frame.h>
+#include <epan/filesystem.h>
 
 #define PDML_VERSION "0"
 #define PSML_VERSION "0"
@@ -56,7 +59,7 @@ typedef struct {
 	GSList		 	*src_list;
 	print_dissections_e	print_dissections;
 	gboolean		print_hex_for_data;
-	char_enc		encoding;
+	packet_char_enc		encoding;
 	epan_dissect_t		*edt;
 } print_data;
 
@@ -75,19 +78,25 @@ typedef struct {
 struct _output_fields {
     gboolean print_header;
     gchar separator;
+    gchar occurrence;
+    gchar aggregator;
     GPtrArray* fields;
     GHashTable* field_indicies;
-    const gchar** field_values;
+    emem_strbuf_t** field_values;
     gchar quote;
 };
 
+GHashTable *output_only_tables = NULL;
+
+static gboolean write_headers = FALSE;
+
 static const gchar* get_field_hex_value(GSList* src_list, field_info *fi);
 static void proto_tree_print_node(proto_node *node, gpointer data);
 static void proto_tree_write_node_pdml(proto_node *node, gpointer data);
 static const guint8 *get_field_data(GSList *src_list, field_info *fi);
 static void write_pdml_field_hex_value(write_pdml_data *pdata, field_info *fi);
 static gboolean print_hex_data_buffer(print_stream_t *stream, const guchar *cp,
-    guint length, char_enc encoding);
+    guint length, packet_char_enc encoding);
 static void ps_clean_string(unsigned char *out, const unsigned char *in,
 			int outbuf_size);
 static void print_escaped_xml(FILE *fh, const char *unescaped_string);
@@ -184,6 +193,19 @@ void proto_tree_print_node(proto_node *node, gpointer data)
 		return;
 	}
 
+	/*
+	 * If -O is specified, only display the protocols which are in the
+	 * lookup table.  Only check on the first level: once we start printing
+	 * a tree, print the rest of the subtree.  Otherwise we won't print
+	 * subitems whose abbreviation doesn't match the protocol--for example
+	 * text items (whose abbreviation is simply "text").
+	 */
+	if (output_only_tables != NULL && pdata->level == 0
+	 && g_hash_table_lookup(output_only_tables, fi->hfinfo->abbrev) == NULL) {
+	  pdata->success = TRUE;
+	  return;
+	}
+
 	if (PROTO_ITEM_IS_GENERATED(node)) {
 		g_free(label_ptr);
 	}
@@ -222,12 +244,19 @@ void proto_tree_print_node(proto_node *node, gpointer data)
 	}
 }
 
+#define PDML2HTML_XSL "pdml2html.xsl"
 void
-write_pdml_preamble(FILE *fh)
+write_pdml_preamble(FILE *fh, const gchar* filename)
 {
+	time_t t=time(NULL);
+	char *ts=asctime(localtime(&t));
+	ts[strlen(ts)-1]=0; /* overwrite \n */
+
 	fputs("<?xml version=\"1.0\"?>\n", fh);
+	fputs("<?xml-stylesheet type=\"text/xsl\" href=\"" PDML2HTML_XSL "\"?>\n", fh);
+	fprintf(fh, "<!-- You can find " PDML2HTML_XSL " in %s or at http://anonsvn.wireshark.org/trunk/wireshark/" PDML2HTML_XSL ". -->\n", get_datafile_dir());
 	fputs("<pdml version=\"" PDML_VERSION "\" ", fh);
-	fprintf(fh, "creator=\"%s/%s\">\n", PACKAGE, VERSION);
+	fprintf(fh, "creator=\"%s/%s\" time=\"%s\" capture_file=\"%s\">\n", PACKAGE, VERSION, ts, filename);
 }
 
 void
@@ -241,10 +270,6 @@ proto_tree_write_pdml(epan_dissect_t *edt, FILE *fh)
 	data.src_list = edt->pi.data_src;
 	data.edt = edt;
 
-	/* We shouldn't be called with a NULL pointer here because we've
-	 * created a visible protocol tree */
-	g_assert(data.src_list);
-
 	fprintf(fh, "<packet>\n");
 
 	/* Print a "geninfo" protocol as required by PDML */
@@ -344,7 +369,7 @@ proto_tree_write_node_pdml(proto_node *node, gpointer data)
 
 #if 0
 	/* PDML spec, see:
-	 * http://analyzer.polito.it/30alpha/docs/dissectors/PDMLSpec.htm
+	 * http://www.nbee.org/doku.php?id=netpdl:pdml_specification
 	 *
 	 * the show fields contains things in 'human readable' format
 	 * showname: contains only the name of the field
@@ -492,7 +517,7 @@ print_pdml_geninfo(proto_tree *tree, FILE *fh)
 	if (g_ptr_array_len(finfo_array) < 1) {
 		return;
 	}
-	frame_finfo = finfo_array->pdata[0];
+	frame_finfo = (field_info *)finfo_array->pdata[0];
 	g_ptr_array_free(finfo_array, TRUE);
 
 	/* frame.number --> geninfo.num */
@@ -524,7 +549,7 @@ print_pdml_geninfo(proto_tree *tree, FILE *fh)
 	if (g_ptr_array_len(finfo_array) < 1) {
 		return;
 	}
-	timestamp = fvalue_get(&((field_info*)finfo_array->pdata[0])->value);
+	timestamp = (nstime_t *)fvalue_get(&((field_info*)finfo_array->pdata[0])->value);
 	g_ptr_array_free(finfo_array, TRUE);
 
 	/* Print geninfo start */
@@ -550,7 +575,7 @@ print_pdml_geninfo(proto_tree *tree, FILE *fh)
 	/* Print geninfo.timestamp */
 	fprintf(fh,
 "    <field name=\"timestamp\" pos=\"0\" show=\"%s\" showname=\"Captured Time\" value=\"%d.%09d\" size=\"%u\"/>\n",
-		abs_time_to_str(timestamp), (int) timestamp->secs, timestamp->nsecs, frame_finfo->length);
+		abs_time_to_str(timestamp, ABSOLUTE_TIME_LOCAL, TRUE), (int) timestamp->secs, timestamp->nsecs, frame_finfo->length);
 
 	/* Print geninfo end */
 	fprintf(fh,
@@ -569,6 +594,7 @@ write_psml_preamble(FILE *fh)
 	fputs("<?xml version=\"1.0\"?>\n", fh);
 	fputs("<psml version=\"" PSML_VERSION "\" ", fh);
 	fprintf(fh, "creator=\"%s/%s\">\n", PACKAGE, VERSION);
+	write_headers = TRUE;
 }
 
 void
@@ -577,7 +603,7 @@ proto_tree_write_psml(epan_dissect_t *edt, FILE *fh)
 	gint	i;
 
 	/* if this is the first packet, we have to create the PSML structure output */
-	if(edt->pi.fd->num == 1) {
+	if(write_headers) {
 	    fprintf(fh, "<structure>\n");
 
 	    for(i=0; i < edt->pi.cinfo->num_cols; i++) {
@@ -587,6 +613,8 @@ proto_tree_write_psml(epan_dissect_t *edt, FILE *fh)
 	    }
 
 	    fprintf(fh, "</structure>\n\n");
+
+	    write_headers = FALSE;
 	}
 
 	fprintf(fh, "<packet>\n");
@@ -609,26 +637,46 @@ write_psml_finale(FILE *fh)
 void
 write_csv_preamble(FILE *fh _U_)
 {
+	write_headers = TRUE;
+}
 
+static gchar *csv_massage_str(const gchar *source, const gchar *exceptions)
+{
+    gchar *csv_str;
+    gchar *tmp_str;
+
+    csv_str = g_strescape(source, exceptions);
+    tmp_str = csv_str;
+    while ( (tmp_str = strstr(tmp_str, "\\\"")) != NULL )
+        *tmp_str = '\"';
+    return csv_str;
+}
+
+static void csv_write_str(const char *str, char sep, FILE *fh)
+{
+    gchar *csv_str;
+
+    csv_str = csv_massage_str(str, NULL);
+    fprintf(fh, "\"%s\"%c", csv_str, sep);
+    g_free(csv_str);
 }
 
 void
 proto_tree_write_csv(epan_dissect_t *edt, FILE *fh)
 {
-        gint    i;
-
-        /* if this is the first packet, we have to write the CSV header */
-        if(edt->pi.fd->num == 1) {
-            for(i=0; i < edt->pi.cinfo->num_cols - 1; i++)
-	        fprintf(fh, "\"%s\",", edt->pi.cinfo->col_title[i]);
-
-            fprintf(fh, "\"%s\"\n", edt->pi.cinfo->col_title[i]);
-        }
+    gint i;
 
+    /* if this is the first packet, we have to write the CSV header */
+    if(write_headers) {
         for(i=0; i < edt->pi.cinfo->num_cols - 1; i++)
-            fprintf(fh, "\"%s\",", edt->pi.cinfo->col_data[i]);
+            csv_write_str(edt->pi.cinfo->col_title[i], ',', fh);
+        csv_write_str(edt->pi.cinfo->col_title[i], '\n', fh);
+        write_headers = FALSE;
+    }
 
-        fprintf(fh, "\"%s\"\n", edt->pi.cinfo->col_data[i]);
+    for(i=0; i < edt->pi.cinfo->num_cols - 1; i++)
+        csv_write_str(edt->pi.cinfo->col_data[i], ',', fh);
+    csv_write_str(edt->pi.cinfo->col_data[i], '\n', fh);
 }
 
 void
@@ -689,7 +737,7 @@ get_field_data(GSList *src_list, field_info *fi)
 	gint length, tvbuff_length;
 
 	for (src_le = src_list; src_le != NULL; src_le = src_le->next) {
-		src = src_le->data;
+		src = (data_source *)src_le->data;
 		src_tvb = src->tvb;
 		if (fi->ds_tvb == src_tvb) {
 			/*
@@ -723,6 +771,7 @@ static void
 print_escaped_xml(FILE *fh, const char *unescaped_string)
 {
 	const char *p;
+	char temp_str[8];
 
 	for (p = unescaped_string; *p != '\0'; p++) {
 		switch (*p) {
@@ -742,7 +791,12 @@ print_escaped_xml(FILE *fh, const char *unescaped_string)
 				fputs("&apos;", fh);
 				break;
 			default:
-				fputc(*p, fh);
+				if (g_ascii_isprint(*p))
+					fputc(*p, fh);
+				else {
+					g_snprintf(temp_str, sizeof(temp_str), "\\x%x", (guint8)*p);
+					fputs(temp_str, fh);
+				}
 		}
 	}
 }
@@ -784,10 +838,6 @@ print_hex_data(print_stream_t *stream, epan_dissect_t *edt)
 	const guchar *cp;
 	guint length;
 
-	/* We shouldn't be called with a NULL pointer here because we've
-	 * created a visible protocol tree */
-	g_assert(edt->pi.data_src);
-
 	/*
 	 * Set "multiple_sources" iff this frame has more than one
 	 * data source; if it does, we need to print the name of
@@ -798,7 +848,7 @@ print_hex_data(print_stream_t *stream, epan_dissect_t *edt)
 
 	for (src_le = edt->pi.data_src; src_le != NULL;
 	    src_le = src_le->next) {
-		src = src_le->data;
+		src = (data_source *)src_le->data;
 		tvb = src->tvb;
 		if (multiple_sources) {
 			name = get_data_source_name(src);
@@ -842,7 +892,7 @@ print_hex_data(print_stream_t *stream, epan_dissect_t *edt)
 
 static gboolean
 print_hex_data_buffer(print_stream_t *stream, const guchar *cp,
-    guint length, char_enc encoding)
+    guint length, packet_char_enc encoding)
 {
 	register unsigned int ad, i, j, k, l;
 	guchar c;
@@ -900,7 +950,7 @@ print_hex_data_buffer(print_stream_t *stream, const guchar *cp,
 		line[j++] = binhex[c>>4];
 		line[j++] = binhex[c&0xf];
 		j++;
-		if (encoding == CHAR_EBCDIC) {
+		if (encoding == PACKET_CHAR_ENC_CHAR_EBCDIC) {
 			c = EBCDIC_to_ASCII1(c);
 		}
 		line[k++] = c >= ' ' && c < 0x7f ? c : '.';
@@ -928,6 +978,11 @@ void ps_clean_string(unsigned char *out, const unsigned char *in,
 	int rd, wr;
 	char c;
 
+	if (in == NULL) {
+		out[0] = '\0';
+		return;
+	}
+
 	for (rd = 0, wr = 0 ; wr < outbuf_size; rd++, wr++ ) {
 		c = in[rd];
 		switch (c) {
@@ -1003,7 +1058,7 @@ print_preamble_text(print_stream_t *self _U_, gchar *filename _U_)
 static gboolean
 print_line_text(print_stream_t *self, int indent, const char *line)
 {
-	output_text *output = self->data;
+	output_text *output = (output_text *)self->data;
 	char space[MAX_INDENT+1];
 	int i;
 	int num_spaces;
@@ -1036,7 +1091,7 @@ print_bookmark_text(print_stream_t *self _U_, const gchar *name _U_,
 static gboolean
 new_page_text(print_stream_t *self)
 {
-	output_text *output = self->data;
+	output_text *output = (output_text *)self->data;
 
 	fputs("\f", output->fh);
 	return !ferror(output->fh);
@@ -1052,7 +1107,7 @@ print_finale_text(print_stream_t *self _U_)
 static gboolean
 destroy_text(print_stream_t *self)
 {
-	output_text *output = self->data;
+	output_text *output = (output_text *)self->data;
 	gboolean ret;
 
 	ret = close_print_dest(output->to_file, output->fh);
@@ -1076,10 +1131,10 @@ print_stream_text_alloc(int to_file, FILE *fh)
 	print_stream_t *stream;
 	output_text *output;
 
-	output = g_malloc(sizeof *output);
+	output = (output_text *)g_malloc(sizeof *output);
 	output->to_file = to_file;
 	output->fh = fh;
-	stream = g_malloc(sizeof (print_stream_t));
+	stream = (print_stream_t *)g_malloc(sizeof (print_stream_t));
 	stream->ops = &print_text_ops;
 	stream->data = output;
 
@@ -1112,14 +1167,11 @@ typedef struct {
 static gboolean
 print_preamble_ps(print_stream_t *self, gchar *filename)
 {
-	output_ps *output = self->data;
+	output_ps *output = (output_ps *)self->data;
 	unsigned char psbuffer[MAX_PS_LINE_LENGTH]; /* static sized buffer! */
 
 	print_ps_preamble(output->fh);
 
-	fputs("%% Set the font to 8 point\n", output->fh);
-	fputs("/Courier findfont 8 scalefont setfont\n", output->fh);
-	fputs("\n", output->fh);
 	fputs("%% the page title\n", output->fh);
 	ps_clean_string(psbuffer, filename, MAX_PS_LINE_LENGTH);
 	fprintf(output->fh, "/ws_pagetitle (%s - Wireshark " VERSION "%s) def\n", psbuffer, wireshark_svnversion);
@@ -1130,7 +1182,7 @@ print_preamble_ps(print_stream_t *self, gchar *filename)
 static gboolean
 print_line_ps(print_stream_t *self, int indent, const char *line)
 {
-	output_ps *output = self->data;
+	output_ps *output = (output_ps *)self->data;
 	unsigned char psbuffer[MAX_PS_LINE_LENGTH]; /* static sized buffer! */
 
 	ps_clean_string(psbuffer, line, MAX_PS_LINE_LENGTH);
@@ -1141,7 +1193,7 @@ print_line_ps(print_stream_t *self, int indent, const char *line)
 static gboolean
 print_bookmark_ps(print_stream_t *self, const gchar *name, const gchar *title)
 {
-	output_ps *output = self->data;
+	output_ps *output = (output_ps *)self->data;
 	unsigned char psbuffer[MAX_PS_LINE_LENGTH]; /* static sized buffer! */
 
 	/*
@@ -1171,7 +1223,7 @@ print_bookmark_ps(print_stream_t *self, const gchar *name, const gchar *title)
 static gboolean
 new_page_ps(print_stream_t *self)
 {
-	output_ps *output = self->data;
+	output_ps *output = (output_ps *)self->data;
 
 	fputs("formfeed\n", output->fh);
 	return !ferror(output->fh);
@@ -1180,7 +1232,7 @@ new_page_ps(print_stream_t *self)
 static gboolean
 print_finale_ps(print_stream_t *self)
 {
-	output_ps *output = self->data;
+	output_ps *output = (output_ps *)self->data;
 
 	print_ps_finale(output->fh);
 	return !ferror(output->fh);
@@ -1189,7 +1241,7 @@ print_finale_ps(print_stream_t *self)
 static gboolean
 destroy_ps(print_stream_t *self)
 {
-	output_ps *output = self->data;
+	output_ps *output = (output_ps *)self->data;
 	gboolean ret;
 
 	ret = close_print_dest(output->to_file, output->fh);
@@ -1213,10 +1265,10 @@ print_stream_ps_alloc(int to_file, FILE *fh)
 	print_stream_t *stream;
 	output_ps *output;
 
-	output = g_malloc(sizeof *output);
+	output = (output_ps *)g_malloc(sizeof *output);
 	output->to_file = to_file;
 	output->fh = fh;
-	stream = g_malloc(sizeof (print_stream_t));
+	stream = (print_stream_t *)g_malloc(sizeof (print_stream_t));
 	stream->ops = &print_ps_ops;
 	stream->data = output;
 
@@ -1241,11 +1293,13 @@ print_stream_ps_stdio_new(FILE *fh)
 	return print_stream_ps_alloc(TRUE, fh);
 }
 
-output_fields_t* output_fields_new()
+output_fields_t* output_fields_new(void)
 {
     output_fields_t* fields = g_new(output_fields_t, 1);
     fields->print_header = FALSE;
     fields->separator = '\t';
+    fields->occurrence = 'a';
+    fields->aggregator = ',';
     fields->fields = NULL; /*Do lazy initialisation */
     fields->field_indicies = NULL;
     fields->field_values = NULL;
@@ -1277,7 +1331,7 @@ void output_fields_free(output_fields_t* fields)
     if(NULL != fields->fields) {
         gsize i;
         for(i = 0; i < fields->fields->len; ++i) {
-            gchar* field = g_ptr_array_index(fields->fields,i);
+            gchar* field = (gchar *)g_ptr_array_index(fields->fields,i);
             g_free(field);
         }
         g_ptr_array_free(fields->fields, TRUE);
@@ -1315,6 +1369,9 @@ gboolean output_fields_set_option(output_fields_t* info, gchar* option)
         return FALSE; /* Is this guarded against by option parsing? */
     }
     option_name = strtok(option,"=");
+    if (!option_name) {
+        return FALSE;
+    }
     option_value = option + strlen(option_name) + 1;
     if(0 == strcmp(option_name, "header")) {
         switch(NULL == option_value ? '\0' : *option_value) {
@@ -1353,6 +1410,39 @@ gboolean output_fields_set_option(output_fields_t* info, gchar* option)
         return TRUE;
     }
 
+    if(0 == strcmp(option_name, "occurrence")) {
+        switch(NULL == option_value ? '\0' : *option_value) {
+        case 'f':
+        case 'l':
+        case 'a':
+            info->occurrence = *option_value;
+            break;
+        default:
+            return FALSE;
+        }
+        return TRUE;
+    }
+
+    if(0 == strcmp(option_name,"aggregator")) {
+        switch(NULL == option_value ? '\0' : *option_value) {
+        case '\0':
+            return FALSE;
+        case '/':
+            switch(*++option_value) {
+            case 's':
+                info->aggregator = ' ';
+                break;
+            default:
+                info->aggregator = '\\';
+            }
+            break;
+        default:
+            info->aggregator = *option_value;
+            break;
+        }
+        return TRUE;
+    }
+
     if(0 == strcmp(option_name, "quote")) {
         switch(NULL == option_value ? '\0' : *option_value) {
         default: /* Fall through */
@@ -1378,9 +1468,11 @@ gboolean output_fields_set_option(output_fields_t* info, gchar* option)
 void output_fields_list_options(FILE *fh)
 {
     fprintf(fh, "TShark: The available options for field output \"E\" are:\n");
-    fputs("header=y|n   Print field abbreviations as first line of output (def: N: no)\n", fh);
-    fputs("separator=/t|/s|<character>   Set the separator to use; \"/t\" = tab,\n \"/s\" = space (def: /t: tab)\n", fh);
-    fputs("quote=d|s|n   Print either d: double-quotes, s: single quotes or n: no quotes around field values (def: n: none)\n", fh);
+    fputs("header=y|n    Print field abbreviations as first line of output (def: N: no)\n", fh);
+    fputs("separator=/t|/s|<character>   Set the separator to use;\n     \"/t\" = tab, \"/s\" = space (def: /t: tab)\n", fh);
+    fputs("occurrence=f|l|a  Select the occurrence of a field to use;\n     \"f\" = first, \"l\" = last, \"a\" = all (def: a: all)\n", fh);
+    fputs("aggregator=,|/s|<character>   Set the aggregator to use;\n     \",\" = comma, \"/s\" = space (def: ,: comma)\n", fh);
+    fputs("quote=d|s|n   Print either d: double-quotes, s: single quotes or \n     n: no quotes around field values (def: n: none)\n", fh);
 }
 
 
@@ -1396,7 +1488,7 @@ void write_fields_preamble(output_fields_t* fields, FILE *fh)
     }
 
     for(i = 0; i < fields->fields->len; ++i) {
-        const gchar* field = g_ptr_array_index(fields->fields,i);
+        const gchar* field = (const gchar *)g_ptr_array_index(fields->fields,i);
         if(i != 0 ) {
             fputc(fields->separator, fh);
         }
@@ -1411,7 +1503,7 @@ static void proto_tree_get_node_field_values(proto_node *node, gpointer data)
     field_info *fi;
     gpointer field_index;
 
-    call_data = data;
+    call_data = (write_field_data_t *)data;
     fi = PNODE_FINFO(node);
 
     g_assert(fi && "dissection with an invisible proto tree?");
@@ -1426,7 +1518,16 @@ static void proto_tree_get_node_field_values(proto_node *node, gpointer data)
             guint actual_index;
             actual_index = GPOINTER_TO_UINT(field_index);
             /* Unwrap change made to disambiguiate zero / null */
-            call_data->fields->field_values[actual_index - 1] = value;
+            if ( call_data->fields->field_values[actual_index - 1] == NULL ) {
+                call_data->fields->field_values[actual_index - 1] = ep_strbuf_new(value);
+            } else if ( call_data->fields->occurrence == 'l' ) {
+                /* print only the value of the last occurrence of the field */
+                ep_strbuf_printf(call_data->fields->field_values[actual_index - 1],"%s",value);
+            } else if ( call_data->fields->occurrence == 'a' ) {
+                /* print the value of all accurrences of the field */
+                ep_strbuf_append_printf(call_data->fields->field_values[actual_index - 1],
+                    "%c%s",call_data->fields->aggregator,value);
+            }
         }
     }
 
@@ -1456,7 +1557,7 @@ void proto_tree_write_fields(output_fields_t* fields, epan_dissect_t *edt, FILE
 
         i = 0;
         while( i < fields->fields->len) {
-            gchar* field = g_ptr_array_index(fields->fields, i);
+            gchar* field = (gchar *)g_ptr_array_index(fields->fields, i);
              /* Store field indicies +1 so that zero is not a valid value,
               * and can be distinguished from NULL as a pointer.
               */
@@ -1466,7 +1567,7 @@ void proto_tree_write_fields(output_fields_t* fields, epan_dissect_t *edt, FILE
     }
 
     /* Buffer to store values for this packet */
-    fields->field_values = ep_alloc_array0(const gchar*, fields->fields->len);
+    fields->field_values = ep_alloc_array0(emem_strbuf_t*, fields->fields->len);
 
     proto_tree_children_foreach(edt->tree, proto_tree_get_node_field_values,
                                 &data);
@@ -1479,7 +1580,7 @@ void proto_tree_write_fields(output_fields_t* fields, epan_dissect_t *edt, FILE
             if(fields->quote != '\0') {
                 fputc(fields->quote, fh);
             }
-            fputs(fields->field_values[i], fh);
+            fputs(fields->field_values[i]->str, fh);
             if(fields->quote != '\0') {
                 fputc(fields->quote, fh);
             }
