@@ -2,7 +2,7 @@
  * Definitions for alert box routines with toolkit-independent APIs but
  * toolkit-dependent implementations.
  *
- * $Id: simple_dialog.h 29804 2009-09-08 18:02:43Z krj $
+ * $Id: simple_dialog.h 35041 2010-11-27 21:50:49Z guy $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -26,8 +26,6 @@
 #ifndef __DIALOG_H__
 #define __DIALOG_H__
 
-#include <epan/gnuc_format_check.h>
-
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
@@ -89,7 +87,8 @@ typedef enum {
  */
 extern gpointer simple_dialog(ESD_TYPE_E type, gint btn_mask,
     const gchar *msg_format, ...)
-    GNUC_FORMAT_CHECK(printf, 3, 4);
+    G_GNUC_PRINTF(3, 4);
+
 /** Create and show a simple dialog using a va_list.
  *
  * @param type type of dialog
