@@ -6,7 +6,7 @@
  *
  * (c) Copyright 2001 Ashok Narayanan <ashokn@cisco.com>
  *
- * $Id: text2pcap.c 30370 2009-10-06 16:01:18Z stig $
+ * $Id: text2pcap.c 37531 2011-06-03 12:24:22Z etxrab $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -51,7 +51,7 @@
  *   exactly 32 bytes must have been read into this packet before this. If the offset
  *   is wrong, the packet is immediately terminated
  *
- * A packet start is signalled by a zero offset.
+ * A packet start is signaled by a zero offset.
  *
  * Lines starting with #TEXT2PCAP are directives. These allow the user
  * to embed instructions into the capture file which allows text2pcap
@@ -128,14 +128,19 @@
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #else
-#include "wsgetopt.h"
+#include "wsutil/wsgetopt.h"
 #endif
 
 #ifdef NEED_STRPTIME_H
-# include "strptime.h"
+# include "wsutil/strptime.h"
 #endif
 
 #include "text2pcap.h"
+#include "svnversion.h"
+
+#ifdef _WIN32
+#include <wsutil/unicode-utils.h>
+#endif /* _WIN32 */
 
 /*--- Options --------------------------------------------------------------------*/
 
@@ -249,8 +254,8 @@ typedef struct {
 } hdr_ethernet_t;
 
 static hdr_ethernet_t HDR_ETHERNET = {
-    {0x02, 0x02, 0x02, 0x02, 0x02, 0x02},
-    {0x01, 0x01, 0x01, 0x01, 0x01, 0x01},
+    {0x0a, 0x02, 0x02, 0x02, 0x02, 0x02},
+    {0x0a, 0x01, 0x01, 0x01, 0x01, 0x01},
     0};
 
 typedef struct {
@@ -267,7 +272,7 @@ typedef struct {
     guint32 dest_addr;
 } hdr_ip_t;
 
-static hdr_ip_t HDR_IP = {0x45, 0, 0, 0x3412, 0, 0, 0xff, 0, 0, 0x01010101, 0x02020202};
+static hdr_ip_t HDR_IP = {0x45, 0, 0, 0x3412, 0, 0, 0xff, 0, 0, 0x0101010a, 0x0202020a};
 
 static struct {			/* pseudo header for checksum calculation */
 	guint32 src_addr;
@@ -417,7 +422,8 @@ in_checksum (void *buf, unsigned long count)
     while (sum>>16)
         sum = (sum & 0xffff) + (sum >> 16);
 
-    return g_htons(~sum);
+    sum = ~sum;
+    return g_htons(sum);
 }
 
 /* The CRC32C code is taken from draft-ietf-tsvwg-sctpcsum-01.txt.
@@ -599,16 +605,22 @@ write_current_packet (void)
 
         /* Write UDP header */
         if (hdr_udp) {
+            guint16 x16;
             HDR_UDP.source_port = g_htons(hdr_src_port);
             HDR_UDP.dest_port = g_htons(hdr_dest_port);
             HDR_UDP.length = g_htons(proto_length);
 
+            /* Note: g_ntohs()/g_htons() macro arg may be eval'd twice so calc value before invoking macro */
 	    HDR_UDP.checksum = 0;
-	    u = g_ntohs(in_checksum(&pseudoh, sizeof(pseudoh))) +
-		    g_ntohs(in_checksum(&HDR_UDP, sizeof(HDR_UDP))) +
-		    g_ntohs(in_checksum(packet_buf, curr_offset));
-	    HDR_UDP.checksum = g_htons((u & 0xffff) + (u>>16));
-	    if (HDR_UDP.checksum == 0) /* differenciate between 'none' and 0 */
+            x16  = in_checksum(&pseudoh, sizeof(pseudoh));
+            u    = g_ntohs(x16);
+            x16  = in_checksum(&HDR_UDP, sizeof(HDR_UDP));
+            u   += g_ntohs(x16);
+            x16  = in_checksum(packet_buf, curr_offset);
+            u   += g_ntohs(x16);
+            x16  = (u & 0xffff) + (u>>16);
+	    HDR_UDP.checksum = g_htons(x16);
+	    if (HDR_UDP.checksum == 0) /* differentiate between 'none' and 0 */
 	    	    HDR_UDP.checksum = g_htons(1);
 
             fwrite(&HDR_UDP, sizeof(HDR_UDP), 1, output_file);
@@ -616,18 +628,24 @@ write_current_packet (void)
 
         /* Write TCP header */
         if (hdr_tcp) {
+            guint16 x16;
             HDR_TCP.source_port = g_htons(hdr_src_port);
             HDR_TCP.dest_port = g_htons(hdr_dest_port);
     	    /* HDR_TCP.seq_num already correct */
 	    HDR_TCP.window = g_htons(0x2000);
 
+            /* Note: g_ntohs()/g_htons() macro arg may be eval'd twice so calc value before invoking macro */
 	    HDR_TCP.checksum = 0;
-	    u = g_ntohs(in_checksum(&pseudoh, sizeof(pseudoh))) +
-		    g_ntohs(in_checksum(&HDR_TCP, sizeof(HDR_TCP))) +
-		    g_ntohs(in_checksum(packet_buf, curr_offset));
-	    HDR_TCP.checksum = g_htons((u & 0xffff) + (u>>16));
-	    if (HDR_TCP.checksum == 0) /* differenciate between 'none' and 0 */
-	    	    HDR_TCP.checksum = g_htons(1);
+            x16  = in_checksum(&pseudoh, sizeof(pseudoh));
+            u    = g_ntohs(x16);
+            x16  = in_checksum(&HDR_TCP, sizeof(HDR_TCP));
+            u   += g_ntohs(x16);
+            x16  = in_checksum(packet_buf, curr_offset);
+            u   += g_ntohs(x16);
+            x16  = (u & 0xffff) + (u>>16);
+	    HDR_TCP.checksum = g_htons(x16);
+	    if (HDR_TCP.checksum == 0) /* differentiate between 'none' and 0 */
+                HDR_TCP.checksum = g_htons(1);
 
             fwrite(&HDR_TCP, sizeof(HDR_TCP), 1, output_file);
         }
@@ -649,6 +667,7 @@ write_current_packet (void)
 
         /* Write SCTP header */
         if (hdr_sctp) {
+            guint32 x32;
             HDR_SCTP.src_port  = g_htons(hdr_sctp_src);
             HDR_SCTP.dest_port = g_htons(hdr_sctp_dest);
             HDR_SCTP.tag       = g_htonl(hdr_sctp_tag);
@@ -656,7 +675,9 @@ write_current_packet (void)
             HDR_SCTP.checksum  = crc32c((guint8 *)&HDR_SCTP, sizeof(HDR_SCTP), ~0L);
             if (hdr_data_chunk)
               HDR_SCTP.checksum  = crc32c((guint8 *)&HDR_DATA_CHUNK, sizeof(HDR_DATA_CHUNK), HDR_SCTP.checksum);
-            HDR_SCTP.checksum  = g_htonl(finalize_crc32c(crc32c(packet_buf, curr_offset, HDR_SCTP.checksum)));
+            /* Note: g_ntohl() macro arg may be eval'd twice so calc value before invoking macro */
+            x32 = finalize_crc32c(crc32c(packet_buf, curr_offset, HDR_SCTP.checksum));
+            HDR_SCTP.checksum  = g_htonl(x32);
 
             fwrite(&HDR_SCTP, sizeof(HDR_SCTP), 1, output_file);
         }
@@ -678,8 +699,8 @@ write_current_packet (void)
             fprintf(stderr, "Wrote packet of %lu bytes at %u\n", curr_offset, g_ntohl(HDR_TCP.seq_num));
         num_packets_written ++;
     }
-
-    HDR_TCP.seq_num = g_htonl(g_ntohl(HDR_TCP.seq_num) + curr_offset);
+    HDR_TCP.seq_num = g_ntohl(HDR_TCP.seq_num) + curr_offset;
+    HDR_TCP.seq_num = g_htonl(HDR_TCP.seq_num);
 
     packet_start += curr_offset;
     curr_offset = 0;
@@ -767,7 +788,7 @@ parse_preamble (void)
 	 */
 	packet_preamble[packet_preamble_len] = '\0';
 
-	/* Ensure preamble has more than two chars before atempting to parse.
+	/* Ensure preamble has more than two chars before attempting to parse.
 	 * This should cover line breaks etc that get counted.
 	 */
 	if ( strlen(packet_preamble) > 2 ) {
@@ -1042,15 +1063,16 @@ usage (void)
             "      <outfile> specifies output filename (use - for standard output)\n"
             "\n"
             "Input:\n"
-            "  -o hex|oct|dec         parse offsets as (h)ex, (o)ctal or (d)ecimal; default is hex.\n"
+            "  -o hex|oct|dec         parse offsets as (h)ex, (o)ctal or (d)ecimal;\n"
+            "                         default is hex.\n"
             "  -t <timefmt>           treat the text before the packet as a date/time code;\n"
             "                         the specified argument is a format string of the sort\n"
             "                         supported by strptime.\n"
             "                         Example: The time \"10:15:14.5476\" has the format code\n"
             "                         \"%%H:%%M:%%S.\"\n"
-            "                         NOTE: The subsecond component delimiter must be given\n"
-            "                          (.) but no pattern is required; the remaining number\n"
-            "                          is assumed to be fractions of a second.\n"
+            "                         NOTE: The subsecond component delimiter, '.', must be\n"
+            "                         given, but no pattern is required; the remaining\n"
+            "                         number is assumed to be fractions of a second.\n"
             "                         NOTE: Date/time fields from the current date/time are\n"
             "                         used as the default for unspecified fields.\n"
             "\n"
@@ -1074,7 +1096,8 @@ usage (void)
             "  -u <srcp>,<destp>      prepend dummy UDP header with specified\n"
             "                         dest and source ports (in DECIMAL).\n"
             "                         Automatically prepends Ethernet & IP headers as well.\n"
-            "                         Example: -u 1000 69 to make the packets look like TFTP/UDP packets.\n" 
+            "                         Example: -u 1000,69 to make the packets look like\n"
+            "                         TFTP/UDP packets.\n"
             "  -T <srcp>,<destp>      prepend dummy TCP header with specified\n"
             "                         dest and source ports (in DECIMAL).\n"
             "                         Automatically prepends Ethernet & IP headers as well.\n"
@@ -1108,6 +1131,10 @@ parse_options (int argc, char *argv[])
     int c;
     char *p;
 
+#ifdef _WIN32
+    arg_list_utf_16to8(argc, argv);
+#endif /* _WIN32 */
+
     /* Scan CLI parameters */
     while ((c = getopt(argc, argv, "dhqe:i:l:m:o:u:s:S:t:T:")) != -1) {
         switch(c) {
@@ -1328,6 +1355,7 @@ parse_options (int argc, char *argv[])
 
     ts_sec = time(0);		/* initialize to current time */
     timecode_default = *localtime(&ts_sec);
+    timecode_default.tm_isdst = -1;	/* Unknown for now, depends on time given to the strptime() function */
 
     /* Display summary of our state */
     if (!quiet) {
@@ -1349,7 +1377,8 @@ parse_options (int argc, char *argv[])
     }
 }
 
-int main(int argc, char *argv[])
+int
+main(int argc, char *argv[])
 {
     parse_options(argc, argv);
 
