@@ -1,6 +1,6 @@
 /* 5views.c
  *
- * $Id: 5views.c 27181 2009-01-07 20:06:24Z wmeier $
+ * $Id: 5views.c 36748 2011-04-21 09:41:52Z guy $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -106,9 +106,9 @@ typedef struct
 static gboolean _5views_read(wtap *wth, int *err, gchar **err_info,
     gint64 *data_offset);
 static gboolean _5views_read_rec_data(FILE_T fh, guchar *pd, int length,
-    int *err);
+    int *err, gchar **err_info);
 static int _5views_read_header(wtap *wth, FILE_T fh,
-    t_5VW_TimeStamped_Header  *hdr, int *err);
+    t_5VW_TimeStamped_Header  *hdr, int *err, gchar **err_info);
 static gboolean _5views_seek_read(wtap *wth, gint64 seek_off,
     union wtap_pseudo_header *pseudo_header, guchar *pd, int length,
     int *err, gchar **err_info);
@@ -126,9 +126,9 @@ int _5views_open(wtap *wth, int *err, gchar **err_info)
 	int encap = WTAP_ENCAP_UNKNOWN;
 
 	errno = WTAP_ERR_CANT_READ;
-	bytes_read = file_read(&Capture_Header.Info_Header, 1, sizeof(t_5VW_Info_Header), wth->fh);
+	bytes_read = file_read(&Capture_Header.Info_Header, sizeof(t_5VW_Info_Header), wth->fh);
 	if (bytes_read != sizeof(t_5VW_Info_Header)) {
-		*err = file_error(wth->fh);
+		*err = file_error(wth->fh, err_info);
 		if (*err != 0)
 			return -1;
 		return 0;
@@ -182,9 +182,9 @@ int _5views_open(wtap *wth, int *err, gchar **err_info)
 	}
 
 	/* read the remaining header information */
-	bytes_read = file_read(&Capture_Header.HeaderDateCreation, 1, sizeof (t_5VW_Capture_Header) - sizeof(t_5VW_Info_Header), wth->fh);
+	bytes_read = file_read(&Capture_Header.HeaderDateCreation, sizeof (t_5VW_Capture_Header) - sizeof(t_5VW_Info_Header), wth->fh);
 	if (bytes_read != sizeof (t_5VW_Capture_Header)- sizeof(t_5VW_Info_Header) ) {
-		*err = file_error(wth->fh);
+		*err = file_error(wth->fh, err_info);
 		if (*err != 0)
 			return -1;
 		return 0;
@@ -204,7 +204,7 @@ int _5views_open(wtap *wth, int *err, gchar **err_info)
 
 /* Read the next packet */
 static gboolean
-_5views_read(wtap *wth, int *err, gchar **err_info _U_, gint64 *data_offset)
+_5views_read(wtap *wth, int *err, gchar **err_info, gint64 *data_offset)
 {
 	t_5VW_TimeStamped_Header TimeStamped_Header;
 	int	bytes_read;
@@ -213,7 +213,7 @@ _5views_read(wtap *wth, int *err, gchar **err_info _U_, gint64 *data_offset)
 
 	do
 	{
-		bytes_read = _5views_read_header(wth, wth->fh, &TimeStamped_Header, err);
+		bytes_read = _5views_read_header(wth, wth->fh, &TimeStamped_Header, err, err_info);
 		if (bytes_read == -1) {
 			/*
 			 * We failed to read the header.
@@ -245,7 +245,7 @@ _5views_read(wtap *wth, int *err, gchar **err_info _U_, gint64 *data_offset)
 
 	buffer_assure_space(wth->frame_buffer, packet_size);
 	if (!_5views_read_rec_data(wth->fh, buffer_start_ptr(wth->frame_buffer),
-	    packet_size, err))
+	    packet_size, err, err_info))
 		return FALSE;	/* Read error */
 
 	wth->data_offset += packet_size;
@@ -271,15 +271,16 @@ _5views_read(wtap *wth, int *err, gchar **err_info _U_, gint64 *data_offset)
 
 
 static gboolean
-_5views_read_rec_data(FILE_T fh, guchar *pd, int length, int *err)
+_5views_read_rec_data(FILE_T fh, guchar *pd, int length, int *err,
+   gchar **err_info)
 {
 	int	bytes_read;
 
 	errno = WTAP_ERR_CANT_READ;
-	bytes_read = file_read(pd, 1, length, fh);
+	bytes_read = file_read(pd, length, fh);
 
 	if (bytes_read != length) {
-		*err = file_error(fh);
+		*err = file_error(fh, err_info);
 		if (*err == 0)
 			*err = WTAP_ERR_SHORT_READ;
 		return FALSE;
@@ -294,16 +295,16 @@ _5views_read_rec_data(FILE_T fh, guchar *pd, int length, int *err)
 
    Return -1 on an error, or the number of bytes of header read on success. */
 static int
-_5views_read_header(wtap *wth _U_, FILE_T fh, t_5VW_TimeStamped_Header  *hdr,   int *err)
+_5views_read_header(wtap *wth _U_, FILE_T fh, t_5VW_TimeStamped_Header  *hdr,   int *err, gchar **err_info)
 {
 	int	bytes_read, bytes_to_read;
 
 	bytes_to_read = sizeof(t_5VW_TimeStamped_Header);
 
 	/* Read record header. */
-	bytes_read = file_read(hdr, 1, bytes_to_read, fh);
+	bytes_read = file_read(hdr, bytes_to_read, fh);
 	if (bytes_read != bytes_to_read) {
-		*err = file_error(fh);
+		*err = file_error(fh, err_info);
 		if (*err == 0 && bytes_read != 0) {
 			*err = WTAP_ERR_SHORT_READ;
 		}
@@ -316,14 +317,14 @@ _5views_read_header(wtap *wth _U_, FILE_T fh, t_5VW_TimeStamped_Header  *hdr,
 static gboolean
 _5views_seek_read(wtap *wth, gint64 seek_off,
     union wtap_pseudo_header *pseudo_header, guchar *pd, int length,
-    int *err, gchar **err_info _U_)
+    int *err, gchar **err_info)
 {
 	if (file_seek(wth->random_fh, seek_off, SEEK_SET, err) == -1)
 		return FALSE;
 	/*
 	 * Read the packet data.
 	 */
-	if (!_5views_read_rec_data(wth->random_fh, pd, length, err))
+	if (!_5views_read_rec_data(wth->random_fh, pd, length, err, err_info))
 		return FALSE;
 
 	switch (wth->file_encap) {
@@ -339,6 +340,10 @@ _5views_seek_read(wtap *wth, gint64 seek_off,
 
 
 
+typedef struct {
+	guint32	nframes;
+} _5views_dump_t;
+
 static const int wtap_encap[] = {
 	-1,		/* WTAP_ENCAP_UNKNOWN -> unsupported */
 	CST_5VW_CAPTURE_ETH_FILEID,		/* WTAP_ENCAP_ETHERNET -> Ehernet Ethernet */
@@ -373,8 +378,9 @@ int _5views_dump_can_write_encap(int encap)
 
 /* Returns TRUE on success, FALSE on failure; sets "*err" to an error code on
    failure */
-gboolean _5views_dump_open(wtap_dumper *wdh, gboolean cant_seek _U_, int *err)
+gboolean _5views_dump_open(wtap_dumper *wdh, int *err)
 {
+	_5views_dump_t *_5views;
 
 	/* We can't fill in all the fields in the file header, as we
 	   haven't yet written any packets.  As we'll have to rewrite
@@ -388,8 +394,9 @@ gboolean _5views_dump_open(wtap_dumper *wdh, gboolean cant_seek _U_, int *err)
 	/* This is a 5Views file */
 	wdh->subtype_write = _5views_dump;
 	wdh->subtype_close = _5views_dump_close;
-	wdh->dump._5views = g_malloc(sizeof(_5views_dump_t));
-	wdh->dump._5views->nframes = 0;
+	_5views = (_5views_dump_t *)g_malloc(sizeof(_5views_dump_t));
+	wdh->priv = (void *)_5views;
+	_5views->nframes = 0;
 
 	return TRUE;
 }
@@ -401,8 +408,7 @@ static gboolean _5views_dump(wtap_dumper *wdh,
 	const union wtap_pseudo_header *pseudo_header _U_,
 	const guchar *pd, int *err)
 {
-
-	size_t nwritten;
+	_5views_dump_t *_5views = (_5views_dump_t *)wdh->priv;
 	static t_5VW_TimeStamped_Header HeaderFrame;
 
 	/* Frame Header */
@@ -421,34 +427,23 @@ static gboolean _5views_dump(wtap_dumper *wdh,
 	HeaderFrame.RecInfo = htolel(0);
 
 	/* write the record header */
-	nwritten = fwrite(&HeaderFrame, 1, sizeof(t_5VW_TimeStamped_Header), wdh->fh);
-	if (nwritten != sizeof(t_5VW_TimeStamped_Header)) {
-		if (nwritten == 0 && ferror(wdh->fh))
-			*err = errno;
-		else
-			*err = WTAP_ERR_SHORT_WRITE;
+	if (!wtap_dump_file_write(wdh, &HeaderFrame,
+	    sizeof(t_5VW_TimeStamped_Header), err))
 		return FALSE;
-	}
 
 	/* write the data */
-	nwritten = fwrite(pd, 1, phdr->caplen, wdh->fh);
-	if (nwritten != phdr->caplen) {
-		if (nwritten == 0 && ferror(wdh->fh))
-			*err = errno;
-		else
-			*err = WTAP_ERR_SHORT_WRITE;
+	if (!wtap_dump_file_write(wdh, pd, phdr->caplen, err))
 		return FALSE;
-	}
 
-	wdh->dump._5views->nframes ++;
+	_5views->nframes ++;
 
 	return TRUE;
 }
 
 static gboolean _5views_dump_close(wtap_dumper *wdh, int *err)
 {
+	_5views_dump_t *_5views = (_5views_dump_t *)wdh->priv;
 	t_5VW_Capture_Header file_hdr;
-	size_t nwritten;
 
 	if (fseek(wdh->fh, 0, SEEK_SET) == -1) {
 		*err = errno;
@@ -486,17 +481,12 @@ static gboolean _5views_dump_close(wtap_dumper *wdh, int *err)
 	file_hdr.HeaderNbFrames.Nb = htoles(1);			/* Number of elements */
 
 	/* fill in the number of frames saved */
-	file_hdr.TramesStockeesInFile = htolel(wdh->dump._5views->nframes);
+	file_hdr.TramesStockeesInFile = htolel(_5views->nframes);
 
 	/* Write the file header. */
-	nwritten = fwrite(&file_hdr, 1, sizeof(t_5VW_Capture_Header), wdh->fh);
-	if (nwritten != sizeof(t_5VW_Capture_Header)) {
-		if (nwritten == 0 && ferror(wdh->fh))
-			*err = errno;
-		else
-			*err = WTAP_ERR_SHORT_WRITE;
+	if (!wtap_dump_file_write(wdh, &file_hdr, sizeof(t_5VW_Capture_Header),
+	    err))
 		return FALSE;
-	}
 
 	return TRUE;
 }
