@@ -2,7 +2,7 @@
  * Definitions for routines common to multiple modules in the Lucent/Ascend
  * capture file reading code code, but not used outside that code.
  *
- * $Id: ascend-int.h 19814 2006-11-05 22:46:44Z ulfl $
+ * $Id: ascend-int.h 37572 2011-06-06 16:39:23Z gerald $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -26,10 +26,16 @@
 #ifndef __ASCEND_INT_H__
 #define __ASCEND_INT_H__
 
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
+
+#include <glib.h>
+
 typedef struct {
   time_t start_time;
   time_t secs;
-  time_t usecs;
+  int usecs;
   guint32 caplen;
   guint32 len;
 } ascend_pkthdr;
@@ -48,7 +54,12 @@ int ascendlex(void);
 
 void init_parse_ascend(void);
 void ascend_init_lexer(FILE_T fh);
-int parse_ascend(FILE_T fh, guint8 *pd, struct ascend_phdr *phdr,
+typedef enum {
+    PARSED_RECORD,
+    PARSED_NONRECORD,
+    PARSE_FAILED
+} parse_t;
+parse_t parse_ascend(FILE_T fh, guint8 *pd, struct ascend_phdr *phdr,
 		ascend_pkthdr *hdr, gint64 *start_of_data);
 
 #endif /* ! __ASCEND_INT_H__ */
