@@ -1,6 +1,6 @@
 /* buffer.h
  *
- * $Id: buffer.h 23956 2007-12-28 07:38:06Z kukosa $
+ * $Id: buffer.h 37572 2011-06-06 16:39:23Z gerald $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -23,21 +23,22 @@
 
 #ifndef __W_BUFFER_H__
 #define __W_BUFFER_H__
+#include <glib.h>
 
 #define SOME_FUNCTIONS_ARE_DEFINES
 
 typedef struct Buffer {
-	guchar		*data;
-	unsigned int	allocated;
-	unsigned int	start;
-	unsigned int	first_free;
+	guchar	*data;
+	gsize	allocated;
+	gsize	start;
+	gsize	first_free;
 } Buffer;
 
-void buffer_init(Buffer* buffer, unsigned int space);
+void buffer_init(Buffer* buffer, gsize space);
 void buffer_free(Buffer* buffer);
-void buffer_assure_space(Buffer* buffer, unsigned int space);
-void buffer_append(Buffer* buffer, guchar *from, unsigned int bytes);
-void buffer_remove_start(Buffer* buffer, unsigned int bytes);
+void buffer_assure_space(Buffer* buffer, gsize space);
+void buffer_append(Buffer* buffer, guchar *from, gsize bytes);
+void buffer_remove_start(Buffer* buffer, gsize bytes);
 
 #ifdef SOME_FUNCTIONS_ARE_DEFINES
 # define buffer_clean(buffer) buffer_remove_start((buffer), buffer_length(buffer))
