@@ -1,6 +1,6 @@
 /* catapult_dct2000.c
  *
- * $Id: catapult_dct2000.c 30892 2009-11-09 14:24:02Z stig $
+ * $Id: catapult_dct2000.c 36575 2011-04-12 00:44:44Z guy $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -118,7 +118,6 @@ static void catapult_dct2000_close(wtap *wth);
 static gboolean catapult_dct2000_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
                                       const union wtap_pseudo_header *pseudo_header,
                                       const guchar *pd, int *err);
-static gboolean catapult_dct2000_dump_close(wtap_dumper *wdh, int *err);
 
 
 /************************************************************/
@@ -176,7 +175,7 @@ int catapult_dct2000_open(wtap *wth, int *err, gchar **err_info _U_)
     guint32 usecs;
     gint firstline_length = 0;
     dct2000_file_externals_t *file_externals;
-    gchar linebuff[MAX_LINE_LENGTH];
+    static gchar linebuff[MAX_LINE_LENGTH];
 
     /* Clear errno before reading from the file */
     errno = 0;
@@ -257,7 +256,7 @@ int catapult_dct2000_open(wtap *wth, int *err, gchar **err_info _U_)
         g_hash_table_new(packet_offset_hash_func, packet_offset_equal);
 
     /* Set this wtap to point to the file_externals */
-    wth->capture.generic = (void*)file_externals;
+    wth->priv = (void*)file_externals;
 
     *err = errno;
     return 1;
@@ -279,7 +278,7 @@ gboolean catapult_dct2000_read(wtap *wth, int *err, gchar **err_info _U_,
 
     /* Get wtap external structure for this wtap */
     dct2000_file_externals_t *file_externals =
-        (dct2000_file_externals_t*)wth->capture.generic;
+        (dct2000_file_externals_t*)wth->priv;
 
     /* There *has* to be an entry for this wth */
     if (!file_externals) {
@@ -291,7 +290,7 @@ gboolean catapult_dct2000_read(wtap *wth, int *err, gchar **err_info _U_,
         int line_length, seconds, useconds, data_chars;
         int is_comment = FALSE;
         gint64 this_offset = offset;
-        gchar linebuff[MAX_LINE_LENGTH+1];
+        static gchar linebuff[MAX_LINE_LENGTH+1];
         gchar aal_header_chars[AAL_HEADER_CHARS];
         gchar context_name[MAX_CONTEXT_NAME];
         guint8 context_port;
@@ -361,7 +360,7 @@ gboolean catapult_dct2000_read(wtap *wth, int *err, gchar **err_info _U_,
                                 strlen(protocol_name)+1 +    /* Protocol name */
                                 1 +                          /* direction */
                                 1 +                          /* encap */
-                                is_comment ? data_chars : (data_chars/2));
+                                (is_comment ? data_chars : (data_chars/2)));
             frame_buffer = buffer_start_ptr(wth->frame_buffer);
 
 
@@ -450,7 +449,7 @@ catapult_dct2000_seek_read(wtap *wth, gint64 seek_off,
 {
     gint64 offset = wth->data_offset;
     long dollar_offset, before_time_offset, after_time_offset;
-    gchar linebuff[MAX_LINE_LENGTH+1];
+    static gchar linebuff[MAX_LINE_LENGTH+1];
     gchar aal_header_chars[AAL_HEADER_CHARS];
     gchar context_name[MAX_CONTEXT_NAME];
     guint8 context_port;
@@ -542,7 +541,7 @@ void catapult_dct2000_close(wtap *wth)
 {
     /* Get externals for this file */
     dct2000_file_externals_t *file_externals =
-        (dct2000_file_externals_t*)wth->capture.generic;
+        (dct2000_file_externals_t*)wth->priv;
 
     /* The entry *has* to be found */
     if (!file_externals) {
@@ -554,9 +553,6 @@ void catapult_dct2000_close(wtap *wth)
                                 free_line_prefix_info, NULL);
     /* Free up its line prefix table */
     g_hash_table_destroy(file_externals->packet_prefix_table);
-
-    /* Can now free this too */
-    g_free(file_externals);
 }
 
 
@@ -566,15 +562,19 @@ void catapult_dct2000_close(wtap *wth)
 /* Dump functions          */
 /***************************/
 
+typedef struct {
+    gboolean           first_packet_written;
+    struct wtap_nstime start_time;
+} dct2000_dump_t;
+
 /*****************************************************/
 /* The file that we are writing to has been opened.  */
 /* Set other dump callbacks.                         */
 /*****************************************************/
-gboolean catapult_dct2000_dump_open(wtap_dumper *wdh, gboolean cant_seek _U_, int *err _U_)
+gboolean catapult_dct2000_dump_open(wtap_dumper *wdh, int *err _U_)
 {
     /* Fill in other dump callbacks */
     wdh->subtype_write = catapult_dct2000_dump;
-    wdh->subtype_close = catapult_dct2000_dump_close;
 
     return TRUE;
 }
@@ -601,24 +601,6 @@ int catapult_dct2000_dump_can_write_encap(int encap)
 /* Write a single packet out to the file */
 /*****************************************/
 
-static gboolean do_fwrite(const void *data, size_t size, size_t count, FILE *stream, int *err_p)
-{
-    size_t nwritten;
-
-    nwritten = fwrite(data, size, count, stream);
-    if (nwritten != count) {
-        if (nwritten == 0 && ferror(stream)) {
-            *err_p = errno;
-        }
-        else {
-            *err_p = WTAP_ERR_SHORT_WRITE;
-        }
-
-        return FALSE;
-    }
-    return TRUE;
-}
-
 gboolean catapult_dct2000_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
                                const union wtap_pseudo_header *pseudo_header,
                                const guchar *pd, int *err)
@@ -627,43 +609,48 @@ gboolean catapult_dct2000_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
     line_prefix_info_t *prefix = NULL;
     gchar time_string[16];
     gboolean is_comment;
+    dct2000_dump_t *dct2000;
 
     /******************************************************/
     /* Get the file_externals structure for this file */
     /* Find wtap external structure for this wtap */
     dct2000_file_externals_t *file_externals =
-        (dct2000_file_externals_t*)pseudo_header->dct2000.wth->capture.generic;
+        (dct2000_file_externals_t*)pseudo_header->dct2000.wth->priv;
 
-    if (wdh->dump.dct2000 == NULL) {
+    dct2000 = (dct2000_dump_t *)wdh->priv;
+    if (dct2000 == NULL) {
 
         /* Write out saved first line */
-        if (!do_fwrite(file_externals->firstline, 1, file_externals->firstline_length, wdh->fh, err)) {
+        if (!wtap_dump_file_write(wdh, file_externals->firstline,
+                                  file_externals->firstline_length, err)) {
             return FALSE;
         }
-        if (!do_fwrite("\n", 1, 1, wdh->fh, err)) {
+        if (!wtap_dump_file_write(wdh, "\n", 1, err)) {
             return FALSE;
         }
 
         /* Also write out saved second line with timestamp corresponding to the
            opening time of the log.
         */
-        if (!do_fwrite(file_externals->secondline, 1, file_externals->secondline_length, wdh->fh, err)) {
+        if (!wtap_dump_file_write(wdh, file_externals->secondline,
+                                  file_externals->secondline_length, err)) {
             return FALSE;
         }
-        if (!do_fwrite("\n", 1, 1, wdh->fh, err)) {
+        if (!wtap_dump_file_write(wdh, "\n", 1, err)) {
             return FALSE;
         }
 
         /* Allocate the dct2000-specific dump structure */
-        wdh->dump.dct2000 = g_malloc(sizeof(dct2000_dump_t));
+        dct2000 = (dct2000_dump_t *)g_malloc(sizeof(dct2000_dump_t));
+        wdh->priv = (void *)dct2000;
 
         /* Copy time of beginning of file */
-        wdh->dump.dct2000->start_time.secs = file_externals->start_secs;
-        wdh->dump.dct2000->start_time.nsecs =
+        dct2000->start_time.secs = file_externals->start_secs;
+        dct2000->start_time.nsecs =
             (file_externals->start_usecs * 1000);
 
         /* Set flag do don't write header out again */
-        wdh->dump.dct2000->first_packet_written = TRUE;
+        dct2000->first_packet_written = TRUE;
     }
 
 
@@ -675,7 +662,8 @@ gboolean catapult_dct2000_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
                                                       (const void*)&(pseudo_header->dct2000.seek_off));
 
     /* Write out text before timestamp */
-    if (!do_fwrite(prefix->before_time, 1, strlen(prefix->before_time), wdh->fh, err)) {
+    if (!wtap_dump_file_write(wdh, prefix->before_time,
+                              strlen(prefix->before_time), err)) {
         return FALSE;
     }
 
@@ -683,30 +671,31 @@ gboolean catapult_dct2000_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
     is_comment = (strstr(prefix->before_time, "/////") != NULL);
 
     /* Calculate time of this packet to write, relative to start of dump */
-    if (phdr->ts.nsecs >= wdh->dump.dct2000->start_time.nsecs) {
+    if (phdr->ts.nsecs >= dct2000->start_time.nsecs) {
         g_snprintf(time_string, 16, "%ld.%04d",
-                  (long)(phdr->ts.secs - wdh->dump.dct2000->start_time.secs),
-                  (phdr->ts.nsecs - wdh->dump.dct2000->start_time.nsecs) / 100000);
+                  (long)(phdr->ts.secs - dct2000->start_time.secs),
+                  (phdr->ts.nsecs - dct2000->start_time.nsecs) / 100000);
     }
     else {
         g_snprintf(time_string, 16, "%ld.%04u",
-                  (long)(phdr->ts.secs - wdh->dump.dct2000->start_time.secs-1),
-                  ((1000000000 + (phdr->ts.nsecs / 100000)) - (wdh->dump.dct2000->start_time.nsecs / 100000)) % 10000);
+                  (long)(phdr->ts.secs - dct2000->start_time.secs-1),
+                  ((1000000000 + (phdr->ts.nsecs / 100000)) - (dct2000->start_time.nsecs / 100000)) % 10000);
     }
 
     /* Write out the calculated timestamp */
-    if (!do_fwrite(time_string, 1, strlen(time_string), wdh->fh, err)) {
+    if (!wtap_dump_file_write(wdh, time_string, strlen(time_string), err)) {
         return FALSE;
     }
 
     /* Write out text between timestamp and start of hex data */
     if (prefix->after_time == NULL) {
-        if (!do_fwrite(" l ", 1, strlen(" l "), wdh->fh, err)) {
+        if (!wtap_dump_file_write(wdh, " l ", strlen(" l "), err)) {
             return FALSE;
         }
     }
     else {
-        if (!do_fwrite(prefix->after_time, 1, strlen(prefix->after_time), wdh->fh, err)) {
+        if (!wtap_dump_file_write(wdh, prefix->after_time,
+                                  strlen(prefix->after_time), err)) {
             return FALSE;
         }
     }
@@ -744,7 +733,7 @@ gboolean catapult_dct2000_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
 
     /**************************************/
     /* Remainder is encapsulated protocol */
-    if (!do_fwrite("$", 1, 1, wdh->fh, err)) {
+    if (!wtap_dump_file_write(wdh, "$", 1, err)) {
         return FALSE;
     }
 
@@ -756,7 +745,7 @@ gboolean catapult_dct2000_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
             c[1] = char_from_hex((guchar)(pd[n] & 0x0f));
 
             /* Write both hex chars of byte together */
-            if (!do_fwrite(c, 1, 2, wdh->fh, err)) {
+            if (!wtap_dump_file_write(wdh, c, 2, err)) {
                 return FALSE;
             }
         }
@@ -767,14 +756,14 @@ gboolean catapult_dct2000_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
             c[0] = pd[n];
 
             /* Write both hex chars of byte together */
-            if (!do_fwrite(c, 1, 1, wdh->fh, err)) {
+            if (!wtap_dump_file_write(wdh, c, 1, err)) {
                 return FALSE;
             }
         }
     }
 
     /* End the line */
-    if (!do_fwrite("\n", 1, 1, wdh->fh, err)) {
+    if (!wtap_dump_file_write(wdh, "\n", 1, err)) {
         return FALSE;
     }
 
@@ -782,17 +771,6 @@ gboolean catapult_dct2000_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
 }
 
 
-/******************************************************/
-/* Close a file we've been writing to.                */
-/******************************************************/
-static gboolean catapult_dct2000_dump_close(wtap_dumper *wdh _U_, int *err _U_)
-{
-    return TRUE;
-}
-
-
-
-
 /****************************/
 /* Private helper functions */
 /****************************/
@@ -806,10 +784,8 @@ static gboolean catapult_dct2000_dump_close(wtap_dumper *wdh _U_, int *err _U_)
 gboolean read_new_line(FILE_T fh, gint64 *offset, gint *length,
                        gchar *linebuff, size_t linebuffsize)
 {
-    char *result;
-
     /* Read in a line */
-    result = file_gets(linebuff, (int)linebuffsize - 1, fh);
+    char *result = file_gets(linebuff, (int)linebuffsize - 1, fh);
     if (result == NULL) {
         /* No characters found, or error */
         return FALSE;
@@ -888,10 +864,10 @@ static gboolean parse_line(gchar *linebuff, gint line_length,
         return FALSE;
     }
 
-    /* Reset strings (that won't be set be comments) */
-    g_strlcpy(variant_name, "0", MAX_VARIANT_DIGITS);
-    g_strlcpy(outhdr_name, "", MAX_OUTHDR_NAME);
-    g_strlcpy(port_number_string, "0", MAX_PORT_DIGITS);
+    /* Reset strings (that won't be set by comments) */
+    variant_name[0] = '\0';
+    outhdr_name[0] = '\0';
+    port_number_string[0] = '\0';
 
     if (!(*is_comment)) {
         /* '.' must follow context name */
@@ -1015,7 +991,8 @@ static gboolean parse_line(gchar *linebuff, gint line_length,
         (strcmp(protocol_name, "fp_r4") == 0) ||
         (strcmp(protocol_name, "fp_r5") == 0) ||
         (strcmp(protocol_name, "fp_r6") == 0) ||
-        (strcmp(protocol_name, "fp_r7") == 0)) {
+        (strcmp(protocol_name, "fp_r7") == 0) ||
+        (strcmp(protocol_name, "fp_r8") == 0)) {
 
         if ((variant > 256) && (variant % 256 == 3)) {
             /* FP over udp is contained in IPPrim... */
@@ -1136,7 +1113,7 @@ static gboolean parse_line(gchar *linebuff, gint line_length,
         n++;
     }
     else {
-        *direction = received;
+        *direction = sent;
     }
 
 
@@ -1367,7 +1344,7 @@ static void set_aal_info(union wtap_pseudo_header *pseudo_header,
     }
     else {
         pseudo_header->dct2000.inner_pseudo_header.atm.aal2_cid =
-            (int)aal_header_chars[11] - 48;
+            (int)aal_header_chars[11] - '0';
     }
 }
 
