@@ -3,7 +3,7 @@
  * and V7 files
  * Copyright (c) 2001, Daniel Thompson <d.thompson@gmx.net>
  *
- * $Id: etherpeek.c 15544 2005-08-26 19:40:46Z guy $
+ * $Id: etherpeek.c 36748 2011-04-21 09:41:52Z guy $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -142,25 +142,29 @@ static const etherpeek_encap_lookup_t etherpeek_encap[] = {
 #define NUM_ETHERPEEK_ENCAPS \
 	(sizeof (etherpeek_encap) / sizeof (etherpeek_encap[0]))
 
+typedef struct {
+	struct timeval reference_time;
+} etherpeek_t;
+
 static gboolean etherpeek_read_v7(wtap *wth, int *err, gchar **err_info,
-    long *data_offset);
-static gboolean etherpeek_seek_read_v7(wtap *wth, long seek_off,
+    gint64 *data_offset);
+static gboolean etherpeek_seek_read_v7(wtap *wth, gint64 seek_off,
     union wtap_pseudo_header *pseudo_header, guchar *pd, int length,
     int *err, gchar **err_info);
 static void etherpeek_fill_pseudo_header_v7(
     union wtap_pseudo_header *pseudo_header, airopeek_radio_hdr_t *radio_hdr);
 static gboolean etherpeek_read_v56(wtap *wth, int *err, gchar **err_info,
-    long *data_offset);
-static gboolean etherpeek_seek_read_v56(wtap *wth, long seek_off,
+    gint64 *data_offset);
+static gboolean etherpeek_seek_read_v56(wtap *wth, gint64 seek_off,
     union wtap_pseudo_header *pseudo_header, guchar *pd, int length,
     int *err, gchar **err_info);
-static void etherpeek_close(wtap *wth);
 
-int etherpeek_open(wtap *wth, int *err, gchar **err_info _U_)
+int etherpeek_open(wtap *wth, int *err, gchar **err_info)
 {
 	etherpeek_header_t ep_hdr;
 	struct timeval reference_time;
 	int file_encap;
+	etherpeek_t *etherpeek;
 
 	/* EtherPeek files do not start with a magic value large enough
 	 * to be unique; hence we use the following algorithm to determine
@@ -172,7 +176,7 @@ int etherpeek_open(wtap *wth, int *err, gchar **err_info _U_)
 	 */
 	g_assert(sizeof(ep_hdr.master) == ETHERPEEK_MASTER_HDR_SIZE);
 	wtap_file_read_unknown_bytes(
-		&ep_hdr.master, sizeof(ep_hdr.master), wth->fh, err);
+		&ep_hdr.master, sizeof(ep_hdr.master), wth->fh, err, err_info);
 	wth->data_offset += sizeof(ep_hdr.master);
 
 	/*
@@ -200,7 +204,7 @@ int etherpeek_open(wtap *wth, int *err, gchar **err_info _U_)
 		        ETHERPEEK_V567_HDR_SIZE);
 		wtap_file_read_unknown_bytes(
 			&ep_hdr.secondary.v567,
-			sizeof(ep_hdr.secondary.v567), wth->fh, err);
+			sizeof(ep_hdr.secondary.v567), wth->fh, err, err_info);
 		wth->data_offset += sizeof(ep_hdr.secondary.v567);
 
 		if ((0 != ep_hdr.secondary.v567.reserved[0]) ||
@@ -322,9 +326,9 @@ int etherpeek_open(wtap *wth, int *err, gchar **err_info _U_)
 	 * At this point we have recognised the file type and have populated
 	 * the whole ep_hdr structure in host byte order.
 	 */
-	wth->capture.etherpeek = g_malloc(sizeof(etherpeek_t));
-	wth->capture.etherpeek->reference_time = reference_time;
-	wth->subtype_close = etherpeek_close;
+	etherpeek = (etherpeek_t *)g_malloc(sizeof(etherpeek_t));
+	wth->priv = (void *)etherpeek;
+	etherpeek->reference_time = reference_time;
 	switch (ep_hdr.master.version) {
 
 	case 5:
@@ -352,24 +356,23 @@ int etherpeek_open(wtap *wth, int *err, gchar **err_info _U_)
 	}
 
 	wth->snapshot_length   = 0; /* not available in header */
-    wth->tsprecision = WTAP_FILE_TSPREC_USEC;
+	wth->tsprecision = WTAP_FILE_TSPREC_USEC;
 
 	return 1;
 }
 
-static void etherpeek_close(wtap *wth)
-{
-	g_free(wth->capture.etherpeek);
-}
-
 static gboolean etherpeek_read_v7(wtap *wth, int *err, gchar **err_info,
-    long *data_offset)
+    gint64 *data_offset)
 {
 	guchar ep_pkt[ETHERPEEK_V7_PKT_SIZE];
+#if 0
 	guint16 protoNum;
+#endif
 	guint16 length;
 	guint16 sliceLength;
+#if 0
 	guint8  flags;
+#endif
 	guint8  status;
 	guint64 timestamp;
 	time_t tsecs;
@@ -378,14 +381,19 @@ static gboolean etherpeek_read_v7(wtap *wth, int *err, gchar **err_info,
 
 	*data_offset = wth->data_offset;
 
-	wtap_file_read_expected_bytes(ep_pkt, sizeof(ep_pkt), wth->fh, err);
+	wtap_file_read_expected_bytes(ep_pkt, sizeof(ep_pkt), wth->fh, err,
+	    err_info);
 	wth->data_offset += sizeof(ep_pkt);
 
 	/* Extract the fields from the packet */
+#if 0
 	protoNum = pntohs(&ep_pkt[ETHERPEEK_V7_PROTONUM_OFFSET]);
+#endif
 	length = pntohs(&ep_pkt[ETHERPEEK_V7_LENGTH_OFFSET]);
 	sliceLength = pntohs(&ep_pkt[ETHERPEEK_V7_SLICE_LENGTH_OFFSET]);
+#if 0
 	flags = ep_pkt[ETHERPEEK_V7_FLAGS_OFFSET];
+#endif
 	status = ep_pkt[ETHERPEEK_V7_STATUS_OFFSET];
 	timestamp = pntohll(&ep_pkt[ETHERPEEK_V7_TIMESTAMP_OFFSET]);
 
@@ -417,7 +425,8 @@ static gboolean etherpeek_read_v7(wtap *wth, int *err, gchar **err_info,
 			*err_info = g_strdup("etherpeek: packet not long enough for 802.11 radio header");
 			return FALSE;
 		}
-		wtap_file_read_expected_bytes(&radio_hdr, 4, wth->fh, err);
+		wtap_file_read_expected_bytes(&radio_hdr, 4, wth->fh, err,
+		    err_info);
 
 		/*
 		 * We don't treat the radio information as packet data.
@@ -442,7 +451,7 @@ static gboolean etherpeek_read_v7(wtap *wth, int *err, gchar **err_info,
 	/* read the frame data */
 	buffer_assure_space(wth->frame_buffer, sliceLength);
 	wtap_file_read_expected_bytes(buffer_start_ptr(wth->frame_buffer),
-	                              sliceLength, wth->fh, err);
+	                              sliceLength, wth->fh, err, err_info);
 	wth->data_offset += sliceLength;
 
 	/* fill in packet header values */
@@ -468,7 +477,7 @@ static gboolean etherpeek_read_v7(wtap *wth, int *err, gchar **err_info,
 }
 
 static gboolean
-etherpeek_seek_read_v7(wtap *wth, long seek_off,
+etherpeek_seek_read_v7(wtap *wth, gint64 seek_off,
     union wtap_pseudo_header *pseudo_header, guchar *pd, int length,
     int *err, gchar **err_info)
 {
@@ -481,7 +490,7 @@ etherpeek_seek_read_v7(wtap *wth, long seek_off,
 
 	/* Read the packet header. */
 	wtap_file_read_expected_bytes(ep_pkt, sizeof(ep_pkt), wth->random_fh,
-	    err);
+	    err, err_info);
 	status = ep_pkt[ETHERPEEK_V7_STATUS_OFFSET];
 
 	switch (wth->file_encap) {
@@ -500,7 +509,7 @@ etherpeek_seek_read_v7(wtap *wth, long seek_off,
 			return FALSE;
 		}
 		wtap_file_read_expected_bytes(&radio_hdr, 4, wth->random_fh,
-		    err);
+		    err, err_info);
 
 		etherpeek_fill_pseudo_header_v7(pseudo_header,
 		    &radio_hdr);
@@ -518,7 +527,8 @@ etherpeek_seek_read_v7(wtap *wth, long seek_off,
 	 * XXX - should "errno" be set in "wtap_file_read_expected_bytes()"?
 	 */
 	errno = WTAP_ERR_CANT_READ;
-	wtap_file_read_expected_bytes(pd, length, wth->random_fh, err);
+	wtap_file_read_expected_bytes(pd, length, wth->random_fh, err,
+	    err_info);
 	return TRUE;
 }
 
@@ -532,17 +542,22 @@ etherpeek_fill_pseudo_header_v7(union wtap_pseudo_header *pseudo_header,
 	pseudo_header->ieee_802_11.signal_level = radio_hdr->signal_level;
 }
 
-static gboolean etherpeek_read_v56(wtap *wth, int *err, gchar **err_info _U_,
-    long *data_offset)
+static gboolean etherpeek_read_v56(wtap *wth, int *err, gchar **err_info,
+    gint64 *data_offset)
 {
+	etherpeek_t *etherpeek = (etherpeek_t *)wth->priv;
 	guchar ep_pkt[ETHERPEEK_V56_PKT_SIZE];
 	guint16 length;
 	guint16 sliceLength;
+#if 0
 	guint8  flags;
 	guint8  status;
+#endif
 	guint32 timestamp;
+#if 0
 	guint16 destNum;
 	guint16 srcNum;
+#endif
 	guint16 protoNum;
 	char    protoStr[8];
 	unsigned int i;
@@ -559,17 +574,22 @@ static gboolean etherpeek_read_v56(wtap *wth, int *err, gchar **err_info _U_,
 	 */
 	*data_offset = wth->data_offset;
 
-	wtap_file_read_expected_bytes(ep_pkt, sizeof(ep_pkt), wth->fh, err);
+	wtap_file_read_expected_bytes(ep_pkt, sizeof(ep_pkt), wth->fh, err,
+	    err_info);
 	wth->data_offset += sizeof(ep_pkt);
 
 	/* Extract the fields from the packet */
 	length = pntohs(&ep_pkt[ETHERPEEK_V56_LENGTH_OFFSET]);
 	sliceLength = pntohs(&ep_pkt[ETHERPEEK_V56_SLICE_LENGTH_OFFSET]);
+#if 0
 	flags = ep_pkt[ETHERPEEK_V56_FLAGS_OFFSET];
 	status = ep_pkt[ETHERPEEK_V56_STATUS_OFFSET];
+#endif
 	timestamp = pntohl(&ep_pkt[ETHERPEEK_V56_TIMESTAMP_OFFSET]);
+#if 0
 	destNum = pntohs(&ep_pkt[ETHERPEEK_V56_DESTNUM_OFFSET]);
 	srcNum = pntohs(&ep_pkt[ETHERPEEK_V56_SRCNUM_OFFSET]);
+#endif
 	protoNum = pntohs(&ep_pkt[ETHERPEEK_V56_PROTONUM_OFFSET]);
 	memcpy(protoStr, &ep_pkt[ETHERPEEK_V56_PROTOSTR_OFFSET],
 	    sizeof protoStr);
@@ -587,14 +607,14 @@ static gboolean etherpeek_read_v56(wtap *wth, int *err, gchar **err_info _U_,
 	/* read the frame data */
 	buffer_assure_space(wth->frame_buffer, sliceLength);
 	wtap_file_read_expected_bytes(buffer_start_ptr(wth->frame_buffer),
-	                              sliceLength, wth->fh, err);
+	                              sliceLength, wth->fh, err, err_info);
 	wth->data_offset += sliceLength;
 
 	/* fill in packet header values */
 	wth->phdr.len        = length;
 	wth->phdr.caplen     = sliceLength;
 	/* timestamp is in milliseconds since reference_time */
-	wth->phdr.ts.secs  = wth->capture.etherpeek->reference_time.tv_sec
+	wth->phdr.ts.secs  = etherpeek->reference_time.tv_sec
 	    + (timestamp / 1000);
 	wth->phdr.ts.nsecs = 1000 * (timestamp % 1000) * 1000;
 
@@ -616,9 +636,9 @@ static gboolean etherpeek_read_v56(wtap *wth, int *err, gchar **err_info _U_,
 }
 
 static gboolean
-etherpeek_seek_read_v56(wtap *wth, long seek_off,
+etherpeek_seek_read_v56(wtap *wth, gint64 seek_off,
     union wtap_pseudo_header *pseudo_header, guchar *pd, int length,
-    int *err, gchar **err_info _U_)
+    int *err, gchar **err_info)
 {
 	guchar ep_pkt[ETHERPEEK_V56_PKT_SIZE];
 	int pkt_encap;
@@ -629,7 +649,7 @@ etherpeek_seek_read_v56(wtap *wth, long seek_off,
 		return FALSE;
 
 	wtap_file_read_expected_bytes(ep_pkt, sizeof(ep_pkt), wth->random_fh,
-	    err);
+	    err, err_info);
 
 	protoNum = pntohs(&ep_pkt[ETHERPEEK_V56_PROTONUM_OFFSET]);
 	pkt_encap = WTAP_ENCAP_UNKNOWN;
@@ -651,6 +671,7 @@ etherpeek_seek_read_v56(wtap *wth, long seek_off,
 	 * XXX - should "errno" be set in "wtap_file_read_expected_bytes()"?
 	 */
 	errno = WTAP_ERR_CANT_READ;
-	wtap_file_read_expected_bytes(pd, length, wth->random_fh, err);
+	wtap_file_read_expected_bytes(pd, length, wth->random_fh, err,
+	    err_info);
 	return TRUE;
 }
