@@ -1,6 +1,6 @@
 /* file_access.c
  *
- * $Id: file_access.c 30588 2009-10-17 20:56:06Z stig $
+ * $Id: file_access.c 37026 2011-05-09 08:12:26Z guy $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -83,6 +83,7 @@
 #include "daintree-sna.h"
 #include "netscaler.h"
 #include "jpeg_jfif.h"
+#include "ipfix.h"
 
 
 /* The open_file_* routines should return:
@@ -145,6 +146,7 @@ static wtap_open_routine_t open_routines_base[] = {
 	 */
 	netscreen_open,
 	erf_open,
+	ipfix_open,
 	k12text_open,
 	etherpeek_open,
 	pppdump_open,
@@ -168,7 +170,7 @@ static wtap_open_routine_t* open_routines = NULL;
 static GArray* open_routines_arr = NULL;
 
 
-/* initialize the open routines array if it has not being initialized yet */
+/* initialize the open routines array if it has not been initialized yet */
 static void init_open_routines(void) {
 
 	if (open_routines_arr) return;
@@ -177,7 +179,7 @@ static void init_open_routines(void) {
 
 	g_array_append_vals(open_routines_arr,open_routines_base,N_FILE_TYPES);
 
-	open_routines = (void*)open_routines_arr->data;
+	open_routines = (wtap_open_routine_t*)(void *)open_routines_arr->data;
 }
 
 void wtap_register_open_routine(wtap_open_routine_t open_routine, gboolean has_magic) {
@@ -188,7 +190,7 @@ void wtap_register_open_routine(wtap_open_routine_t open_routine, gboolean has_m
 	else
 		g_array_append_val(open_routines_arr,open_routine);
 
-	open_routines = (void*)open_routines_arr->data;
+	open_routines = (wtap_open_routine_t*)(void *)open_routines_arr->data;
 }
 
 /*
@@ -220,7 +222,8 @@ void wtap_register_open_routine(wtap_open_routine_t open_routine, gboolean has_m
 wtap* wtap_open_offline(const char *filename, int *err, char **err_info,
 			gboolean do_random)
 {
-	struct stat statb;
+	int	fd;
+	ws_statb64 statb;
 	wtap	*wth;
 	unsigned int	i;
 	gboolean use_stdin = FALSE;
@@ -231,12 +234,12 @@ wtap* wtap_open_offline(const char *filename, int *err, char **err_info,
 
 	/* First, make sure the file is valid */
 	if (use_stdin) {
-		if (fstat(0, &statb) < 0) {
+		if (ws_fstat64(0, &statb) < 0) {
 			*err = errno;
 			return NULL;
 		}
 	} else {
-		if (ws_stat(filename, &statb) < 0) {
+		if (ws_stat64(filename, &statb) < 0) {
 			*err = errno;
 			return NULL;
 		}
@@ -284,40 +287,46 @@ wtap* wtap_open_offline(const char *filename, int *err, char **err_info,
 	}
 
 	errno = ENOMEM;
-	wth = g_malloc(sizeof(wtap));
-	if (wth == NULL) {
-		*err = errno;
-		return NULL;
-	}
+	wth = (wtap *)g_malloc0(sizeof(wtap));
 
 	/* Open the file */
 	errno = WTAP_ERR_CANT_OPEN;
 	if (use_stdin) {
 		/*
 		 * We dup FD 0, so that we don't have to worry about
-		 * an fclose or gzclose of wth->fh closing the standard
+		 * a file_close of wth->fh closing the standard
 		 * input of the process.
 		 */
-		wth->fd = ws_dup(0);
+		fd = ws_dup(0);
+		if (fd < 0) {
+			*err = errno;
+			g_free(wth);
+			return NULL;
+		}
 #ifdef _WIN32
-		_setmode(wth->fd, O_BINARY);
+		if (_setmode(fd, O_BINARY) == -1) {
+			/* "Shouldn't happen" */
+			*err = errno;
+			g_free(wth);
+			return NULL;
+		}
 #endif
-	} else
-		wth->fd = ws_open(filename, O_RDONLY|O_BINARY, 0000 /* no creation so don't matter */);
-	if (wth->fd < 0) {
-		*err = errno;
-		g_free(wth);
-		return NULL;
-	}
-	if (!(wth->fh = filed_open(wth->fd, "rb"))) {
-		*err = errno;
-		ws_close(wth->fd);
-		g_free(wth);
-		return NULL;
+		if (!(wth->fh = filed_open(fd))) {
+			*err = errno;
+			ws_close(fd);
+			g_free(wth);
+			return NULL;
+		}
+	} else {
+		if (!(wth->fh = file_open(filename))) {
+			*err = errno;
+			g_free(wth);
+			return NULL;
+		}
 	}
 
 	if (do_random) {
-		if (!(wth->random_fh = file_open(filename, "rb"))) {
+		if (!(wth->random_fh = file_open(filename))) {
 			*err = errno;
 			file_close(wth->fh);
 			g_free(wth);
@@ -332,8 +341,15 @@ wtap* wtap_open_offline(const char *filename, int *err, char **err_info,
 	wth->subtype_sequential_close = NULL;
 	wth->subtype_close = NULL;
 	wth->tsprecision = WTAP_FILE_TSPREC_USEC;
+	wth->priv = NULL;
 
 	init_open_routines();
+	if (wth->random_fh) {
+		wth->fast_seek = g_ptr_array_new();
+
+		file_set_random_access(wth->fh, FALSE, wth->fast_seek);
+		file_set_random_access(wth->random_fh, TRUE, wth->fast_seek);
+	}
 
 	/* Try all file types */
 	for (i = 0; i < open_routines_arr->len; i++) {
@@ -375,15 +391,12 @@ wtap* wtap_open_offline(const char *filename, int *err, char **err_info,
 	}
 
 	/* Well, it's not one of the types of file we know about. */
-	if (wth->random_fh != NULL)
-		file_close(wth->random_fh);
-	file_close(wth->fh);
-	g_free(wth);
+	wtap_close(wth);
 	*err = WTAP_ERR_FILE_UNKNOWN_FORMAT;
 	return NULL;
 
 success:
-	wth->frame_buffer = g_malloc(sizeof(struct Buffer));
+	wth->frame_buffer = (struct Buffer *)g_malloc(sizeof(struct Buffer));
 	buffer_init(wth->frame_buffer, 1500);
 	return wth;
 }
@@ -391,239 +404,248 @@ success:
 /* Table of the file types we know about. */
 static const struct file_type_info dump_open_table_base[] = {
 	/* WTAP_FILE_UNKNOWN (only used internally for initialization) */
-	{ NULL, NULL, NULL, NULL, FALSE,
+	{ NULL, NULL, NULL, NULL, FALSE, FALSE,
 	  NULL, NULL },
 
 	/* WTAP_FILE_WTAP (only used internally while capturing) */
-	{ NULL, NULL, NULL, NULL, FALSE,
+	{ NULL, NULL, NULL, NULL, FALSE, FALSE,
 	  NULL, NULL },
 
 	/* WTAP_FILE_PCAP */
-	{ "Wireshark/tcpdump/... - libpcap", "libpcap", "*.pcap;*.cap", ".pcap", TRUE,
+	{ "Wireshark/tcpdump/... - libpcap", "libpcap", "*.pcap;*.cap", ".pcap", FALSE, FALSE,
 	  libpcap_dump_can_write_encap, libpcap_dump_open },
 
 	/* WTAP_FILE_PCAP_NSEC */
-	{ "Wireshark - nanosecond libpcap", "nseclibpcap", "*.pcap;*.cap", ".pcap", TRUE,
+	{ "Wireshark - nanosecond libpcap", "nseclibpcap", "*.pcap;*.cap", ".pcap", FALSE, FALSE,
 	  libpcap_dump_can_write_encap, libpcap_dump_open },
 
 	/* WTAP_FILE_PCAP_AIX */
-	{ "AIX tcpdump - libpcap", "aixlibpcap", "*.pcap;*.cap", ".pcap", TRUE,
+	{ "AIX tcpdump - libpcap", "aixlibpcap", "*.pcap;*.cap", ".pcap", FALSE, FALSE,
 	  NULL, NULL },
 
 	/* WTAP_FILE_PCAP_SS991029 */
-	{ "Modified tcpdump - libpcap", "modlibpcap", "*.pcap;*.cap", ".pcap", TRUE,
+	{ "Modified tcpdump - libpcap", "modlibpcap", "*.pcap;*.cap", ".pcap", FALSE, FALSE,
 	  libpcap_dump_can_write_encap, libpcap_dump_open },
 
 	/* WTAP_FILE_PCAP_NOKIA */
-	{ "Nokia tcpdump - libpcap ", "nokialibpcap", "*.pcap;*.cap", ".pcap", TRUE,
+	{ "Nokia tcpdump - libpcap ", "nokialibpcap", "*.pcap;*.cap", ".pcap", FALSE, FALSE,
 	  libpcap_dump_can_write_encap, libpcap_dump_open },
 
 	/* WTAP_FILE_PCAP_SS990417 */
-	{ "RedHat 6.1 tcpdump - libpcap", "rh6_1libpcap", "*.pcap;*.cap", ".pcap", TRUE,
+	{ "RedHat 6.1 tcpdump - libpcap", "rh6_1libpcap", "*.pcap;*.cap", ".pcap", FALSE, FALSE,
 	  libpcap_dump_can_write_encap, libpcap_dump_open },
 
 	/* WTAP_FILE_PCAP_SS990915 */
-	{ "SuSE 6.3 tcpdump - libpcap", "suse6_3libpcap", "*.pcap;*.cap", ".pcap", TRUE,
+	{ "SuSE 6.3 tcpdump - libpcap", "suse6_3libpcap", "*.pcap;*.cap", ".pcap", FALSE, FALSE,
 	  libpcap_dump_can_write_encap, libpcap_dump_open },
 
 	/* WTAP_FILE_5VIEWS */
-	{ "Accellent 5Views capture", "5views", "*.5vw", ".5vw", FALSE,
+	{ "Accellent 5Views capture", "5views", "*.5vw", ".5vw", TRUE, FALSE,
 	  _5views_dump_can_write_encap, _5views_dump_open },
 
 	/* WTAP_FILE_IPTRACE_1_0 */
-	{ "AIX iptrace 1.0", "iptrace_1", "*.*", NULL, FALSE,
+	{ "AIX iptrace 1.0", "iptrace_1", "*.*", NULL, FALSE, FALSE,
 	  NULL, NULL },
 
 	/* WTAP_FILE_IPTRACE_2_0 */
-	{ "AIX iptrace 2.0", "iptrace_2", "*.*", NULL, FALSE,
+	{ "AIX iptrace 2.0", "iptrace_2", "*.*", NULL, FALSE, FALSE,
 	  NULL, NULL },
 
 	/* WTAP_FILE_BER */
-	{ "ASN.1 Basic Encoding Rules", "ber", "*.*", NULL, FALSE,
+	{ "ASN.1 Basic Encoding Rules", "ber", "*.*", NULL, FALSE, FALSE,
 		NULL, NULL },
 
 	/* WTAP_FILE_HCIDUMP */
-	{ "Bluetooth HCI dump", "hcidump", "*.*", NULL, FALSE,
+	{ "Bluetooth HCI dump", "hcidump", "*.*", NULL, FALSE, FALSE,
 	  NULL, NULL },
 
 	/* WTAP_FILE_CATAPULT_DCT2000 */
-	{ "Catapult DCT2000 trace (.out format)", "dct2000", "*.out", ".out", FALSE,
+	{ "Catapult DCT2000 trace (.out format)", "dct2000", "*.out", ".out", FALSE, FALSE,
 	  catapult_dct2000_dump_can_write_encap, catapult_dct2000_dump_open },
 
 	/* WTAP_FILE_NETXRAY_OLD */
-	{ "Cinco Networks NetXRay 1.x", "netxray1", "*.cap", ".cap", FALSE,
+	{ "Cinco Networks NetXRay 1.x", "netxray1", "*.cap", ".cap", TRUE, FALSE,
 	  NULL, NULL },
 
 	/* WTAP_FILE_NETXRAY_1_0 */
-	{ "Cinco Networks NetXRay 2.0 or later", "netxray2", "*.cap", ".cap", FALSE,
+	{ "Cinco Networks NetXRay 2.0 or later", "netxray2", "*.cap", ".cap", TRUE, FALSE,
 	  NULL, NULL },
 
 	/* WTAP_FILE_COSINE */
-	{ "CoSine IPSX L2 capture", "cosine", "*.*", NULL, FALSE,
+	{ "CoSine IPSX L2 capture", "cosine", "*.*", NULL, FALSE, FALSE,
 	  NULL, NULL },
 
 	/* WTAP_FILE_CSIDS */
-	{ "CSIDS IPLog", "csids", "*.*", NULL, FALSE,
+	{ "CSIDS IPLog", "csids", "*.*", NULL, FALSE, FALSE,
 	  NULL, NULL },
 
 	/* WTAP_FILE_DBS_ETHERWATCH */
-	{ "DBS Etherwatch (VMS)", "etherwatch", "*.*", NULL, FALSE,
+	{ "DBS Etherwatch (VMS)", "etherwatch", "*.*", NULL, FALSE, FALSE,
 	  NULL, NULL},
 
 	/* WTAP_FILE_ERF */
-	{ "Endace ERF capture", "erf", "*.erf", ".erf", FALSE,
+	{ "Endace ERF capture", "erf", "*.erf", ".erf", FALSE, FALSE,
 	  NULL, NULL },
 
 	/* WTAP_FILE_EYESDN */
-	{ "EyeSDN USB S0/E1 ISDN trace format", "eyesdn", "*.*", NULL, FALSE,
-		NULL, NULL },
+	{ "EyeSDN USB S0/E1 ISDN trace format", "eyesdn", "*.trc", ".trc", FALSE, FALSE,
+	   eyesdn_dump_can_write_encap, eyesdn_dump_open },
 
 	/* WTAP_FILE_NETTL */
-	{ "HP-UX nettl trace", "nettl", "*.TRC0;*.TRC1", ".TRC0", FALSE,
+	{ "HP-UX nettl trace", "nettl", "*.TRC0;*.TRC1", ".TRC0", FALSE, FALSE,
 	  nettl_dump_can_write_encap, nettl_dump_open },
 
 	/* WTAP_FILE_ISERIES */
-	{ "IBM iSeries comm. trace (ASCII)", "iseries_ascii", "*.*", NULL, FALSE,
+	{ "IBM iSeries comm. trace (ASCII)", "iseries_ascii", "*.*", NULL, FALSE, FALSE,
 	  NULL, NULL },
 
 	/* WTAP_FILE_ISERIES_UNICODE */
-	{ "IBM iSeries comm. trace (UNICODE)", "iseries_unicode", "*.*", NULL, FALSE,
+	{ "IBM iSeries comm. trace (UNICODE)", "iseries_unicode", "*.*", NULL, FALSE, FALSE,
 	  NULL, NULL },
 
 	/* WTAP_FILE_I4BTRACE */
-	{ "I4B ISDN trace", "i4btrace", "*.*", NULL, FALSE,
+	{ "I4B ISDN trace", "i4btrace", "*.*", NULL, FALSE, FALSE,
 	  NULL, NULL },
 
 	/* WTAP_FILE_ASCEND */
-	{ "Lucent/Ascend access server trace", "ascend", "*.*", NULL, FALSE,
+	{ "Lucent/Ascend access server trace", "ascend", "*.*", NULL, FALSE, FALSE,
 	  NULL, NULL },
 
 	/* WTAP_FILE_NETMON_1_x */
-	{ "Microsoft NetMon 1.x", "netmon1", "*.cap", ".cap", FALSE,
+	{ "Microsoft NetMon 1.x", "netmon1", "*.cap", ".cap", TRUE, FALSE,
 	  netmon_dump_can_write_encap, netmon_dump_open },
 
 	/* WTAP_FILE_NETMON_2_x */
-	{ "Microsoft NetMon 2.x", "netmon2", "*.cap", ".cap", FALSE,
+	{ "Microsoft NetMon 2.x", "netmon2", "*.cap", ".cap", TRUE, FALSE,
 	  netmon_dump_can_write_encap, netmon_dump_open },
 
 	/* WTAP_FILE_NGSNIFFER_UNCOMPRESSED */
-	{ "NA Sniffer (DOS)", "ngsniffer", "*.cap;*.enc;*.trc;*.fdc;*.syc", ".cap", FALSE,
+	{ "NA Sniffer (DOS)", "ngsniffer", "*.cap;*.enc;*.trc;*.fdc;*.syc", ".cap", FALSE, FALSE,
 	  ngsniffer_dump_can_write_encap, ngsniffer_dump_open },
 
 	/* WTAP_FILE_NGSNIFFER_COMPRESSED */
-	{ "NA Sniffer (DOS), compressed", "ngsniffer_comp", "*.caz", ".caz", FALSE,
+	{ "NA Sniffer (DOS), compressed", "ngsniffer_comp", "*.caz", ".caz", FALSE, FALSE,
 	  NULL, NULL },
 
 	/* WTAP_FILE_NETXRAY_1_1 */
-	{ "NA Sniffer (Windows) 1.1", "ngwsniffer_1_1", "*.cap", ".cap", FALSE,
+	{ "NA Sniffer (Windows) 1.1", "ngwsniffer_1_1", "*.cap", ".cap", TRUE, FALSE,
 	  netxray_dump_can_write_encap_1_1, netxray_dump_open_1_1 },
 
 	/* WTAP_FILE_NETXRAY_2_00x */
-	{ "NA Sniffer (Windows) 2.00x", "ngwsniffer_2_0", "*.cap", ".cap", FALSE,
+	{ "NA Sniffer (Windows) 2.00x", "ngwsniffer_2_0", "*.cap", ".cap", TRUE, FALSE,
 	  netxray_dump_can_write_encap_2_0, netxray_dump_open_2_0 },
 
-	/* WTAP_FILE_NETWORK_INSTRUMENTS_V9 */
-	{ "Network Instruments Observer (V9)", "niobserverv9", "*.bfr", ".bfr", FALSE,
+	/* WTAP_FILE_NETWORK_INSTRUMENTS */
+	{ "Network Instruments Observer", "niobserverv", "*.bfr", ".bfr", FALSE, FALSE,
 	  network_instruments_dump_can_write_encap, network_instruments_dump_open },
 
 	/* WTAP_FILE_LANALYZER */
-	{ "Novell LANalyzer","lanalyzer", "*.tr1", ".tr1", FALSE,
+	{ "Novell LANalyzer","lanalyzer", "*.tr1", ".tr1", TRUE, FALSE,
 	  lanalyzer_dump_can_write_encap, lanalyzer_dump_open },
 
 	/* WTAP_FILE_PPPDUMP */
-	{ "pppd log (pppdump format)", "pppd", "*.*", NULL, FALSE,
+	{ "pppd log (pppdump format)", "pppd", "*.*", NULL, FALSE, FALSE,
 	  NULL, NULL },
 
 	/* WTAP_FILE_RADCOM */
-	{ "RADCOM WAN/LAN analyzer", "radcom", "*.*", NULL, FALSE,
+	{ "RADCOM WAN/LAN analyzer", "radcom", "*.*", NULL, FALSE, FALSE,
 	  NULL, NULL },
 
 	/* WTAP_FILE_SNOOP */
-	{ "Sun snoop", "snoop", "*.snoop;*.cap", ".snoop", FALSE,
+	{ "Sun snoop", "snoop", "*.snoop;*.cap", ".snoop", FALSE, FALSE,
 	  snoop_dump_can_write_encap, snoop_dump_open },
 
 	/* WTAP_FILE_SHOMITI */
-	{ "Shomiti/Finisar Surveyor", "shomiti", "*.cap", ".cap", FALSE,
+	{ "Shomiti/Finisar Surveyor", "shomiti", "*.cap", ".cap", FALSE, FALSE,
 	  NULL, NULL },
 
 	/* WTAP_FILE_VMS */
-	{ "TCPIPtrace (VMS)", "tcpiptrace", "*.*", NULL, FALSE,
+	{ "TCPIPtrace (VMS)", "tcpiptrace", "*.*", NULL, FALSE, FALSE,
 	  NULL, NULL},
 
 	/* WTAP_FILE_K12 */
-	{ "Tektronix K12xx 32-bit .rf5 format", "rf5", "*.rf5", ".rf5", TRUE,
+	{ "Tektronix K12xx 32-bit .rf5 format", "rf5", "*.rf5", ".rf5", TRUE, FALSE,
 		k12_dump_can_write_encap, k12_dump_open },
 
 	/* WTAP_FILE_TOSHIBA */
-	{ "Toshiba Compact ISDN Router snoop", "toshiba", "*.*", NULL, FALSE,
+	{ "Toshiba Compact ISDN Router snoop", "toshiba", "*.*", NULL, FALSE, FALSE,
 	  NULL, NULL },
 
 	/* WTAP_FILE_VISUAL_NETWORKS */
-	{ "Visual Networks traffic capture", "visual", "*.*", NULL, FALSE,
+	{ "Visual Networks traffic capture", "visual", "*.*", NULL, TRUE, FALSE,
 	  visual_dump_can_write_encap, visual_dump_open },
 
 	/* WTAP_FILE_ETHERPEEK_V56 */
-	{ "Wildpacket Ether/TokenPeek (V5 & V6)", "peek56", "*.tpc;*.apc;*.pkt;*.wpz", ".pkt", FALSE,
+	{ "WildPackets Ether/TokenPeek (V5 & V6)", "peek56", "*.tpc;*.apc;*.pkt;*.wpz", ".pkt", FALSE, FALSE,
 	  NULL, NULL },
 
 	/* WTAP_FILE_ETHERPEEK_V7 */
-	{ "Wildpacket Ether/Token/AiroPeek (V7)", "peek7", "*.tpc;*.apc;*.pkt;*.wpz", ".pkt", FALSE,
+	{ "WildPackets Ether/Token/AiroPeek (V7)", "peek7", "*.tpc;*.apc;*.pkt;*.wpz", ".pkt", FALSE, FALSE,
 	  NULL, NULL },
 
 	/* WTAP_FILE_ETHERPEEK_V9 */
-	{ "Wildpacket Ether/AiroPeek (V9)", "peek9", "*.tpc;*.apc;*.pkt;*.wpz", ".pkt", FALSE,
+	{ "WildPackets Ether/AiroPeek (V9)", "peek9", "*.tpc;*.apc;*.pkt;*.wpz", ".pkt", FALSE, FALSE,
 	  NULL, NULL },
 
 	/* WTAP_FILE_MPEG */
-	{ "MPEG", "mpeg", "*.mpeg;*.mpg;*.mp3", ".mpeg", FALSE,
+	{ "MPEG", "mpeg", "*.mpeg;*.mpg;*.mp3", ".mpeg", FALSE, FALSE,
 	  NULL, NULL },
 
 	/* WTAP_FILE_K12TEXT  */
-	{ "K12 text file", "k12text", "*.txt", ".txt", TRUE,
+	{ "K12 text file", "k12text", "*.txt", ".txt", FALSE, FALSE,
 	  k12text_dump_can_write_encap, k12text_dump_open },
 
 	/* WTAP_FILE_NETSCREEN */
-	{ "NetScreen snoop text file", "netscreen", "*.*", NULL, FALSE,
+	{ "NetScreen snoop text file", "netscreen", "*.*", NULL, FALSE, FALSE,
 	  NULL, NULL },
 
 	/* WTAP_FILE_COMMVIEW */
-	{ "TamoSoft CommView", "commview", "*.ncf", ".ncf", TRUE,
+	{ "TamoSoft CommView", "commview", "*.ncf", ".ncf", FALSE, FALSE,
 	  commview_dump_can_write_encap, commview_dump_open },
 
 	/* WTAP_FILE_PCAPNG */
-	{ "Wireshark - pcapng (experimental)", "pcapng", "*.pcapng", NULL, FALSE,
+	{ "Wireshark - pcapng", "pcapng", "*.pcapng", NULL, FALSE, TRUE,
 	  pcapng_dump_can_write_encap, pcapng_dump_open },
 
 	/* WTAP_FILE_BTSNOOP */
-	{ "Symbian OS btsnoop", "btsnoop", "*.log", ".log", FALSE,
+	{ "Symbian OS btsnoop", "btsnoop", "*.log", ".log", FALSE, FALSE,
 	  btsnoop_dump_can_write_encap, btsnoop_dump_open_h4 },
-        
+
 	/* WTAP_FILE_X2E_XORAYA */
-	{ NULL, NULL, NULL, NULL, FALSE, NULL, NULL },
+	{ NULL, NULL, NULL, NULL, FALSE, FALSE,
+	  NULL, NULL },
 
 	/* WTAP_FILE_TNEF */
-	{ "Transport-Neutral Encapsulation Format", "tnef", "*.*", NULL, FALSE, NULL, NULL },
+	{ "Transport-Neutral Encapsulation Format", "tnef", "*.*", NULL, FALSE, FALSE,
+	  NULL, NULL },
 
 	/* WTAP_FILE_DCT3TRACE */
-	{ "Gammu DCT3 trace", "dct3trace", "*.xml", NULL, FALSE, NULL, NULL },
+	{ "Gammu DCT3 trace", "dct3trace", "*.xml", NULL, FALSE, FALSE,
+	  NULL, NULL },
 
 	/* WTAP_FILE_PACKETLOGGER */
-	{ "PacketLogger", "pklg", "*.pklg", NULL, FALSE, NULL, NULL },
+	{ "PacketLogger", "pklg", "*.pklg", NULL, FALSE, FALSE,
+	  NULL, NULL },
 
 	/* WTAP_FILE_DAINTREE_SNA */
-	{ "Daintree SNA", "dsna", "*.dcf", NULL, FALSE, NULL, NULL },
+	{ "Daintree SNA", "dsna", "*.dcf", NULL, FALSE, FALSE,
+	  NULL, NULL },
 
 	/* WTAP_FILE_NETSCALER_1_0 */
-	{ "NetScaler Trace (Version 1.0)", "nstrace10", "*.*", "*.*", FALSE,
+	{ "NetScaler Trace (Version 1.0)", "nstrace10", "*.*", "*.*", TRUE, FALSE,
 	  nstrace_10_dump_can_write_encap, nstrace_dump_open },
 
 	/* WTAP_FILE_NETSCALER_2_0 */
-	{ "NetScaler Trace (Version 2.0)", "nstrace20", "*.cap", "*.cap", FALSE,
+	{ "NetScaler Trace (Version 2.0)", "nstrace20", "*.cap", "*.cap", TRUE, FALSE,
 	  nstrace_20_dump_can_write_encap, nstrace_dump_open },
 
 	/* WTAP_FILE_JPEG_JFIF */
-	{ "JPEG/JFIF", "jpeg", "*.jpg;*.jpeg;*.jfif", ".jpg", FALSE, NULL, NULL },
+	{ "JPEG/JFIF", "jpeg", "*.jpg;*.jpeg;*.jfif", ".jpg", FALSE, FALSE,
+	  NULL, NULL },
 
+	/* WTAP_FILE_IPFIX */
+	{ "IPFIX File Format", "ipfix", "*.pfx;*.ipfix", NULL, FALSE, FALSE,
+	  NULL, NULL }
 };
 
 gint wtap_num_file_types = sizeof(dump_open_table_base) / sizeof(struct file_type_info);
@@ -640,7 +662,7 @@ static void init_file_types(void) {
 
 	g_array_append_vals(dump_open_table_arr,dump_open_table_base,wtap_num_file_types);
 
-	dump_open_table = (void*)dump_open_table_arr->data;
+	dump_open_table = (const struct file_type_info*)(void *)dump_open_table_arr->data;
 }
 
 int wtap_register_file_type(const struct file_type_info* fi) {
@@ -648,7 +670,7 @@ int wtap_register_file_type(const struct file_type_info* fi) {
 
 	g_array_append_val(dump_open_table_arr,*fi);
 
-	dump_open_table = (void*)dump_open_table_arr->data;
+	dump_open_table = (const struct file_type_info*)(void *)dump_open_table_arr->data;
 
 	return wtap_num_file_types++;
 }
@@ -663,7 +685,8 @@ const char *wtap_file_type_string(int filetype)
 {
 	if (filetype < 0 || filetype >= wtap_num_file_types) {
 		g_error("Unknown capture file type %d", filetype);
-		return NULL;
+		/** g_error() does an abort() and thus never returns **/
+		return "";
 	} else
 		return dump_open_table[filetype].name;
 }
@@ -732,8 +755,13 @@ gboolean wtap_dump_can_write_encap(int filetype, int encap)
 #ifdef HAVE_LIBZ
 gboolean wtap_dump_can_compress(int filetype)
 {
+	/*
+	 * If this is an unknown file type, or if we have to
+	 * seek when writing out a file with this file type,
+	 * return FALSE.
+	 */
 	if (filetype < 0 || filetype >= wtap_num_file_types
-	    || dump_open_table[filetype].can_compress == FALSE)
+	    || dump_open_table[filetype].writing_must_seek)
 		return FALSE;
 
 	return TRUE;
@@ -745,21 +773,29 @@ gboolean wtap_dump_can_compress(int filetype _U_)
 }
 #endif
 
+gboolean wtap_dump_has_name_resolution(int filetype)
+{
+	if (filetype < 0 || filetype >= wtap_num_file_types
+	    || dump_open_table[filetype].has_name_resolution == FALSE)
+		return FALSE;
+
+	return TRUE;
+}
 
 static gboolean wtap_dump_open_check(int filetype, int encap, gboolean comressed, int *err);
 static wtap_dumper* wtap_dump_alloc_wdh(int filetype, int encap, int snaplen,
 					gboolean compressed, int *err);
 static gboolean wtap_dump_open_finish(wtap_dumper *wdh, int filetype, gboolean compressed, int *err);
 
-static FILE *wtap_dump_file_open(wtap_dumper *wdh, const char *filename);
-static FILE *wtap_dump_file_fdopen(wtap_dumper *wdh, int fd);
+static WFILE_T wtap_dump_file_open(wtap_dumper *wdh, const char *filename);
+static WFILE_T wtap_dump_file_fdopen(wtap_dumper *wdh, int fd);
 static int wtap_dump_file_close(wtap_dumper *wdh);
 
 wtap_dumper* wtap_dump_open(const char *filename, int filetype, int encap,
 				int snaplen, gboolean compressed, int *err)
 {
 	wtap_dumper *wdh;
-	FILE *fh;
+	WFILE_T fh;
 
 	/* Check whether we can open a capture file with that file type
 	   and that encapsulation. */
@@ -773,12 +809,18 @@ wtap_dumper* wtap_dump_open(const char *filename, int filetype, int encap,
 
 	/* "-" means stdout */
 	if (strcmp(filename, "-") == 0) {
-		if(compressed) {
+		if (compressed) {
+			*err = EINVAL;	/* XXX - return a Wiretap error code for this */
 			g_free(wdh);
 			return NULL;	/* compress won't work on stdout */
 		}
 #ifdef _WIN32
-		setmode(fileno(stdout), O_BINARY);
+		if (_setmode(fileno(stdout), O_BINARY) == -1) {
+			/* "Should not happen" */
+			*err = errno;
+			g_free(wdh);
+			return NULL;	/* couldn't put standard output in binary mode */
+		}
 #endif
 		wdh->fh = stdout;
 	} else {
@@ -811,7 +853,7 @@ wtap_dumper* wtap_dump_fdopen(int fd, int filetype, int encap, int snaplen,
 				gboolean compressed, int *err)
 {
 	wtap_dumper *wdh;
-	FILE *fh;
+	WFILE_T fh;
 
 	/* Check whether we can open a capture file with that file type
 	   and that encapsulation. */
@@ -824,8 +866,13 @@ wtap_dumper* wtap_dump_fdopen(int fd, int filetype, int encap, int snaplen,
 		return NULL;	/* couldn't allocate it */
 
 #ifdef _WIN32
-	if(fd == 1) {
-		setmode(fileno(stdout), O_BINARY);
+	if (fd == 1) {
+		if (_setmode(fileno(stdout), O_BINARY) == -1) {
+			/* "Should not happen" */
+			*err = errno;
+			g_free(wdh);
+			return NULL;	/* couldn't put standard output in binary mode */
+		}
 	}
 #endif
 
@@ -872,7 +919,6 @@ static gboolean wtap_dump_open_check(int filetype, int encap, gboolean compresse
 	if (*err != 0)
 		return FALSE;
 
-
 	/* All systems go! */
 	return TRUE;
 }
@@ -882,20 +928,16 @@ static wtap_dumper* wtap_dump_alloc_wdh(int filetype, int encap, int snaplen,
 {
 	wtap_dumper *wdh;
 
-	wdh = g_malloc(sizeof (wtap_dumper));
+	wdh = (wtap_dumper *)g_malloc0(sizeof (wtap_dumper));
 	if (wdh == NULL) {
 		*err = errno;
 		return NULL;
 	}
-	wdh->fh = NULL;
+
 	wdh->file_type = filetype;
 	wdh->snaplen = snaplen;
 	wdh->encap = encap;
 	wdh->compressed = compressed;
-	wdh->bytes_dumped = 0;
-	wdh->dump.opaque = NULL;
-	wdh->subtype_write = NULL;
-	wdh->subtype_close = NULL;
 	return wdh;
 }
 
@@ -909,7 +951,7 @@ static gboolean wtap_dump_open_finish(wtap_dumper *wdh, int filetype, gboolean c
 	if(compressed) {
 		cant_seek = TRUE;
 	} else {
-		fd = fileno(wdh->fh);
+		fd = fileno((FILE *)wdh->fh);
 		if (lseek(fd, 1, SEEK_CUR) == -1)
 			cant_seek = TRUE;
 		else {
@@ -919,8 +961,14 @@ static gboolean wtap_dump_open_finish(wtap_dumper *wdh, int filetype, gboolean c
 		}
 	}
 
+	/* If this file type requires seeking, and we can't seek, fail. */
+	if (dump_open_table[filetype].writing_must_seek && cant_seek) {
+		*err = WTAP_ERR_CANT_WRITE_TO_PIPE;
+		return FALSE;
+	}
+
 	/* Now try to open the file for writing. */
-	if (!(*dump_open_table[filetype].dump_open)(wdh, cant_seek, err)) {
+	if (!(*dump_open_table[filetype].dump_open)(wdh, err)) {
 		return FALSE;
 	}
 
@@ -937,11 +985,11 @@ void wtap_dump_flush(wtap_dumper *wdh)
 {
 #ifdef HAVE_LIBZ
 	if(wdh->compressed) {
-		gzflush(wdh->fh, Z_SYNC_FLUSH);	/* XXX - is Z_SYNC_FLUSH the right one? */
+		gzwfile_flush((GZWFILE_T)wdh->fh);
 	} else
 #endif
 	{
-		fflush(wdh->fh);
+		fflush((FILE *)wdh->fh);
 	}
 }
 
@@ -971,8 +1019,8 @@ gboolean wtap_dump_close(wtap_dumper *wdh, int *err)
 		/* as we don't close stdout, at least try to flush it */
 		wtap_dump_flush(wdh);
 	}
-	if (wdh->dump.opaque != NULL)
-		g_free(wdh->dump.opaque);
+	if (wdh->priv != NULL)
+		g_free(wdh->priv);
 	g_free(wdh);
 	return ret;
 }
@@ -987,19 +1035,27 @@ void wtap_set_bytes_dumped(wtap_dumper *wdh, gint64 bytes_dumped)
 	wdh->bytes_dumped = bytes_dumped;
 }
 
+gboolean wtap_dump_set_addrinfo_list(wtap_dumper *wdh, struct addrinfo *addrinfo_list)
+{
+	if (!wdh || wdh->file_type < 0 || wdh->file_type >= wtap_num_file_types
+		|| dump_open_table[wdh->file_type].has_name_resolution == FALSE)
+			return FALSE;
+	wdh->addrinfo_list = addrinfo_list;
+	return TRUE;
+}
 
 /* internally open a file for writing (compressed or not) */
 #ifdef HAVE_LIBZ
-static FILE *wtap_dump_file_open(wtap_dumper *wdh, const char *filename)
+static WFILE_T wtap_dump_file_open(wtap_dumper *wdh, const char *filename)
 {
 	if(wdh->compressed) {
-		return gzopen(filename, "wb");
+		return gzwfile_open(filename);
 	} else {
 		return ws_fopen(filename, "wb");
 	}
 }
 #else
-static FILE *wtap_dump_file_open(wtap_dumper *wdh _U_, const char *filename)
+static WFILE_T wtap_dump_file_open(wtap_dumper *wdh _U_, const char *filename)
 {
 	return ws_fopen(filename, "wb");
 }
@@ -1007,47 +1063,54 @@ static FILE *wtap_dump_file_open(wtap_dumper *wdh _U_, const char *filename)
 
 /* internally open a file for writing (compressed or not) */
 #ifdef HAVE_LIBZ
-static FILE *wtap_dump_file_fdopen(wtap_dumper *wdh, int fd)
+static WFILE_T wtap_dump_file_fdopen(wtap_dumper *wdh, int fd)
 {
 	if(wdh->compressed) {
-		return gzdopen(fd, "wb");
+		return gzwfile_fdopen(fd);
 	} else {
 		return fdopen(fd, "wb");
 	}
 }
 #else
-static FILE *wtap_dump_file_fdopen(wtap_dumper *wdh _U_, int fd)
+static WFILE_T wtap_dump_file_fdopen(wtap_dumper *wdh _U_, int fd)
 {
 	return fdopen(fd, "wb");
 }
 #endif
 
 /* internally writing raw bytes (compressed or not) */
-size_t wtap_dump_file_write(wtap_dumper *wdh, const void *buf, size_t bufsize)
-{
-#ifdef HAVE_LIBZ
-	if(wdh->compressed) {
-		return gzwrite(wdh->fh, buf, (unsigned) bufsize);
-	} else
-#endif
-	{
-		return fwrite(buf, 1, bufsize, wdh->fh);
-	}
-}
-
-gboolean wtap_dump_file_write_all(wtap_dumper *wdh, const void *buf, unsigned bufsize, int *err)
+gboolean wtap_dump_file_write(wtap_dumper *wdh, const void *buf, size_t bufsize,
+		     int *err)
 {
 	size_t nwritten;
 
-    nwritten = wtap_dump_file_write(wdh, buf, bufsize);
-    if (nwritten != bufsize) {
-        if (nwritten == 0 && wtap_dump_file_ferror(wdh))
-            *err = wtap_dump_file_ferror(wdh);
-        else
-            *err = WTAP_ERR_SHORT_WRITE;
-        return FALSE;
-    }
-    return TRUE;
+#ifdef HAVE_LIBZ
+	if (wdh->compressed) {
+		nwritten = gzwfile_write((GZWFILE_T)wdh->fh, buf, (unsigned) bufsize);
+		/*
+		 * gzwfile_write() returns 0 on error.
+		 */
+		if (nwritten == 0) {
+			*err = gzwfile_geterr((GZWFILE_T)wdh->fh);
+			return FALSE;
+		}
+	} else
+#endif
+	{
+		nwritten = fwrite(buf, 1, bufsize, (FILE *)wdh->fh);
+		/*
+		 * At least according to the Mac OS X man page,
+		 * this can return a short count on an error.
+		 */
+		if (nwritten != bufsize) {
+			if (ferror((FILE *)wdh->fh))
+				*err = errno;
+			else
+				*err = WTAP_ERR_SHORT_WRITE;
+			return FALSE;
+		}
+	}
+	return TRUE;
 }
 
 /* internally close a file for writing (compressed or not) */
@@ -1055,32 +1118,10 @@ static int wtap_dump_file_close(wtap_dumper *wdh)
 {
 #ifdef HAVE_LIBZ
 	if(wdh->compressed) {
-		return gzclose(wdh->fh);
+		return gzwfile_close((GZWFILE_T)wdh->fh);
 	} else
 #endif
 	{
-		return fclose(wdh->fh);
+		return fclose((FILE *)wdh->fh);
 	}
 }
-
-int wtap_dump_file_ferror(wtap_dumper *wdh)
-{
-#ifdef HAVE_LIBZ
-	int errnum;
-
-	if(wdh->compressed) {
-		gzerror(wdh->fh, &errnum);
-
-		if(errnum == Z_ERRNO) {
-			return errno;
-		} else {
-			/* XXX - what to do with this zlib specific number? */
-			return errnum;
-		}
-	} else
-#endif
-	{
-		return ferror(wdh->fh);
-	}
-}
-
