@@ -1,6 +1,6 @@
 /* libpcap.c
  *
- * $Id: libpcap.c 28865 2009-06-27 18:10:10Z tuexen $
+ * $Id: libpcap.c 36748 2011-04-21 09:41:52Z guy $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -32,11 +32,28 @@
 #include "buffer.h"
 #include "atm.h"
 #include "pcap-common.h"
+#include "pcap-encap.h"
 #include "libpcap.h"
 
 /* See source to the "libpcap" library for information on the "libpcap"
    file format. */
 
+/*
+ * Private per-wtap_t data needed to read a file.
+ */
+typedef enum {
+	NOT_SWAPPED,
+	SWAPPED,
+	MAYBE_SWAPPED
+} swapped_type_t;
+
+typedef struct {
+	gboolean byte_swapped;
+	swapped_type_t lengths_swapped;
+	guint16	version_major;
+	guint16	version_minor;
+} libpcap_t;
+
 /* On some systems, the FDDI MAC addresses are bit-swapped. */
 #if !defined(ultrix) && !defined(__alpha) && !defined(__bsdi__)
 #define BIT_SWAPPED_MAC_ADDRS
@@ -59,8 +76,7 @@ static int libpcap_read_header(wtap *wth, int *err, gchar **err_info,
     struct pcaprec_ss990915_hdr *hdr);
 static void adjust_header(wtap *wth, struct pcaprec_hdr *hdr);
 static gboolean libpcap_read_rec_data(FILE_T fh, guchar *pd, int length,
-    int *err);
-static void libpcap_close(wtap *wth);
+    int *err, gchar **err_info);
 static gboolean libpcap_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
     const union wtap_pseudo_header *pseudo_header, const guchar *pd, int *err);
 
@@ -73,12 +89,13 @@ int libpcap_open(wtap *wth, int *err, gchar **err_info)
 	gboolean modified;
 	gboolean aix;
 	int file_encap;
+	libpcap_t *libpcap;
 
 	/* Read in the number that should be at the start of a "libpcap" file */
 	errno = WTAP_ERR_CANT_READ;
-	bytes_read = file_read(&magic, 1, sizeof magic, wth->fh);
+	bytes_read = file_read(&magic, sizeof magic, wth->fh);
 	if (bytes_read != sizeof magic) {
-		*err = file_error(wth->fh);
+		*err = file_error(wth->fh, err_info);
 		if (*err != 0)
 			return -1;
 		return 0;
@@ -122,8 +139,9 @@ int libpcap_open(wtap *wth, int *err, gchar **err_info)
 		break;
 
 	case PCAP_NSEC_MAGIC:
-		/* Host that wrote it has our byte order, and was running
-		   a program using either standard or ss990417 libpcap. */
+		/* Host that wrote it has our byte order, and was writing
+		   the file in a format similar to standard libpcap
+		   except that the time stamps have nanosecond resolution. */
 		byte_swapped = FALSE;
 		modified = FALSE;
 		wth->tsprecision = WTAP_FILE_TSPREC_NSEC;
@@ -131,8 +149,9 @@ int libpcap_open(wtap *wth, int *err, gchar **err_info)
 
 	case PCAP_SWAPPED_NSEC_MAGIC:
 		/* Host that wrote it out has a byte order opposite to
-		   ours, and was running a program using either ss990915
-		   or ss991029 libpcap. */
+		   ours, and was writing the file in a format similar to
+		   standard libpcap except that the time stamps have
+		   nanosecond resolution. */
 		byte_swapped = TRUE;
 		modified = FALSE;
 		wth->tsprecision = WTAP_FILE_TSPREC_NSEC;
@@ -145,9 +164,9 @@ int libpcap_open(wtap *wth, int *err, gchar **err_info)
 
 	/* Read the rest of the header. */
 	errno = WTAP_ERR_CANT_READ;
-	bytes_read = file_read(&hdr, 1, sizeof hdr, wth->fh);
+	bytes_read = file_read(&hdr, sizeof hdr, wth->fh);
 	if (bytes_read != sizeof hdr) {
-		*err = file_error(wth->fh);
+		*err = file_error(wth->fh, err_info);
 		if (*err != 0)
 			return -1;
 		return 0;
@@ -230,18 +249,8 @@ int libpcap_open(wtap *wth, int *err, gchar **err_info)
 		}
 	}
 
-	/*
-	 * We treat a DLT_ value of 13 specially - it appears that in
-	 * Nokia libpcap format, it's some form of ATM with what I
-	 * suspect is a pseudo-header (even though Nokia's IPSO is
-	 * based on FreeBSD, which #defines DLT_SLIP_BSDOS as 13).
-	 *
-	 * We don't yet know whether this is a Nokia capture, so if
-	 * "wtap_pcap_encap_to_wtap_encap()" returned WTAP_ENCAP_UNKNOWN
-	 * but "hdr.network" is 13, we don't treat that as an error yet.
-	 */
 	file_encap = wtap_pcap_encap_to_wtap_encap(hdr.network);
-	if (file_encap == WTAP_ENCAP_UNKNOWN && hdr.network != 13) {
+	if (file_encap == WTAP_ENCAP_UNKNOWN) {
 		*err = WTAP_ERR_UNSUPPORTED_ENCAP;
 		*err_info = g_strdup_printf("pcap: network type %u unknown or unsupported",
 		    hdr.network);
@@ -249,13 +258,13 @@ int libpcap_open(wtap *wth, int *err, gchar **err_info)
 	}
 
 	/* This is a libpcap file */
-	wth->capture.pcap = g_malloc(sizeof(libpcap_t));
-	wth->capture.pcap->byte_swapped = byte_swapped;
-	wth->capture.pcap->version_major = hdr.version_major;
-	wth->capture.pcap->version_minor = hdr.version_minor;
+	libpcap = (libpcap_t *)g_malloc(sizeof(libpcap_t));;
+	libpcap->byte_swapped = byte_swapped;
+	libpcap->version_major = hdr.version_major;
+	libpcap->version_minor = hdr.version_minor;
+	wth->priv = (void *)libpcap;
 	wth->subtype_read = libpcap_read;
 	wth->subtype_seek_read = libpcap_seek_read;
-	wth->subtype_close = libpcap_close;
 	wth->file_encap = file_encap;
 	wth->snapshot_length = hdr.snaplen;
 
@@ -279,19 +288,19 @@ int libpcap_open(wtap *wth, int *err, gchar **err_info)
 
 	case 2:
 		if (hdr.version_minor < 3)
-			wth->capture.pcap->lengths_swapped = SWAPPED;
+			libpcap->lengths_swapped = SWAPPED;
 		else if (hdr.version_minor == 3)
-			wth->capture.pcap->lengths_swapped = MAYBE_SWAPPED;
+			libpcap->lengths_swapped = MAYBE_SWAPPED;
 		else
-			wth->capture.pcap->lengths_swapped = NOT_SWAPPED;
+			libpcap->lengths_swapped = NOT_SWAPPED;
 		break;
 
 	case 543:
-		wth->capture.pcap->lengths_swapped = SWAPPED;
+		libpcap->lengths_swapped = SWAPPED;
 		break;
 
 	default:
-		wth->capture.pcap->lengths_swapped = NOT_SWAPPED;
+		libpcap->lengths_swapped = NOT_SWAPPED;
 		break;
 	}
 
@@ -356,7 +365,7 @@ int libpcap_open(wtap *wth, int *err, gchar **err_info)
 			 * Well, we couldn't even read it.
 			 * Give up.
 			 */
-			g_free(wth->capture.pcap);
+			g_free(wth->priv);
 			return -1;
 
 		case THIS_FORMAT:
@@ -365,7 +374,7 @@ int libpcap_open(wtap *wth, int *err, gchar **err_info)
 			 * Put the seek pointer back, and return success.
 			 */
 			if (file_seek(wth->fh, wth->data_offset, SEEK_SET, err) == -1) {
-				g_free(wth->capture.pcap);
+				g_free(wth->priv);
 				return -1;
 			}
 			return 1;
@@ -386,7 +395,7 @@ int libpcap_open(wtap *wth, int *err, gchar **err_info)
 		 */
 		wth->file_type = WTAP_FILE_PCAP_SS990915;
 		if (file_seek(wth->fh, wth->data_offset, SEEK_SET, err) == -1) {
-			g_free(wth->capture.pcap);
+			g_free(wth->priv);
 			return -1;
 		}
 	} else {
@@ -407,7 +416,7 @@ int libpcap_open(wtap *wth, int *err, gchar **err_info)
 			 * Well, we couldn't even read it.
 			 * Give up.
 			 */
-			g_free(wth->capture.pcap);
+			g_free(wth->priv);
 			return -1;
 
 		case THIS_FORMAT:
@@ -417,7 +426,7 @@ int libpcap_open(wtap *wth, int *err, gchar **err_info)
 			 * Put the seek pointer back, and return success.
 			 */
 			if (file_seek(wth->fh, wth->data_offset, SEEK_SET, err) == -1) {
-				g_free(wth->capture.pcap);
+				g_free(wth->priv);
 				return -1;
 			}
 			return 1;
@@ -436,7 +445,7 @@ int libpcap_open(wtap *wth, int *err, gchar **err_info)
 		 */
 		wth->file_type = WTAP_FILE_PCAP_SS990417;
 		if (file_seek(wth->fh, wth->data_offset, SEEK_SET, err) == -1) {
-			g_free(wth->capture.pcap);
+			g_free(wth->priv);
 			return -1;
 		}
 		switch (libpcap_try(wth, err)) {
@@ -446,7 +455,7 @@ int libpcap_open(wtap *wth, int *err, gchar **err_info)
 			 * Well, we couldn't even read it.
 			 * Give up.
 			 */
-			g_free(wth->capture.pcap);
+			g_free(wth->priv);
 			return -1;
 
 		case THIS_FORMAT:
@@ -455,7 +464,7 @@ int libpcap_open(wtap *wth, int *err, gchar **err_info)
 			 * Put the seek pointer back, and return success.
 			 */
 			if (file_seek(wth->fh, wth->data_offset, SEEK_SET, err) == -1) {
-				g_free(wth->capture.pcap);
+				g_free(wth->priv);
 				return -1;
 			}
 			return 1;
@@ -476,27 +485,22 @@ int libpcap_open(wtap *wth, int *err, gchar **err_info)
 		 */
 		wth->file_type = WTAP_FILE_PCAP_NOKIA;
 		if (file_seek(wth->fh, wth->data_offset, SEEK_SET, err) == -1) {
-			g_free(wth->capture.pcap);
+			g_free(wth->priv);
 			return -1;
 		}
 	}
 
-	if (hdr.network == 13) {
-		/*
-		 * OK, if this was a Nokia capture, make it
-		 * WTAP_ENCAP_ATM_PDUS, otherwise return
-		 * an error.
-		 */
-		if (wth->file_type == WTAP_FILE_PCAP_NOKIA)
-			wth->file_encap = WTAP_ENCAP_ATM_PDUS;
-		else {
-			*err = WTAP_ERR_UNSUPPORTED_ENCAP;
-			*err_info = g_strdup_printf("pcap: network type %u unknown or unsupported",
-			    hdr.network);
-			g_free(wth->capture.pcap);
-			return -1;
-		}
-	}
+	/*
+	 * We treat a DLT_ value of 13 specially - it appears that in
+	 * Nokia libpcap format, it's some form of ATM with what I
+	 * suspect is a pseudo-header (even though Nokia's IPSO is
+	 * based on FreeBSD, which #defines DLT_SLIP_BSDOS as 13).
+	 *
+	 * If this is a Nokia capture, treat 13 as WTAP_ENCAP_ATM_PDUS,
+	 * rather than as what we normally treat it.
+	 */
+	if (wth->file_type == WTAP_FILE_PCAP_NOKIA && hdr.network == 13)
+		wth->file_encap = WTAP_ENCAP_ATM_PDUS;
 
 	return 1;
 }
@@ -599,6 +603,7 @@ static gboolean libpcap_read(wtap *wth, int *err, gchar **err_info,
 	int bytes_read;
 	guchar fddi_padding[3];
 	int phdr_len;
+	libpcap_t *libpcap;
 
 	bytes_read = libpcap_read_header(wth, err, err_info, &hdr);
 	if (bytes_read == -1) {
@@ -630,14 +635,17 @@ static gboolean libpcap_read(wtap *wth, int *err, gchar **err_info,
 		/*
 		 * Read the padding.
 		 */
-		if (!libpcap_read_rec_data(wth->fh, fddi_padding, 3, err))
+		if (!libpcap_read_rec_data(wth->fh, fddi_padding, 3, err,
+		    err_info))
 			return FALSE;	/* Read error */
 	}
 
 	*data_offset = wth->data_offset;
 
-	phdr_len = pcap_process_pseudo_header(wth->fh, wth->file_type, wth->file_encap, wth->capture.pcap->byte_swapped, packet_size,
-	    TRUE, &wth->phdr, &wth->pseudo_header, err, err_info);
+	libpcap = (libpcap_t *)wth->priv;
+	phdr_len = pcap_process_pseudo_header(wth->fh, wth->file_type,
+	    wth->file_encap, packet_size, TRUE, &wth->phdr,
+	    &wth->pseudo_header, err, err_info);
 	if (phdr_len < 0)
 		return FALSE;	/* error */
 
@@ -650,7 +658,7 @@ static gboolean libpcap_read(wtap *wth, int *err, gchar **err_info,
 
 	buffer_assure_space(wth->frame_buffer, packet_size);
 	if (!libpcap_read_rec_data(wth->fh, buffer_start_ptr(wth->frame_buffer),
-	    packet_size, err))
+	    packet_size, err, err_info))
 		return FALSE;	/* Read error */
 	wth->data_offset += packet_size;
 
@@ -691,6 +699,8 @@ static gboolean libpcap_read(wtap *wth, int *err, gchar **err_info,
 		}
 	}
 
+	pcap_read_post_process(wth->file_encap, wth->phdr.caplen,
+	    libpcap->byte_swapped, buffer_start_ptr(wth->frame_buffer));
 	return TRUE;
 }
 
@@ -700,19 +710,21 @@ libpcap_seek_read(wtap *wth, gint64 seek_off,
     int *err, gchar **err_info)
 {
 	int phdr_len;
+	libpcap_t *libpcap;
 
 	if (file_seek(wth->random_fh, seek_off, SEEK_SET, err) == -1)
 		return FALSE;
 
-	phdr_len = pcap_process_pseudo_header(wth->random_fh, wth->file_type, wth->file_encap, wth->capture.pcap->byte_swapped, length,
-	    FALSE, NULL, pseudo_header, err, err_info);
+	libpcap = (libpcap_t *)wth->priv;
+	phdr_len = pcap_process_pseudo_header(wth->random_fh, wth->file_type,
+	    wth->file_encap, length, FALSE, NULL, pseudo_header, err, err_info);
 	if (phdr_len < 0)
 		return FALSE;	/* error */
 
 	/*
 	 * Read the packet data.
 	 */
-	if (!libpcap_read_rec_data(wth->random_fh, pd, length, err))
+	if (!libpcap_read_rec_data(wth->random_fh, pd, length, err, err_info))
 		return FALSE;	/* failed */
 
 	if (wth->file_encap == WTAP_ENCAP_ATM_PDUS) {
@@ -736,6 +748,8 @@ libpcap_seek_read(wtap *wth, gint64 seek_off,
 				atm_guess_lane_type(pd, length, pseudo_header);
 		}
 	}
+	pcap_read_post_process(wth->file_encap, length,
+	    libpcap->byte_swapped, pd);
 	return TRUE;
 }
 
@@ -774,9 +788,9 @@ static int libpcap_read_header(wtap *wth, int *err, gchar **err_info,
 		g_assert_not_reached();
 		bytes_to_read = 0;
 	}
-	bytes_read = file_read(hdr, 1, bytes_to_read, wth->fh);
+	bytes_read = file_read(hdr, bytes_to_read, wth->fh);
 	if (bytes_read != bytes_to_read) {
-		*err = file_error(wth->fh);
+		*err = file_error(wth->fh, err_info);
 		if (*err == 0 && bytes_read != 0) {
 			*err = WTAP_ERR_SHORT_READ;
 		}
@@ -826,8 +840,10 @@ static void
 adjust_header(wtap *wth, struct pcaprec_hdr *hdr)
 {
 	guint32 temp;
+	libpcap_t *libpcap;
 
-	if (wth->capture.pcap->byte_swapped) {
+	libpcap = (libpcap_t *)wth->priv;
+	if (libpcap->byte_swapped) {
 		/* Byte-swap the record header fields. */
 		hdr->ts_sec = BSWAP32(hdr->ts_sec);
 		hdr->ts_usec = BSWAP32(hdr->ts_usec);
@@ -836,7 +852,7 @@ adjust_header(wtap *wth, struct pcaprec_hdr *hdr)
 	}
 
 	/* Swap the "incl_len" and "orig_len" fields, if necessary. */
-	switch (wth->capture.pcap->lengths_swapped) {
+	switch (libpcap->lengths_swapped) {
 
 	case NOT_SWAPPED:
 		break;
@@ -860,15 +876,16 @@ adjust_header(wtap *wth, struct pcaprec_hdr *hdr)
 }
 
 static gboolean
-libpcap_read_rec_data(FILE_T fh, guchar *pd, int length, int *err)
+libpcap_read_rec_data(FILE_T fh, guchar *pd, int length, int *err,
+    gchar **err_info)
 {
 	int	bytes_read;
 
 	errno = WTAP_ERR_CANT_READ;
-	bytes_read = file_read(pd, 1, length, fh);
+	bytes_read = file_read(pd, length, fh);
 
 	if (bytes_read != length) {
-		*err = file_error(fh);
+		*err = file_error(fh, err_info);
 		if (*err == 0)
 			*err = WTAP_ERR_SHORT_READ;
 		return FALSE;
@@ -876,12 +893,6 @@ libpcap_read_rec_data(FILE_T fh, guchar *pd, int length, int *err)
 	return TRUE;
 }
 
-static void
-libpcap_close(wtap *wth)
-{
-	g_free(wth->capture.pcap);
-}
-
 /* Returns 0 if we could write the specified encapsulation type,
    an error indication otherwise. */
 int libpcap_dump_can_write_encap(int encap)
@@ -898,11 +909,10 @@ int libpcap_dump_can_write_encap(int encap)
 
 /* Returns TRUE on success, FALSE on failure; sets "*err" to an error code on
    failure */
-gboolean libpcap_dump_open(wtap_dumper *wdh, gboolean cant_seek _U_, int *err)
+gboolean libpcap_dump_open(wtap_dumper *wdh, int *err)
 {
 	guint32 magic;
 	struct pcap_hdr file_hdr;
-	size_t nwritten;
 
 	/* This is a libpcap file */
 	wdh->subtype_write = libpcap_dump;
@@ -936,14 +946,8 @@ gboolean libpcap_dump_open(wtap_dumper *wdh, gboolean cant_seek _U_, int *err)
 		return FALSE;
 	}
 
-	nwritten = wtap_dump_file_write(wdh, &magic, sizeof magic);
-	if (nwritten != sizeof magic) {
-		if (nwritten == 0 && wtap_dump_file_ferror(wdh))
-			*err = wtap_dump_file_ferror(wdh);
-		else
-			*err = WTAP_ERR_SHORT_WRITE;
+	if (!wtap_dump_file_write(wdh, &magic, sizeof magic, err))
 		return FALSE;
-	}
 	wdh->bytes_dumped += sizeof magic;
 
 	/* current "libpcap" format is 2.4 */
@@ -965,14 +969,8 @@ gboolean libpcap_dump_open(wtap_dumper *wdh, gboolean cant_seek _U_, int *err)
 	file_hdr.snaplen = (wdh->snaplen != 0) ? wdh->snaplen :
 						 WTAP_MAX_PACKET_SIZE;
 	file_hdr.network = wtap_wtap_encap_to_pcap_encap(wdh->encap);
-	nwritten = wtap_dump_file_write(wdh, &file_hdr, sizeof file_hdr);
-	if (nwritten != sizeof file_hdr) {
-		if (nwritten == 0 && wtap_dump_file_ferror(wdh))
-			*err = wtap_dump_file_ferror(wdh);
-		else
-			*err = WTAP_ERR_SHORT_WRITE;
+	if (!wtap_dump_file_write(wdh, &file_hdr, sizeof file_hdr, err))
 		return FALSE;
-	}
 	wdh->bytes_dumped += sizeof file_hdr;
 
 	return TRUE;
@@ -987,7 +985,6 @@ static gboolean libpcap_dump(wtap_dumper *wdh,
 {
 	struct pcaprec_ss990915_hdr rec_hdr;
 	size_t hdr_size;
-	size_t nwritten;
 	int phdrsize;
 
 	phdrsize = pcap_get_phdr_size(wdh->encap, pseudo_header);
@@ -1060,27 +1057,15 @@ static gboolean libpcap_dump(wtap_dumper *wdh,
 		return FALSE;
 	}
 
-	nwritten = wtap_dump_file_write(wdh, &rec_hdr, hdr_size);
-	if (nwritten != hdr_size) {
-		if (nwritten == 0 && wtap_dump_file_ferror(wdh))
-			*err = wtap_dump_file_ferror(wdh);
-		else
-			*err = WTAP_ERR_SHORT_WRITE;
+	if (!wtap_dump_file_write(wdh, &rec_hdr, hdr_size, err))
 		return FALSE;
-	}
 	wdh->bytes_dumped += hdr_size;
 
 	if (!pcap_write_phdr(wdh, wdh->encap, pseudo_header, err))
 		return FALSE;
 
-	nwritten = wtap_dump_file_write(wdh, pd, phdr->caplen);
-	if (nwritten != phdr->caplen) {
-		if (nwritten == 0 && wtap_dump_file_ferror(wdh))
-			*err = wtap_dump_file_ferror(wdh);
-		else
-			*err = WTAP_ERR_SHORT_WRITE;
+	if (!wtap_dump_file_write(wdh, pd, phdr->caplen, err))
 		return FALSE;
-	}
         wdh->bytes_dumped += phdr->caplen;
 	return TRUE;
 }
