@@ -4,7 +4,7 @@
  * Written by Shaun Jackman <sjackman@gmail.com>
  * Copyright 2007 Shaun Jackman
  *
- * $Id: mpeg.c 25690 2008-07-09 19:15:50Z morriss $
+ * $Id: mpeg.c 36748 2011-04-21 09:41:52Z guy $
  *
  * Wiretap Library
  * This program is free software; you can redistribute it and/or
@@ -48,11 +48,16 @@
 #define PES_PREFIX 1
 #define PES_VALID(n) (((n) >> 8 & 0xffffff) == PES_PREFIX)
 
-static size_t 
+typedef struct {
+	struct wtap_nstime now;
+	time_t t0;
+} mpeg_t;
+
+static int 
 mpeg_resync(wtap *wth, int *err, gchar **err_info _U_)
 {
 	gint64 offset = file_tell(wth->fh);
-	size_t count = 0;
+	int count = 0;
 	int byte = file_getc(wth->fh);
 
 	while (byte != EOF) {
@@ -64,20 +69,20 @@ mpeg_resync(wtap *wth, int *err, gchar **err_info _U_)
 			byte = file_getc(wth->fh);
 		count++;
 	}
-	file_seek(wth->fh, offset, SEEK_SET, err);
+	if (file_seek(wth->fh, offset, SEEK_SET, err) == -1)
+		return 0;
 	return count;
 }
 
 static int 
-mpeg_read_header(wtap *wth, int *err, gchar **err_info _U_,
-		guint32 *n)
+mpeg_read_header(wtap *wth, int *err, gchar **err_info, guint32 *n)
 {
 	int bytes_read;
 
 	errno = WTAP_ERR_CANT_READ;
-	bytes_read = file_read(n, 1, sizeof *n, wth->fh);
+	bytes_read = file_read(n, sizeof *n, wth->fh);
 	if (bytes_read != sizeof *n) {
-		*err = file_error(wth->fh);
+		*err = file_error(wth->fh, err_info);
 		if (*err == 0 && bytes_read != 0)
 			*err = WTAP_ERR_SHORT_READ;
 		return -1;
@@ -89,15 +94,16 @@ mpeg_read_header(wtap *wth, int *err, gchar **err_info _U_,
 }
 
 static gboolean
-mpeg_read_rec_data(FILE_T fh, guchar *pd, int length, int *err)
+mpeg_read_rec_data(FILE_T fh, guchar *pd, int length, int *err,
+		gchar **err_info)
 {
 	int	bytes_read;
 
 	errno = WTAP_ERR_CANT_READ;
-	bytes_read = file_read(pd, 1, length, fh);
+	bytes_read = file_read(pd, length, fh);
 
 	if (bytes_read != length) {
-		*err = file_error(fh);
+		*err = file_error(fh, err_info);
 		if (*err == 0)
 			*err = WTAP_ERR_SHORT_READ;
 		return FALSE;
@@ -108,13 +114,13 @@ mpeg_read_rec_data(FILE_T fh, guchar *pd, int length, int *err)
 #define SCRHZ 27000000
 
 static gboolean 
-mpeg_read(wtap *wth, int *err, gchar **err_info _U_,
-		gint64 *data_offset)
+mpeg_read(wtap *wth, int *err, gchar **err_info, gint64 *data_offset)
 {
+	mpeg_t *mpeg = (mpeg_t *)wth->priv;
 	guint32 n;
 	int bytes_read = mpeg_read_header(wth, err, err_info, &n);
-	unsigned packet_size;
-	struct wtap_nstime ts = wth->capture.mpeg->now;
+	unsigned int packet_size;
+	struct wtap_nstime ts = mpeg->now;
 
 	if (bytes_read == -1)
 		return FALSE;
@@ -127,9 +133,9 @@ mpeg_read(wtap *wth, int *err, gchar **err_info _U_,
 		if (file_seek(wth->fh, 3, SEEK_CUR, err) == -1)
 			return FALSE;
 
-		bytes_read = file_read(&stream, 1, sizeof stream, wth->fh);
+		bytes_read = file_read(&stream, sizeof stream, wth->fh);
 		if (bytes_read != sizeof stream) {
-			*err = file_error(wth->fh);
+			*err = file_error(wth->fh, err_info);
 			return FALSE;
 		}
 
@@ -139,16 +145,16 @@ mpeg_read(wtap *wth, int *err, gchar **err_info _U_,
 			guint64 pack;
 			guint8 stuffing;
 
-			bytes_read = file_read(&pack1, 1, sizeof pack1, wth->fh);
+			bytes_read = file_read(&pack1, sizeof pack1, wth->fh);
 			if (bytes_read != sizeof pack1) {
-				*err = file_error(wth->fh);
+				*err = file_error(wth->fh, err_info);
 				if (*err == 0 && bytes_read != 0)
 					*err = WTAP_ERR_SHORT_READ;
 				return FALSE;
 			}
-			bytes_read = file_read(&pack0, 1, sizeof pack0, wth->fh);
+			bytes_read = file_read(&pack0, sizeof pack0, wth->fh);
 			if (bytes_read != sizeof pack0) {
-				*err = file_error(wth->fh);
+				*err = file_error(wth->fh, err_info);
 				if (*err == 0 && bytes_read != 0)
 					*err = WTAP_ERR_SHORT_READ;
 				return FALSE;
@@ -160,9 +166,9 @@ mpeg_read(wtap *wth, int *err, gchar **err_info _U_,
 					if (file_seek(wth->fh, 1, SEEK_CUR, err) == -1)
 						return FALSE;
 					bytes_read = file_read(&stuffing,
-							1, sizeof stuffing, wth->fh);
+							sizeof stuffing, wth->fh);
 					if (bytes_read != sizeof stuffing) {
-						*err = file_error(wth->fh);
+						*err = file_error(wth->fh, err_info);
 						return FALSE;
 					}
 					stuffing &= 0x07;
@@ -177,21 +183,21 @@ mpeg_read(wtap *wth, int *err, gchar **err_info _U_,
 						unsigned ext = (unsigned)((bytes >> 1) & 0x1ff);
 						guint64 cr = 300 * ts_val + ext;
 						unsigned rem = (unsigned)(cr % SCRHZ);
-						wth->capture.mpeg->now.secs
-							= wth->capture.mpeg->t0 + (time_t)(cr / SCRHZ);
-						wth->capture.mpeg->now.nsecs
+						mpeg->now.secs
+							= mpeg->t0 + (time_t)(cr / SCRHZ);
+						mpeg->now.nsecs
 							= (int)(G_GINT64_CONSTANT(1000000000) * rem / SCRHZ);
 					}
-					ts = wth->capture.mpeg->now;
+					ts = mpeg->now;
 					break;
 				default:
 					packet_size = 12;
 			}
 		} else {
 			guint16 length;
-			bytes_read = file_read(&length, 1, sizeof length, wth->fh);
+			bytes_read = file_read(&length, sizeof length, wth->fh);
 			if (bytes_read != sizeof length) {
-				*err = file_error(wth->fh);
+				*err = file_error(wth->fh, err_info);
 				if (*err == 0 && bytes_read != 0)
 					*err = WTAP_ERR_SHORT_READ;
 				return FALSE;
@@ -208,10 +214,10 @@ mpeg_read(wtap *wth, int *err, gchar **err_info _U_,
 		MPA_UNMARSHAL(&mpa, n);
 		if (MPA_VALID(&mpa)) {
 			packet_size = MPA_BYTES(&mpa);
-			wth->capture.mpeg->now.nsecs += MPA_DURATION_NS(&mpa);
-			if (wth->capture.mpeg->now.nsecs >= 1000000000) {
-				wth->capture.mpeg->now.secs++;
-				wth->capture.mpeg->now.nsecs -= 1000000000;
+			mpeg->now.nsecs += MPA_DURATION_NS(&mpa);
+			if (mpeg->now.nsecs >= 1000000000) {
+				mpeg->now.secs++;
+				mpeg->now.nsecs -= 1000000000;
 			}
 		} else {
 			packet_size = mpeg_resync(wth, err, err_info);
@@ -223,7 +229,7 @@ mpeg_read(wtap *wth, int *err, gchar **err_info _U_,
 
 	buffer_assure_space(wth->frame_buffer, packet_size);
 	if (!mpeg_read_rec_data(wth->fh, buffer_start_ptr(wth->frame_buffer),
-				packet_size, err))
+				packet_size, err, err_info))
 		return FALSE;
 	wth->data_offset += packet_size;
 	wth->phdr.ts = ts;
@@ -235,17 +241,11 @@ mpeg_read(wtap *wth, int *err, gchar **err_info _U_,
 static gboolean
 mpeg_seek_read(wtap *wth, gint64 seek_off,
 		union wtap_pseudo_header *pseudo_header _U_, guchar *pd, int length,
-		int *err, gchar **err_info _U_)
+		int *err, gchar **err_info)
 {
 	if (file_seek(wth->random_fh, seek_off, SEEK_SET, err) == -1)
 		return FALSE;
-	return mpeg_read_rec_data(wth->random_fh, pd, length, err);
-}
-
-static void
-mpeg_close(wtap *wth)
-{
-	g_free(wth->capture.mpeg);
+	return mpeg_read_rec_data(wth->random_fh, pd, length, err, err_info);
 }
 
 struct _mpeg_magic {
@@ -260,16 +260,17 @@ struct _mpeg_magic {
 };
 
 int 
-mpeg_open(wtap *wth, int *err, gchar **err_info _U_)
+mpeg_open(wtap *wth, int *err, gchar **err_info)
 {
 	int bytes_read;
 	char magic_buf[16];
 	struct _mpeg_magic* m;
+	mpeg_t *mpeg;
 	
 	errno = WTAP_ERR_CANT_READ;
-	bytes_read = file_read(magic_buf, 1, sizeof magic_buf, wth->fh);
+	bytes_read = file_read(magic_buf, sizeof magic_buf, wth->fh);
 	if (bytes_read != (int) sizeof magic_buf) {
-		*err = file_error(wth->fh);
+		*err = file_error(wth->fh, err_info);
 		if (*err != 0)
 			return -1;
 		return 0;
@@ -292,13 +293,13 @@ good_magic:
 	wth->tsprecision = WTAP_FILE_TSPREC_NSEC;
 	wth->subtype_read = mpeg_read;
 	wth->subtype_seek_read = mpeg_seek_read;
-	wth->subtype_close = mpeg_close;
 	wth->snapshot_length = 0;
 
-	wth->capture.mpeg = g_malloc(sizeof(mpeg_t));
-	wth->capture.mpeg->now.secs = time(NULL);
-	wth->capture.mpeg->now.nsecs = 0;
-	wth->capture.mpeg->t0 = wth->capture.mpeg->now.secs;
+	mpeg = (mpeg_t *)g_malloc(sizeof(mpeg_t));
+	wth->priv = (void *)mpeg;
+	mpeg->now.secs = time(NULL);
+	mpeg->now.nsecs = 0;
+	mpeg->t0 = mpeg->now.secs;
 
 	return 1;
 }
