@@ -1,6 +1,6 @@
 /* wtap.h
  *
- * $Id: wtap.h 28463 2009-05-24 22:49:36Z stig $
+ * $Id: wtap.h 37572 2011-06-06 16:39:23Z gerald $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -28,7 +28,6 @@
 #endif
 
 #include <glib.h>
-#include <stdio.h>
 #include <time.h>
 
 #ifdef __cplusplus
@@ -167,8 +166,8 @@ extern "C" {
 #define WTAP_ENCAP_NETTL_UNKNOWN                74
 #define WTAP_ENCAP_MTP2_WITH_PHDR               75
 #define WTAP_ENCAP_JUNIPER_PPPOE                76
-#define WTAP_GCOM_TIE1                          77
-#define WTAP_GCOM_SERIAL                        78
+#define WTAP_ENCAP_GCOM_TIE1                    77
+#define WTAP_ENCAP_GCOM_SERIAL                  78
 #define WTAP_ENCAP_NETTL_X25                    79
 #define WTAP_ENCAP_K12                          80
 #define WTAP_ENCAP_JUNIPER_MLPPP                81
@@ -209,6 +208,21 @@ extern "C" {
 #define WTAP_ENCAP_GSM_UM                       116
 #define WTAP_ENCAP_DPNSS                        117
 #define WTAP_ENCAP_PACKETLOGGER                 118
+#define WTAP_ENCAP_NSTRACE_1_0                  119
+#define WTAP_ENCAP_NSTRACE_2_0                  120
+#define WTAP_ENCAP_FIBRE_CHANNEL_FC2            121
+#define WTAP_ENCAP_FIBRE_CHANNEL_FC2_WITH_FRAME_DELIMS 122
+#define WTAP_ENCAP_JPEG_JFIF                    123
+#define WTAP_ENCAP_IPNET                        124
+#define WTAP_ENCAP_SOCKETCAN                    125
+#define WTAP_ENCAP_IEEE802_11_NETMON_RADIO      126
+#define WTAP_ENCAP_IEEE802_15_4_NOFCS           127
+#define WTAP_ENCAP_RAW_IPFIX                    128
+#define WTAP_ENCAP_RAW_IP4                      129
+#define WTAP_ENCAP_RAW_IP6                      130
+#define WTAP_ENCAP_LAPD                         131
+#define WTAP_ENCAP_DVBCI                        132
+#define WTAP_ENCAP_MUX27010                     133
 
 #define WTAP_NUM_ENCAP_TYPES                    wtap_get_num_encap_types()
 
@@ -248,7 +262,7 @@ extern "C" {
 #define WTAP_FILE_NGSNIFFER_COMPRESSED          30
 #define WTAP_FILE_NETXRAY_1_1                   31
 #define WTAP_FILE_NETXRAY_2_00x                 32
-#define WTAP_FILE_NETWORK_INSTRUMENTS_V9        33
+#define WTAP_FILE_NETWORK_INSTRUMENTS           33
 #define WTAP_FILE_LANALYZER                     34
 #define WTAP_FILE_PPPDUMP                       35
 #define WTAP_FILE_RADCOM                        36
@@ -272,6 +286,10 @@ extern "C" {
 #define WTAP_FILE_DCT3TRACE                     54
 #define WTAP_FILE_PACKETLOGGER                  55
 #define WTAP_FILE_DAINTREE_SNA                  56
+#define WTAP_FILE_NETSCALER_1_0                 57
+#define WTAP_FILE_NETSCALER_2_0                 58
+#define WTAP_FILE_JPEG_JFIF                     59
+#define WTAP_FILE_IPFIX                         60
 
 #define WTAP_NUM_FILE_TYPES                     wtap_get_num_file_types()
 
@@ -285,7 +303,8 @@ extern "C" {
 
 /*
  * Maximum packet size we'll support.
- * It must be at least 65535.
+ * 65535 is the largest snapshot length that libpcap supports, so we
+ * use that.
  */
 #define	WTAP_MAX_PACKET_SIZE			65535
 
@@ -305,6 +324,22 @@ extern "C" {
  * of that.
  */
 
+
+struct nstr_phdr {
+	gint64 rec_offset;
+	gint32 rec_len;
+	guint8 nicno_offset;
+	guint8 nicno_len;
+	guint8 dir_offset;
+	guint8 dir_len;
+	guint8 eth_offset;
+	guint8 pcb_offset;
+	guint8 l_pcb_offset;
+	guint8 rec_type;
+	guint8 vlantag_offset;
+	guint8 coreid_offset;
+};
+
 /* Packet "pseudo-header" information for Ethernet capture files. */
 struct eth_phdr {
 	gint	fcs_len;	/* Number of bytes of FCS - -1 means "unknown" */
@@ -331,6 +366,8 @@ struct isdn_phdr {
  * Status bits.
  */
 #define ATM_RAW_CELL	0x01	/* TRUE if the packet is a single cell */
+#define ATM_NO_HEC	0x02	/* TRUE if the cell has HEC stripped out */
+#define ATM_AAL2_NOPHDR	0x04	/* TRUE if the AAL2 PDU has no pseudo-header */
 
 /*
  * AAL types.
@@ -585,60 +622,6 @@ struct catapult_dct2000_phdr
 	struct wtap *wth;
 };
 
-/*
- * possible event type
- */
-#define URB_SUBMIT        'S'
-#define URB_COMPLETE      'C'
-#define URB_ERROR         'E'
-
-/*
- * possible transfer mode
- */
-#define URB_ISOCHRONOUS   0x0
-#define URB_INTERRUPT     0x1
-#define URB_CONTROL       0x2
-#define URB_BULK          0x3
-
-#define URB_TRANSFER_IN   0x80		/* to host */
-
-/*
- * USB setup header as defined in USB specification
- * See usb_20.pdf, Chapter 9.3 'USB Device Requests' for details.
- */
-struct usb_device_setup_hdr {
-    gint8 bmRequestType;
-    guint8 bRequest;
-    guint16 wValue;
-    guint16 wIndex;
-    guint16 wLength;
-};
-
-/*
- * Header prepended by Linux kernel to each USB event.
- * Always followed by a struct usb_device_setup_hdr, although that header
- * is valid only if setup_flag is 0; otherwise it's 8 bytes of junk.
- * (Setup flag is '-', 'D', 'Z', or 0.  Data flag is '<', '>', 'Z', or 0.)
- * See linux/Documentation/usb/usbmon.txt and libpcap/pcap/usb.h for details.
- *
- * We present this as a pseudo-header; the values are in host byte order.
- */
-struct linux_usb_phdr {
-    guint64 id;             /* urb id, to link submission and completion events */
-    guint8 event_type;      /* Submit ('S'), Completed ('C'), Error ('E') */
-    guint8 transfer_type;   /* ISO (0), Intr, Control, Bulk (3) */
-    guint8 endpoint_number; /* Endpoint number (0-15) and transfer direction */
-    guint8 device_address;  /* 0-127 */
-    guint16 bus_id;
-    gint8 setup_flag;       /* 0, if the urb setup header is present */
-    gint8 data_flag;        /* 0, if urb data is present */
-    gint64 ts_sec;
-    gint32 ts_usec;
-    gint32 status;
-    guint32 urb_len;        /* whole len of urb this event refers to */
-    guint32 data_len;       /* amount of urb data really present in this event */
-};
-
 #define LIBPCAP_BT_PHDR_SENT    0
 #define LIBPCAP_BT_PHDR_RECV    1
 
@@ -650,6 +633,16 @@ struct libpcap_bt_phdr {
     guint32 direction;     /* Bit 0 hold the frame direction. */
 };
 
+#define LIBPCAP_PPP_PHDR_RECV    0
+#define LIBPCAP_PPP_PHDR_SENT    1
+
+/*
+ * Header prepended by libpcap to each ppp frame.
+ */
+struct libpcap_ppp_phdr {
+    guint8 direction;
+};
+
 /*
  * Endace Record Format pseudo header
  */
@@ -787,7 +780,7 @@ struct gsm_um_phdr {
 #define GSM_UM_CHANNEL_RACH	6
 #define GSM_UM_CHANNEL_AGCH	7
 #define GSM_UM_CHANNEL_PCH	8
-  
+
 union wtap_pseudo_header {
 	struct eth_phdr		eth;
 	struct x25_phdr		x25;
@@ -799,17 +792,17 @@ union wtap_pseudo_header {
 	struct cosine_phdr	cosine;
 	struct irda_phdr	irda;
 	struct nettl_phdr	nettl;
-	struct mtp2_phdr        mtp2;
+	struct mtp2_phdr	mtp2;
 	struct k12_phdr		k12;
 	struct lapd_phdr	lapd;
 	struct catapult_dct2000_phdr dct2000;
-	struct linux_usb_phdr	linux_usb;
-	struct erf_mc_phdr      erf;
+	struct erf_mc_phdr	erf;
 	struct sita_phdr	sita;
 	struct bthci_phdr	bthci;
 	struct l1event_phdr	l1event;
 	struct i2c_phdr		i2c;
 	struct gsm_um_phdr	gsm_um;
+	struct nstr_phdr	nstr;
 };
 
 struct wtap_nstime {
@@ -824,21 +817,22 @@ struct wtap_pkthdr {
 	int pkt_encap;
 };
 
-struct wtap;
 struct Buffer;
 struct wtap_dumper;
 
 typedef struct wtap wtap;
 typedef struct wtap_dumper wtap_dumper;
 
+typedef struct wtap_reader *FILE_T;
+
 struct file_type_info {
     /* the file type name */
     /* should be NULL for all "pseudo" types that are only internally used and not read/writeable */
-	const char *name;
+    const char *name;
 
     /* the file type short name, used as a shortcut for the command line tools */
     /* should be NULL for all "pseudo" types that are only internally used and not read/writeable */
-	const char *short_name;
+    const char *short_name;
 
     /* the common file extensions for this type (seperated by semicolon) */
     /* should be *.* if no common extension is applicable */
@@ -848,32 +842,57 @@ struct file_type_info {
     /* should be NULL if no default extension is known */
     const char *file_extension_default;
 
-    /* can this type be compressed with gzip? */
-	gboolean	can_compress;
+    /* when writing this file format, is seeking required? */
+    gboolean writing_must_seek;
+
+    /* does this type support name resolution records? */
+    /* should be FALSE is this file type doesn't support name resolution records */
+    gboolean has_name_resolution;
 
     /* can this type write this encapsulation format? */
-    /* should be NULL is this file type don't have write support */
-	int	(*can_write_encap)(int);
+    /* should be NULL is this file type doesn't have write support */
+    int (*can_write_encap)(int);
 
     /* the function to open the capture file for writing */
     /* should be NULL is this file type don't have write support */
-	int	(*dump_open)(wtap_dumper *, gboolean, int *);
+    int (*dump_open)(wtap_dumper *, int *);
 };
 
 
 typedef int (*wtap_open_routine_t)(struct wtap*, int *, char **);
 
 
-/*
- * On failure, "wtap_open_offline()" returns NULL, and puts into the
+/** On failure, "wtap_open_offline()" returns NULL, and puts into the
  * "int" pointed to by its second argument:
  *
- * a positive "errno" value if the capture file can't be opened;
+ * @param filename Name of the file to open
+ * @param err a positive "errno" value if the capture file can't be opened;
  * a negative number, indicating the type of error, on other failures.
+ * @param err_info for some errors, a string giving more details of
+ * the error
+ * @param do_random TRUE if random access to the file will be done,
+ * FALSE if not
  */
 struct wtap* wtap_open_offline(const char *filename, int *err,
     gchar **err_info, gboolean do_random);
 
+/*
+ * If we were compiled with zlib and we're at EOF, unset EOF so that
+ * wtap_read/gzread has a chance to succeed. This is necessary if
+ * we're tailing a file.
+ */
+void wtap_cleareof(wtap *wth);
+
+/*
+ * Set callback functions to add new hostnames. Currently pcapng-only.
+ * MUST match add_ipv4_name and add_ipv6_name in addr_resolv.c.
+ */
+typedef void (*wtap_new_ipv4_callback_t) (const guint addr, const gchar *name);
+void wtap_set_cb_new_ipv4(wtap *wth, wtap_new_ipv4_callback_t add_new_ipv4);
+
+typedef void (*wtap_new_ipv6_callback_t) (const void *addrp, const gchar *name);
+void wtap_set_cb_new_ipv6(wtap *wth, wtap_new_ipv6_callback_t add_new_ipv6);
+
 /* Returns TRUE if read was successful. FALSE if failure. data_offset is
  * set to the offset in the file where the data for the read packet is
  * located. */
@@ -893,9 +912,9 @@ guint8 *wtap_buf_ptr(wtap *wth);
 
 /* Return an approximation of the amount of data we've read sequentially
  * from the file so far. */
-gint64 wtap_read_so_far(wtap *wth, int *err);
+gint64 wtap_read_so_far(wtap *wth);
 gint64 wtap_file_size(wtap *wth, int *err);
-int wtap_snapshot_length(wtap *wth); /* per file */
+guint wtap_snapshot_length(wtap *wth); /* per file */
 int wtap_file_type(wtap *wth);
 int wtap_file_encap(wtap *wth);
 int wtap_file_tsprecision(wtap *wth);
@@ -917,6 +936,8 @@ gboolean wtap_dump(wtap_dumper *, const struct wtap_pkthdr *,
 void wtap_dump_flush(wtap_dumper *);
 gint64 wtap_get_bytes_dumped(wtap_dumper *);
 void wtap_set_bytes_dumped(wtap_dumper *wdh, gint64 bytes_dumped);
+struct addrinfo;
+gboolean wtap_dump_set_addrinfo_list(wtap_dumper *wdh, struct addrinfo *addrinfo_list);
 gboolean wtap_dump_close(wtap_dumper *, int *);
 
 /*** various string converter functions ***/
@@ -984,19 +1005,14 @@ int wtap_register_encap_type(char* name, char* short_name);
 	/* LZ77 compressed data has bad offset to string */
 #define	WTAP_ERR_RANDOM_OPEN_STDIN		-18
 	/* We're trying to open the standard input for random access */
-#define WTAP_ERR_COMPRESSION_NOT_SUPPORTED -19
+#define WTAP_ERR_COMPRESSION_NOT_SUPPORTED	-19
 	/* The filetype doesn't support output compression */
-
-/* Errors from zlib; zlib error Z_xxx turns into Wiretap error
-   WTAP_ERR_ZLIB + Z_xxx.
-
-   WTAP_ERR_ZLIB_MIN and WTAP_ERR_ZLIB_MAX bound the range of zlib
-   errors; we leave room for 100 positive and 100 negative error
-   codes. */
-
-#define	WTAP_ERR_ZLIB				-200
-#define	WTAP_ERR_ZLIB_MAX			-100
-#define	WTAP_ERR_ZLIB_MIN			-300
+#define	WTAP_ERR_CANT_SEEK			-20
+	/* An attempt to seek failed, reason unknown */
+#define WTAP_ERR_DECOMPRESS			-21
+	/* Error decompressing */
+#define WTAP_ERR_INTERNAL			-22
+	/* "Shouldn't happen" internal errors */
 
 #ifdef __cplusplus
 }
